// **************************************************************************************
// el Comandante                                                    guerilla.bplaced.net
// --------------------------------------------------------------------------------------
// Information:
// Controlling a 30 years old RC-Car toy with 2 motors remotly using the wireless ESP-NOW protocol.
// This software is designed for the ESP32 microcontroller unit and the DRV8833 motor driver.
// The complete arduino software bundle contains two parts and the manual:
// - The software for the WT32-SC01 ESP32 Development Board as the ESP-NOW Transmitter.
// - The software for the LILYGO TTGO-T-Display ESP32 Development Board as the ESP-NOW Receiver.
// This is the arduino software for the LILYGO TTGO-T-Display device (Receiver).
// Option to control the machine directly without ESP-NOW, using the build in buttons of the LILYGO TTGO T ESP32 device.
// DRV8833 motor driver: No ENABLE pin available, the PWM signal must be connected to the correct IN pin of the DRV8833.
// --------------------------------------------------------------------------------------
// Arduino:
// Tools --> Board --> ESP32 Dev Module
// --------------------------------------------------------------------------------------
// Used Libraries:
// TFT_eSPI --> https://github.com/Bodmer/TFT_eSPI
// --------------------------------------------------------------------------------------
// LILYGO TTGO T esp32 device:
// ESP32-WROVER-B, 520 KB SRAM, 4 MB or 16 MB SPI Flash
// Dual-Core XtensaRO 32-Bit LX6 MCU, 240 Mhz, CH9102F chipset
// 1.14" 240 x 135 IPS TFT (ST7789V)
// USB-C, WiFi 2.4 ~ 2.5 GHz WLAN 802.11 b/g/n and Bluetooth 4.2 BR/EDR- and BLE-Standards
// 5V DC / up to 1A, -40℃ ~ 85℃
//
// Goodies: 2 Buttons, Reset Button, Battery connector (3.7V LI-IO, JST 2 Pin 1.25mm), Battery charging and discharging
//          5V and 3.7V power supply possible, IPS Display
// Warning: None known
// Issues : None known
//
// LILYGO TTGO T Pins:
// TFT_MISO   = N/A
// TFT_MOSI   = GPIO 19
// TFT_SCLK   = GPIO 18
// TFT_CS     = GPIO 5
// TFT_DC     = GPIO 16
// TFT_RST    = GPIO 23
// TFT_BL     = GPIO 4
// I2C_SDA    = GPIO 21
// I2C_SCL    = GPIO 22
// ADC_IN     = GPIO 34
// BUTTON 1   = GPIO 35
// BUTTON 2   = GPIO 0
// ADC Power  = GPIO 14
//
// --------------------------------------------------------------------------------------
// Interface, Top View:
//                                    TTGO
//                            GND o           o 3V3
//                            GND o     S     o SVP   GPIO36  ADC0
//                  Wire_SDA  21  o     T     o  37   GPIO37
//                  Wire_SCL  22  o     7     o  38   GPIO38
//                            17  o     7     o SVN   GPIO39  ADC3
//    Touch2   ADC12  GPIO2   2   o     8     o  32   GPIO32  ADC4    Touch9
//    Touch3   ADC13  GPIO15  15  o     9     o  33   GPIO33  ADC5    Touch8
//    Touch4   ADC14  GPIO13  13  o     V     o  25   GPIO25  ADC18   DAC1
//    Touch5   ADC15  GPIO12  12  o           o  26   GPIO26  ADC19   DAC2
//                            GND o           o  27   GPIO27  ADC17   Touch7
//                            GND o           o GND
//                            3V3 o           o  5V
//                                            * RST Button
//                                    USB-C
//                     Button GPIO0 * ||||| * Button GPIO35
//
// --------------------------------------------------------------------------------------
// ESP32 - Restricted Pins:
// GPIO0, GPIO2, GPIO4, GPIO5, GPIO12 and GPIO15 are strapping pins that bring the ESP32 into bootloader or flash mode
// while booting. Connecting something to it during boot which pull the pins into the HIGH or LOW state can lead to
// unexpected behavior.
// The ADC2_CHx pins cannot be used as analog inputs when WiFi is active.
// GPIO34-39 are inputs only, no internal pull-up or pull-down resistor available on these pins.
//
// ESP32 - Input Pins:
// Using GPIOs as an input must be enabled with INPUT_PULLDOWN when using the ESP32, i.e: pinMode(pin, INPUT_PULLDOWN)
// Choosing just INPUT, the pin level is undefined.
//
// ESP32 - DEFAULT SPI interfaces, HSPI and VSPI, GPIO pins: 
// SPI-Interface    MOSI      MISO      CLK       CS
//     HSPI        GPIO 13   GPIO 12   GPIO 14   GPIO 15
//     VSPI        GPIO 23   GPIO 19   GPIO 18   GPIO 5
//
// ESP32 - DEFAULT I2C interface GPIO pins:
// I2C-interface  I2C_SDA   I2C_SCL
//                GPIO 21   GPIO 22
//
// --------------------------------------------------------------------------------------
// Documentation and Libraries:
// Overview ESP32 mcu's   - https://socialcompare.com/en/comparison/esp8266-vs-esp32-vs-esp32-s2
// Espressif ESP32 docu's - https://www.espressif.com/en/support/documents/technical-documents
// Device LILYGO-TTGO     - https://github.com/Xinyuan-LilyGO/TTGO-T-Display
// TFT Device ST7789V     - https://newhavendisplay.com/content/datasheets/ST7789V.pdf
// TFT_eSPI library       - https://github.com/Bodmer/TFT_eSPI
// ST7789V library        - included in the TFT_eSPI library
// **************************************************************************************

#include <Arduino.h>                                //
#include <Wire.h>                                   //
#include <esp_now.h>                                //
#include <WiFi.h>                                   //
#include <math.h>                                   //
#include <TFT_eSPI.h>                               // TFT_espi Library
TFT_eSPI tft                = TFT_eSPI();
TFT_eSprite DisplayShow     = TFT_eSprite(&tft);    // Sprite Motor Show
TFT_eSprite DisplayData     = TFT_eSprite(&tft);    // Sprite Motor Data
TFT_eSprite ArrowWhiteLeft  = TFT_eSprite(&tft);    // Sprite Arrow White Left
TFT_eSprite ArrowWhiteRight = TFT_eSprite(&tft);    // Sprite Arrow White Right
TFT_eSprite SpriteStopOff   = TFT_eSprite(&tft);    // Sprite Stop OFF
TFT_eSprite SpriteStopCoast = TFT_eSprite(&tft);    // Sprite Stop Coast
TFT_eSprite SpriteStopBrake = TFT_eSprite(&tft);    // Sprite Stop Brake
TFT_eSprite SpriteForward   = TFT_eSprite(&tft);    // Sprite Forward
TFT_eSprite SpriteReverse   = TFT_eSprite(&tft);    // Sprite Reverse

// GPIO - IO ports definitions
#define   ButtonLeft     0                          // GPIO button left / top
#define   ButtonRight    35                         // GPIO button right / bottom
// Back View - The two H-bridges in the DRV8833 are connected in parallel for double the current of a single H-bridge (~ 3A)
#define   MotorLeftOut1   13                        // motor left  in1/in3 (direction and pwm) - first  DRV8833 breakout modul
#define   MotorLeftOut2   27                        // motor left  in2/in4 (direction and pwm)
#define   MotorRightOut1  33                        // motor right in1/in3 (direction and pwm) - second DRV8833 breakout modul
#define   MotorRightOut2  32                        // motor right in2/in4 (direction and pwm)

// Variables
uint32_t  update                          = 0;
int32_t   x                               = 0;
int32_t   y                               = 0;
int32_t   z                               = 0;
int32_t   temp                            = 0;
int       menu                            = 0;      // 0=Startup, 1=Button control, 2=ESP-NOW control, 3=ESP-NOW Live, 4=ESP-NOW Auto, 5=ESP-NOW Fun
int32_t  width                            = 135;
int32_t  height                           = 240;

// Strings - variables
String    StringTemp                      = "";     // for temporary string
String    StringSlash                     = "/";

// Sprites
uint16_t  ArrowWhiteLeftWidth             = 32;
uint16_t  ArrowWhiteLeftHeight            = 24;
uint16_t  ArrowWhiteRightWidth            = 32;
uint16_t  ArrowWhiteRightHeight           = 24;
uint16_t  SpriteStopWidth                 = 48;
uint16_t  SpriteStopHeight                = 48;
uint16_t  SpriteForwardWidth              = 48;
uint16_t  SpriteForwardHeight             = 48;
uint16_t  SpriteReverseWidth              = 48;
uint16_t  SpriteReverseHeight             = 48;

// Display - variables
int       Display                         = 0;      // 0 = startup display, 1 = sprite motor display show, 2 = sprite motor display data
int       DisplayDataUpdateSide           = 0;      // 0 = update left and right motor data, 1 = update left motor data, 2 = update right motor data

// Display - Show
uint16_t  DisplayShowSpriteXpos           = 0;
uint16_t  DisplayShowSpriteYpos           = 0;

uint16_t  DisplayShowModeXpos             = 80;
uint16_t  DisplayShowModeYpos             = 6;
uint16_t  DisplayShowModeWidth            = 80;
uint16_t  DisplayShowModeHeight           = 16;
uint16_t  DisplayShowModeXcenter          = 120;
uint16_t  DisplayShowModeYcenter          = 14;
char      DisplayShowModeText[6][7]       = {"      ", " Keys ", " Wait ", " Live ", " Auto ", " Fun  "};

uint16_t  DisplayShowLeftArrowXpos        = 8;
uint16_t  DisplayShowLeftArrowYpos        = 4;

uint16_t  DisplayShowRightArrowXpos       = 200;
uint16_t  DisplayShowRightArrowYpos       = 4;

uint16_t  DisplayShowLeftDIRXpos          = 16;
uint16_t  DisplayShowLeftDIRYpos          = 42;

uint16_t  DisplayShowRightDIRXpos         = 176;
uint16_t  DisplayShowRightDIRYpos         = 42;

//                                        = LeftPWMValue, RightPWMValue, PWMText, LeftSteps, RightSteps
uint16_t  DisplayShowXpos[5]              = {0, 160, 80, 69, 87};
uint16_t  DisplayShowYpos[5]              = {98, 98, 98, 43, 71};
uint16_t  DisplayShowWidth[5]             = {80, 80, 80, 86, 86};
uint16_t  DisplayShowHeight[5]            = {32, 32, 32, 18, 18};
uint16_t  DisplayShowXcenter[5]           = {40, 200, 120, 112, 130};
uint16_t  DisplayShowYcenter[5]           = {114, 114, 114, 52, 80};

// Display - Data
uint16_t  DisplayDataSpriteXpos           = 0;
uint16_t  DisplayDataSpriteYpos           = 0;

char      DisplayDataCTR[3][4]            = {"OFF", "FIX", "PWM"};
char      DisplayDataDIR[5][6]            = {" OFF ", " For ", "Coast", " Rev ", "Brake"};

float     DisplayDataFloatTemp            = 0.0;
float     DisplayDataMilliseconds         = 1000.0;

uint16_t  DisplayDataModeXpos             = 80;
uint16_t  DisplayDataModeYpos             = 0;
uint16_t  DisplayDataModeWidth            = 80;
uint16_t  DisplayDataModeHeight           = 24;
uint16_t  DisplayDataModeXcenter          = 120;
uint16_t  DisplayDataModeYcenter          = 12;

//                                        = LeftSteps, LeftTime, LeftCTR, LeftPWM, LeftDIR
uint16_t  DisplayDataLeftXpos[5]          = {0, 0, 0, 0, 0};
uint16_t  DisplayDataLeftYpos[5]          = {30, 51, 72, 93, 114};
uint16_t  DisplayDataLeftWidth[5]         = {88, 88, 88, 88, 88};
uint16_t  DisplayDataLeftHeight[5]        = {20, 20, 20, 20, 20};
uint16_t  DisplayDataLeftXcenter[5]       = {44, 44, 44, 44, 44};
uint16_t  DisplayDataLeftYcenter[5]       = {40, 61, 82, 103, 124};

//                                        = RightSteps, RightTime, RightCTR, RightPWM, RightDIR
uint16_t  DisplayDataRightXpos[5]         = {152, 152, 152, 152, 152};
uint16_t  DisplayDataRightYpos[5]         = {30, 51, 72, 93, 114};
uint16_t  DisplayDataRightWidth[5]        = {88, 88, 88, 88, 88};
uint16_t  DisplayDataRightHeight[5]       = {20, 20, 20, 20, 20};
uint16_t  DisplayDataRightXcenter[5]      = {196, 196, 196, 196, 196};
uint16_t  DisplayDataRightYcenter[5]      = {40, 61, 82, 103, 124};

// Key Control - variables
long      timerButton                     = 0;
long      longPressTime                   = 250;
bool      ButtonLeftActive                = false;
bool      ButtonLeftActiveLong            = false;
bool      ButtonRightActive               = false;
bool      ButtonRightActiveLong           = false;
char      MotorDataDIR[5][6]              = {" OFF ", " For ", "STOPC", "STOPB", " Rev "};

// Motor Control - variables
uint32_t  TimeBase                        = 0;      // time base for the motors step time, millis() right before steps start
uint32_t  LeftStepTime                    = 0;      // left motor step time
uint32_t  RightStepTime                   = 0;      // right motor step time
int       LeftStepUpdate                  = 0;      // 0 = no update, new left motor vaiables are the old left motor variables
int       RightStepUpdate                 = 0;      // 0 = no update, new right motor vaiables are the old right motor variables
int       AutoModeLeftDone                = 0;      // auto mode left done
int       AutoModeRightDone               = 0;      // auto mode right done
int       LeftStepCounter                 = 0;      // counter proceeded left motor step
int       RightStepCounter                = 0;      // counter proceeded right motor step

// espNOW - variables
const int espNowMsgDataArrayMax           = 999;    // max. motor data arrays, index 0 is the first array
int       espNowMsgDataArrayIndex         = 0;      // max. send messages = espNowMsgDataArrayMax + 1
int       espNowMsgDataArrayToReceive     = 0;      // messages send
bool      espNowMessageNew                = false;  // new incoming message(s 
bool      espNowMessageDone               = false;  // only bool variable allowed, wifi is a high task interrupt!

// espNOW - receiver data structure, must match the transmitter data structure, 32 bit aligned!
typedef struct struct_MotorData
{
  uint16_t  LeftMode = 0;           // 3 = Live, 4 = Auto, 5 = Fun - only used in array index 0
  uint16_t  LeftSteps = 0;          // Motor left, total steps - only used in array index 0
  uint32_t  LeftTime = 0;           // Time left motor action starts, in ms
  byte      LeftDIR = 0;            // 0 = OFF, 1 = FORWARD, 2 = STOP COAST, 3 = REVERSE, 4 = STOP BRAKE
  byte      LeftCTR = 1;            // 0 = OFF, 1 = FIX, 2 = PWM
  uint16_t  LeftPWM = 0;            // PWM duty cycle
  uint16_t  RightMode = 0;          // 3 = Live, 4 = Auto, 5 = Fun - only used in array index 0
  uint16_t  RightSteps = 0;         // Motor right, total steps - only used in array index 0
  uint32_t  RightTime = 0;          // Time right motor action starts, in ms
  byte      RightDIR = 0;           // 0 = OFF, 1 = FORWARD, 2 = STOP COAST, 3 = REVERSE, 4 = STOP BRAKE
  byte      RightCTR = 1;           // 0 = OFF, 1 = FIX, 2 = PWM
  uint16_t  RightPWM = 0;           // PWM duty cycle
}
struct_MotorData;
struct_MotorData MotorData[espNowMsgDataArrayMax + 1];

// espNOW - OLD receiver data structure, only update display if new values, 32 bit aligned!
typedef struct struct_MotorDataOld
{
  uint16_t  LeftMode = 0xFFFF;      // 3 = Live, 4 = Auto, 5 = Fun - only used in array index 0
  uint16_t  LeftSteps = 0xFFFF;     // Motor left, total steps - only used in array index 0
  uint32_t  LeftTime = 0xFFFFFFFF;  // Time left motor action starts, in ms
  byte      LeftDIR = 0xFF;         // 0 = OFF, 1 = FORWARD, 2 = STOP COAST, 3 = REVERSE, 4 = STOP BRAKE
  byte      LeftCTR = 0xFF;         // 0 = OFF, 1 = FIX, 2 = PWM
  uint16_t  LeftPWM = 0xFFFF;       // PWM duty cycle
  uint16_t  RightMode = 0xFFFF;     // 3 = Live, 4 = Auto, 5 = Fun - only used in array index 0
  uint16_t  RightSteps = 0xFFFF;    // Motor right, total steps - only used in array index 0
  uint32_t  RightTime = 0xFFFFFFFF; // Time right motor action starts, in ms
  byte      RightDIR = 0xFF;        // 0 = OFF, 1 = FORWARD, 2 = STOP COAST, 3 = REVERSE, 4 = STOP BRAKE
  byte      RightCTR = 0xFF;        // 0 = OFF, 1 = FIX, 2 = PWM
  uint16_t  RightPWM = 0xFFFF;      // PWM duty cycle
}
struct_MotorDataOld;
struct_MotorDataOld MotorDataOld;


// **************************************************************************************
// espNOW - callback function
void espNowDataReceive(const uint8_t * mac, const uint8_t *incomingData, int len)
{

  if (len == sizeof(MotorData[0]))
  {
    if (espNowMsgDataArrayIndex == 0)
    {
      espNowMessageDone = false;
      espNowMessageNew = false;
    
      memcpy(&MotorDataOld.LeftMode, &MotorData[LeftStepCounter].LeftMode, sizeof(MotorDataOld) / 2);
      memcpy(&MotorDataOld.RightMode, &MotorData[RightStepCounter].RightMode, sizeof(MotorDataOld) / 2);

      if (Display == 0)
      {
      Display = 1;
        DisplayShow.pushSprite(DisplayShowSpriteXpos, DisplayShowSpriteYpos);
        MotorPWMLeftRightStopOff();
      }
      else if (Display == 1 && MotorData[0].LeftMode == 4)
      {
        DisplayShow.pushSprite(0, 30, 0, 30, 240, height - 30);
        MotorPWMLeftRightStopOff();
      }
      else if (Display == 2 && MotorData[0].LeftMode == 4)
      {
        DisplayData.pushSprite(0, 30, 0, 30, 240, height - 30);
        MotorPWMLeftRightStopOff();
      }
    }

    tft.setTextSize(2);
    tft.setTextDatum(MC_DATUM);

    memcpy(&MotorData[espNowMsgDataArrayIndex], incomingData, sizeof(MotorData[espNowMsgDataArrayIndex]));
  
    if (Display == 1 && espNowMsgDataArrayIndex <= MotorData[0].LeftSteps)
    {
      tft.setTextColor(0x0500, 0x0000);
      if (espNowMsgDataArrayIndex == 0)
      {
        tft.drawNumber(espNowMsgDataArrayIndex, DisplayShowXcenter[3] - 24, DisplayShowYcenter[3], 1);
        tft.drawString(StringSlash, DisplayShowXcenter[3], DisplayShowYcenter[3], 1);
        tft.drawNumber(MotorData[0].LeftSteps, DisplayShowXcenter[3] + 24, DisplayShowYcenter[3], 1);
      }
      else
      {
        tft.drawNumber(espNowMsgDataArrayIndex, DisplayShowXcenter[3] - 24, DisplayShowYcenter[3], 1);
      }
    }
    if (Display == 1 && espNowMsgDataArrayIndex <= MotorData[0].RightSteps)
    {
      tft.setTextColor(0x035C, 0x0000);
      if (espNowMsgDataArrayIndex == 0)
      {
        tft.drawNumber(espNowMsgDataArrayIndex, DisplayShowXcenter[4] - 24, DisplayShowYcenter[4], 1);
        tft.drawString(StringSlash, DisplayShowXcenter[4], DisplayShowYcenter[4], 1);
        tft.drawNumber(MotorData[0].RightSteps, DisplayShowXcenter[4] + 24, DisplayShowYcenter[4], 1);
      }
      else
      {
        tft.drawNumber(espNowMsgDataArrayIndex, DisplayShowXcenter[4] - 24, DisplayShowYcenter[4], 1);
      }
    }

    if (Display == 2 && espNowMsgDataArrayIndex <= MotorData[0].LeftSteps)
    {
      tft.setTextColor(0x0500, 0x0000);
      if (espNowMsgDataArrayIndex == 0)
      {
        tft.drawNumber(espNowMsgDataArrayIndex, DisplayDataLeftXcenter[0] - 24, DisplayDataLeftYcenter[0], 1);
        tft.drawString(StringSlash, DisplayDataLeftXcenter[0], DisplayDataLeftYcenter[0], 1);
        tft.drawNumber(MotorData[0].LeftSteps, DisplayDataLeftXcenter[0] + 24, DisplayDataLeftYcenter[0], 1);
      }
      else
      {
        tft.drawNumber(espNowMsgDataArrayIndex, DisplayDataLeftXcenter[0] - 24, DisplayDataLeftYcenter[0], 1);
      }
    }
    if (Display == 2 && espNowMsgDataArrayIndex <= MotorData[0].RightSteps)
    {
      tft.setTextColor(0x035C, 0x0000);
      if (espNowMsgDataArrayIndex == 0)
      {
        tft.drawNumber(espNowMsgDataArrayIndex, DisplayDataRightXcenter[0] - 24, DisplayDataRightYcenter[0], 1);
        tft.drawString(StringSlash, DisplayDataRightXcenter[0], DisplayDataRightYcenter[0], 1);
        tft.drawNumber(MotorData[0].RightSteps, DisplayDataRightXcenter[0] + 24, DisplayDataRightYcenter[0], 1);
      }
      else
      {
        tft.drawNumber(espNowMsgDataArrayIndex, DisplayDataRightXcenter[0] - 24, DisplayDataRightYcenter[0], 1);
      }
    }

    espNowMsgDataArrayIndex++;

    if (MotorData[0].LeftMode == 3 && espNowMsgDataArrayIndex > MotorData[0].LeftSteps && espNowMsgDataArrayIndex > MotorData[0].RightSteps)
    {
      menu = 3;
      espNowDataReceiveDone();
    }
  
    if (MotorData[0].LeftMode == 4 && espNowMsgDataArrayIndex > MotorData[0].LeftSteps && espNowMsgDataArrayIndex > MotorData[0].RightSteps)
    {
      menu = 4;
      espNowDataReceiveDone();
    }

    if (MotorData[0].LeftMode == 5 && espNowMsgDataArrayIndex > MotorData[0].LeftSteps && espNowMsgDataArrayIndex > MotorData[0].RightSteps)
    {
      menu = 5;
      espNowDataReceiveDone();
    }
  }
}

void espNowDataReceiveDone()
{
  AutoModeLeftDone = 0;
  AutoModeRightDone = 0;
  LeftStepCounter = 0;
  RightStepCounter = 0;
  if (memcmp(&MotorData[0].LeftMode, &MotorDataOld.LeftMode, sizeof(MotorDataOld)) != 0)
  {
  memset (&MotorDataOld.LeftMode, 0xFF, sizeof(MotorDataOld));
  DisplayDataUpdateSide = 0;
  DisplayUpdate();
  TimeBase = millis();
  LeftStepTime = TimeBase + MotorData[0].LeftTime;    // update time in milliseconds
  RightStepTime = TimeBase + MotorData[0].RightTime;  // update time in milliseconds
  }
  espNowMsgDataArrayIndex = 0;
  espNowMessageNew = true;
  espNowMessageDone = true;
}


// **************************************************************************************
// Button Control Left FIX functions table
void (*ButtonControlLeftFIXFunctions[])(void) =
{
  MotorFIXLeftStopCoast,
  MotorFIXLeftForward,
  MotorFIXLeftStopCoast,
  MotorFIXLeftReverse,
  MotorFIXLeftStopBrake
};
// Button Control Left PWM functions table
void (*ButtonControlLeftPWMFunctions[])(void) =
{
  MotorPWMLeftStopCoast,
  MotorPWMLeftForward,
  MotorPWMLeftStopCoast,
  MotorPWMLeftReverse,
  MotorPWMLeftStopBrake
};


// **************************************************************************************
// Button Control Right FIX functions table
void (*ButtonControlRightFIXFunctions[])(void) =
{
  MotorFIXRightStopCoast,
  MotorFIXRightForward,
  MotorFIXRightStopCoast,
  MotorFIXRightReverse,
  MotorFIXRightStopBrake
};
// Button Control Right PWM functions table
void (*ButtonControlRightPWMFunctions[])(void) =
{
  MotorPWMRightStopCoast,
  MotorPWMRightForward,
  MotorPWMRightStopCoast,
  MotorPWMRightReverse,
  MotorPWMRightStopBrake
};


// **************************************************************************************
void setup(void)
{
  // GPIO - init
  pinMode(TFT_BL, OUTPUT);
  pinMode(ButtonLeft, INPUT_PULLDOWN);
  pinMode(ButtonRight, INPUT_PULLDOWN);
  pinMode(MotorLeftOut1, OUTPUT);
  pinMode(MotorLeftOut2, OUTPUT);
  pinMode(MotorRightOut1, OUTPUT);
  pinMode(MotorRightOut2, OUTPUT);

  // Motor - stop all
  digitalWrite(MotorLeftOut1, LOW);  // STOP
  digitalWrite(MotorLeftOut2, LOW);  // STOP
  digitalWrite(MotorRightOut1, LOW); // STOP
  digitalWrite(MotorRightOut2, LOW); // STOP

  // configure PWM (pwmChannel, frequence, resolution) - test the best frequency for the motors (~25 - ~1000)
  ledcSetup(0, 500, 8);             // motor left FORWARD
  ledcSetup(1, 500, 8);             // motor left REVERSE
  ledcSetup(2, 500, 8);             // motor right FORWARD
  ledcSetup(3, 500, 8);             // motor right REVERSE

  // TFT_eSPI - init
  tft.init();

  // switch backlight off
  digitalWrite(TFT_BL, 0);

  // Screen - basics
  width = 240;
  height = 135;

  // Screen - build
  tft.setRotation(3);
  tft.fillScreen(0x0000);
  tft.setPivot(width / 2, height / 2);
  tft.setTextColor(0xFFFE, 0x0000);
  tft.setTextFont(2);
  tft.setTextSize(2);
  tft.setTextDatum(MC_DATUM);

  // Sprites - create
  ArrowWhiteLeftCreate();
  ArrowWhiteRightCreate();
  DisplayShowCreate();
  SpriteStopOffCreate();
  SpriteStopCoastCreate();
  SpriteStopBrakeCreate();
  SpriteForwardCreate();
  SpriteReverseCreate();
  DisplayDataCreate();

  // ESP-NOW - Set device as a Wi-Fi Station
  WiFi.mode(WIFI_STA);
  WiFi.setSleep(false);                                 // no sleep for the robot
  //esp_wifi_set_ps(WIFI_PS_NONE);

  // Menu - create
  tft.fillTriangle(0, 12, 24, 0, 24, 24, 0xFAE1);       // arrow left ORANGE
  tft.setTextSize(2);
  tft.setCursor(34, 4, 1);
  tft.print("Button Control");
  tft.setTextSize(2);
  tft.setCursor(16, 41, 1);
  tft.print(WiFi.macAddress());
  tft.fillTriangle(0, 122, 24, 110, 24, 134, 0xFAE1);   // arrow left ORANGE
  tft.setTextSize(2);
  tft.setCursor(34, 114, 1);
  tft.print("espNOW Control");

  // ESP-NOW - display init status, on success register for recv CB to get recv packer info
  tft.setTextSize(2);
  tft.setCursor(34, 77, 1);
  tft.print("WiFi");
  
  if (esp_now_init() != ESP_OK)
  {
    tft.fillCircle(14, 84, 8, 0xF800);
    tft.drawCircle(14, 84, 8, 0x9492);
  }
  else
  {
    tft.fillCircle(14, 84, 8, 0x0700);
    tft.drawCircle(14, 84, 8, 0x9492);
    espNowMessageDone = false;
    espNowMsgDataArrayIndex = 0;
    esp_now_register_recv_cb(espNowDataReceive);
  }

  // Switch backlight on
  digitalWrite(TFT_BL, 1);

  // Menu - main, automaticly leaving menu on ESP-NOW connection
  while (menu == 0 && espNowMessageDone == false)
  {
    if (Display == 0 && update <= millis())
    {
      update = millis() + 250;
      tft.setTextColor(random(0xFFFF), 0x0000);
      tft.setCursor(110, 77, 1);
      tft.print("Waiting...");
    }
    if (digitalRead(ButtonLeft) == LOW)
    {
      esp_now_unregister_recv_cb();
      esp_now_deinit();
      WiFi.disconnect();
      WiFi.mode(WIFI_OFF);
      delay (200);
      menu = 1;
    }
    if (digitalRead(ButtonRight) == LOW)
    {
      tft.fillScreen(0x0000);
      tft.print("Wait");
      menu = 2;
    }
  }
  
  // Init screen default text colors
  tft.setTextColor(0xFFFE, 0x0000);

  // Init screen if Button Control or ESP-NOW Control selected
  if (menu == 1 || menu == 2)
  {
    Display = 1;
    DisplayShow.pushSprite(DisplayShowSpriteXpos, DisplayShowSpriteYpos);
    SpriteStopOff.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
    SpriteStopOff.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
    DisplayUpdate();
  }
}


// **************************************************************************************
void loop()
{
  // Main code
  switch (menu)
  {
    case 1: // Keys
    ControlButton();                                        // control machine localy, with buttons
    break;
    
    case 2: // ESP-NOW
    ControlEspNow();                                        // control machine remotly, with espNow
    break;
    
    case 3: // Live
    if (espNowMessageDone == true)
    {
      DisplayToggle();
      if (espNowMessageNew == true)
      {
        MotorMotion();
        espNowMessageNew = false;
      }
    }
    break;
    
    case 4: // Auto
    if (espNowMessageDone == true)
    {
      DisplayToggle();
      AutoMotion();
    }
    break;

    case 5: // Fun
    if (espNowMessageDone == true)
    {
      DisplayToggle();
      if (espNowMessageNew == true)
      {
        MotorMotion();
        espNowMessageNew = false;
      }
    }
    break;
  }
}


// **************************************************************************************
void DisplayToggle()
{
  if (digitalRead(ButtonLeft) == LOW && Display == 2)
  {
    digitalWrite(TFT_BL, 0);  // Switch backlight off
    memset (&MotorDataOld.LeftMode, 0xFF, sizeof(MotorDataOld));
    Display = 1;
    DisplayShow.pushSprite(0, 0);
    DisplayShowUpdateDIR();
    DisplayDataUpdateSide = 0;
    DisplayUpdate();
    tft.setTextColor(0x0500, 0x0000);
    tft.drawString(StringSlash, DisplayShowXcenter[3], DisplayShowYcenter[3], 1);
    tft.drawNumber(MotorData[0].LeftSteps, DisplayShowXcenter[3] + 24, DisplayShowYcenter[3], 1);
    tft.setTextColor(0x035C, 0x0000);
    tft.drawString(StringSlash, DisplayShowXcenter[4], DisplayShowYcenter[4], 1);
    tft.drawNumber(MotorData[0].RightSteps, DisplayShowXcenter[4] + 24, DisplayShowYcenter[4], 1);
    digitalWrite(TFT_BL, 1);  // Switch backlight on
  }
  if (digitalRead(ButtonRight) == LOW && Display == 1)
  {
    digitalWrite(TFT_BL, 0);  // Switch backlight off
    memset (&MotorDataOld.LeftMode, 0xFF, sizeof(MotorDataOld));
    Display = 2;
    DisplayData.pushSprite(0, 0);
    DisplayDataUpdateSide = 0;
    DisplayUpdate();
    tft.setTextColor(0x0500, 0x0000);
    tft.drawString(StringSlash, DisplayDataLeftXcenter[0], DisplayDataLeftYcenter[0], 1);
    tft.drawNumber(MotorData[0].LeftSteps, DisplayDataLeftXcenter[0] + 24, DisplayDataLeftYcenter[0], 1);
    tft.setTextColor(0x035C, 0x0000);
    tft.drawString(StringSlash, DisplayDataRightXcenter[0], DisplayDataRightYcenter[0], 1);
    tft.drawNumber(MotorData[0].RightSteps, DisplayDataRightXcenter[0] + 24, DisplayDataRightYcenter[0], 1);
    digitalWrite(TFT_BL, 1);  // Switch backlight on
  }
}


// **************************************************************************************
void DisplayReset()
{
  if (Display == 1)
  {DisplayShow.pushSprite(0, 40, 0, 40, 240, height - 40);}
  if (Display == 2)
  {DisplayData.pushSprite(0, 20, 0, 20, 240, height - 20);}
}


// **************************************************************************************
void DisplayShowUpdateDIR()
{
  if (MotorData[LeftStepCounter].LeftDIR == 0)
  {
    SpriteStopOff.pushSprite(16, 42);
  }
  if (MotorData[LeftStepCounter].LeftDIR == 1)
  {
    SpriteForward.pushSprite(16, 42);
  }
  if (MotorData[LeftStepCounter].LeftDIR == 2)
  {
    SpriteStopCoast.pushSprite(16, 42);
  }
  if (MotorData[LeftStepCounter].LeftDIR == 3)
  {
    SpriteReverse.pushSprite(16, 42);
  }
  if (MotorData[LeftStepCounter].LeftDIR == 4)
  {
    SpriteStopBrake.pushSprite(16, 42);
  }

// ---

  if (MotorData[RightStepCounter].RightDIR == 0)
  {
    SpriteStopOff.pushSprite(176, 42);
  }
  if (MotorData[RightStepCounter].RightDIR == 1)
  {
    SpriteForward.pushSprite(176, 42);
  }
  if (MotorData[RightStepCounter].RightDIR == 2)
  {
    SpriteStopCoast.pushSprite(176, 42);
  }
  if (MotorData[RightStepCounter].RightDIR == 3)
  {
    SpriteReverse.pushSprite(176, 42);
  }
  if (MotorData[RightStepCounter].RightDIR == 4)
  {
    SpriteStopBrake.pushSprite(176, 42);
  }
}


// **************************************************************************************
void DisplayUpdate()
{
  // display text appearance
  tft.setTextSize(2);
  tft.setTextDatum(MC_DATUM);

  // display show update
  if (Display == 1)
  {
    if (MotorData[LeftStepCounter].LeftMode != MotorDataOld.LeftMode)
    {
      tft.setTextColor(0xFC00, 0x0000);
      tft.drawString(DisplayShowModeText[menu], DisplayShowModeXcenter, DisplayShowModeYcenter, 2);
    }
    switch (DisplayDataUpdateSide)
    {
      case 0:
      DisplayUpdateShowLeft();
      DisplayUpdateShowRight();
      break;
      case 1:
      DisplayUpdateShowLeft();
      break;
      case 2:
      DisplayUpdateShowRight();
      break;
    }
  }

  // display data update
  if (Display == 2)
  {
    if (MotorData[LeftStepCounter].LeftMode != MotorDataOld.LeftMode)
    {
      tft.setTextColor(0xFC00, 0x0000);
      tft.drawString(DisplayShowModeText[menu], DisplayDataModeXcenter, DisplayDataModeYcenter, 2);
    }
    switch (DisplayDataUpdateSide)
    {
      case 0:
      DisplayUpdateDataLeft();
      DisplayUpdateDataRight();
      break;
      case 1:
      DisplayUpdateDataLeft();
      break;
      case 2:
      DisplayUpdateDataRight();
      break;
    }
  }
}

void DisplayUpdateShowLeft()
{
  if (MotorData[LeftStepCounter].LeftCTR != MotorDataOld.LeftCTR && MotorData[LeftStepCounter].LeftCTR == 1)
  {
    tft.fillRect(DisplayShowXpos[0], DisplayShowYpos[0], DisplayShowWidth[0], DisplayShowHeight[0], 0x0000);
    tft.setTextColor(0xFFFE, 0x0000);
    tft.drawString("FIX", DisplayShowXcenter[0], DisplayShowYcenter[0], 2);
  }
  if (MotorData[LeftStepCounter].LeftCTR != MotorDataOld.LeftCTR && MotorData[LeftStepCounter].LeftCTR == 2)
  {
    tft.fillRect(DisplayShowXpos[0], DisplayShowYpos[0], DisplayShowWidth[0], DisplayShowHeight[0], 0x0000);
    tft.setTextColor(0xFFFE, 0x0000);
    tft.drawNumber(MotorData[LeftStepCounter].LeftPWM, DisplayShowXcenter[0], DisplayShowYcenter[0], 2);
  }
  if (MotorData[LeftStepCounter].LeftPWM != MotorDataOld.LeftPWM && MotorData[LeftStepCounter].LeftCTR == 2)
  {
    tft.fillRect(DisplayShowXpos[0], DisplayShowYpos[0], DisplayShowWidth[0], DisplayShowHeight[0], 0x0000);
    tft.setTextColor(0xFFFE, 0x0000);
    tft.drawNumber(MotorData[LeftStepCounter].LeftPWM, DisplayShowXcenter[0], DisplayShowYcenter[0], 2);
  }
  if (MotorData[LeftStepCounter].LeftSteps != MotorDataOld.LeftSteps || (menu == 4 && AutoModeLeftDone == 0))
  {
    tft.setTextColor(0x0500, 0x0000);
    if (LeftStepCounter == 0)
    {
      tft.fillRect(DisplayShowXpos[3], DisplayShowYpos[3], DisplayShowWidth[3], DisplayShowHeight[3], 0x0000);
      tft.drawNumber(LeftStepCounter, DisplayShowXcenter[3] - 24, DisplayShowYcenter[3], 1);
      tft.drawString(StringSlash, DisplayShowXcenter[3], DisplayShowYcenter[3], 1);
      tft.drawNumber(MotorData[0].LeftSteps, DisplayShowXcenter[3] + 24, DisplayShowYcenter[3], 1);
    }
    else
    {
      tft.drawNumber(LeftStepCounter, DisplayShowXcenter[3] - 24, DisplayShowYcenter[3], 1);
    }
  }
}

void DisplayUpdateShowRight()
{
  if (MotorData[RightStepCounter].RightCTR != MotorDataOld.RightCTR && MotorData[RightStepCounter].RightCTR == 1)
  {
    tft.fillRect(DisplayShowXpos[1], DisplayShowYpos[1], DisplayShowWidth[1], DisplayShowHeight[1], 0x0000);
    tft.setTextColor(0xFFFE, 0x0000);
    tft.drawString("FIX", DisplayShowXcenter[1], DisplayShowYcenter[1], 2);
  }
  if (MotorData[RightStepCounter].RightCTR != MotorDataOld.RightCTR && MotorData[RightStepCounter].RightCTR == 2)
  {
    tft.fillRect(DisplayShowXpos[1], DisplayShowYpos[1], DisplayShowWidth[1], DisplayShowHeight[1], 0x0000);
    tft.setTextColor(0xFFFE, 0x0000);
    tft.drawNumber(MotorData[RightStepCounter].RightPWM, DisplayShowXcenter[1], DisplayShowYcenter[1], 2);
  }
  if (MotorData[RightStepCounter].RightPWM != MotorDataOld.RightPWM && MotorData[RightStepCounter].RightCTR == 2)
  {
    tft.fillRect(DisplayShowXpos[1], DisplayShowYpos[1], DisplayShowWidth[1], DisplayShowHeight[1], 0x0000);
    tft.setTextColor(0xFFFE, 0x0000);
    tft.drawNumber(MotorData[RightStepCounter].RightPWM, DisplayShowXcenter[1], DisplayShowYcenter[1], 2);
  }
  if (MotorData[RightStepCounter].RightSteps != MotorDataOld.RightSteps || (menu == 4 && AutoModeRightDone == 0))
  {
    tft.setTextColor(0x035C, 0x0000);
    if (RightStepCounter == 0)
    {
      tft.fillRect(DisplayShowXpos[4], DisplayShowYpos[4], DisplayShowWidth[4], DisplayShowHeight[4], 0x0000);
      tft.drawNumber(RightStepCounter, DisplayShowXcenter[4] - 24, DisplayShowYcenter[4], 1);
      tft.drawString(StringSlash, DisplayShowXcenter[4], DisplayShowYcenter[4], 1);
      tft.drawNumber(MotorData[0].RightSteps, DisplayShowXcenter[4] + 24, DisplayShowYcenter[4], 1);
    }
    else
    {
      tft.drawNumber(RightStepCounter, DisplayShowXcenter[4] - 24, DisplayShowYcenter[4], 1);
    }
  }
}

void DisplayUpdateDataLeft()
{
  if (MotorData[LeftStepCounter].LeftSteps != MotorDataOld.LeftSteps || (menu == 4 && AutoModeLeftDone == 0))
  {
    tft.setTextColor(0x0500, 0x0000);
    if (LeftStepCounter == 0)
    {
      tft.fillRect(DisplayDataLeftXpos[0], DisplayDataLeftYpos[0], DisplayDataLeftWidth[0], DisplayDataLeftHeight[0], 0x0000);
      tft.drawNumber(LeftStepCounter, DisplayDataLeftXcenter[0] - 24, DisplayDataLeftYcenter[0], 1);
      tft.drawString(StringSlash, DisplayDataLeftXcenter[0], DisplayDataLeftYcenter[0], 1);
      tft.drawNumber(MotorData[0].LeftSteps, DisplayDataLeftXcenter[0] + 24, DisplayDataLeftYcenter[0], 1);
    }
    else
    {
      tft.drawNumber(LeftStepCounter, DisplayDataLeftXcenter[0] - 24, DisplayDataLeftYcenter[0], 1);
    }
  }

  if (MotorData[LeftStepCounter].LeftTime != MotorDataOld.LeftTime)
  {
    tft.fillRect(DisplayDataLeftXpos[1], DisplayDataLeftYpos[1], DisplayDataLeftWidth[1], DisplayDataLeftHeight[1], 0x0000);
    tft.setTextColor(0x0500, 0x0000);
    DisplayDataFloatTemp = MotorData[LeftStepCounter].LeftTime / DisplayDataMilliseconds;
    tft.drawFloat(DisplayDataFloatTemp, 1, DisplayDataLeftXcenter[1], DisplayDataLeftYcenter[1], 1);
  }

  if (MotorData[LeftStepCounter].LeftCTR != MotorDataOld.LeftCTR)
  {
    tft.setTextColor(0x0500, 0x0000);
    tft.drawString(DisplayDataCTR[MotorData[LeftStepCounter].LeftCTR], DisplayDataLeftXcenter[2], DisplayDataLeftYcenter[2], 1);
  }

  if (MotorData[LeftStepCounter].LeftPWM != MotorDataOld.LeftPWM)
  {
    tft.fillRect(DisplayDataLeftXpos[3], DisplayDataLeftYpos[3], DisplayDataLeftWidth[3], DisplayDataLeftHeight[3], 0x0000);
    tft.setTextColor(0x0500, 0x0000);
    tft.drawNumber(MotorData[LeftStepCounter].LeftPWM, DisplayDataLeftXcenter[3], DisplayDataLeftYcenter[3], 1);
  }

  if (MotorData[LeftStepCounter].LeftDIR != MotorDataOld.LeftDIR)
  {
    tft.setTextColor(0x0500, 0x0000);
    tft.drawString(DisplayDataDIR[MotorData[LeftStepCounter].LeftDIR], DisplayDataLeftXcenter[4], DisplayDataLeftYcenter[4], 1);
  }
}

void DisplayUpdateDataRight()
{
  if (MotorData[RightStepCounter].RightSteps != MotorDataOld.RightSteps || (menu == 4 && AutoModeRightDone == 0))
  {
    tft.setTextColor(0x035C, 0x0000);
    if (RightStepCounter == 0)
    {
      tft.fillRect(DisplayDataRightXpos[0], DisplayDataRightYpos[0], DisplayDataRightWidth[0], DisplayDataRightHeight[0], 0x0000);
      tft.drawNumber(RightStepCounter, DisplayDataRightXcenter[0] - 24, DisplayDataRightYcenter[0], 1);
      tft.drawString(StringSlash, DisplayDataRightXcenter[0], DisplayDataRightYcenter[0], 1);
      tft.drawNumber(MotorData[0].RightSteps, DisplayDataRightXcenter[0] + 24, DisplayDataRightYcenter[0], 1);
    }
    else
    {
      tft.drawNumber(RightStepCounter, DisplayDataRightXcenter[0] - 24, DisplayDataRightYcenter[0], 1);
    }
  }

  if (MotorData[RightStepCounter].RightTime != MotorDataOld.RightTime)
  {
    tft.fillRect(DisplayDataRightXpos[1], DisplayDataRightYpos[1], DisplayDataRightWidth[1], DisplayDataRightHeight[1], 0x0000);
    tft.setTextColor(0x035C, 0x0000);
    DisplayDataFloatTemp = MotorData[RightStepCounter].RightTime / DisplayDataMilliseconds;
    tft.drawFloat(DisplayDataFloatTemp, 1, DisplayDataRightXcenter[1], DisplayDataRightYcenter[1], 1);
  }

  if (MotorData[RightStepCounter].RightCTR != MotorDataOld.RightCTR)
  {
    tft.setTextColor(0x035C, 0x0000);
    tft.drawString(DisplayDataCTR[MotorData[RightStepCounter].RightCTR], DisplayDataRightXcenter[2], DisplayDataRightYcenter[2], 1);
  }

  if (MotorData[RightStepCounter].RightPWM != MotorDataOld.RightPWM)
  {
    tft.fillRect(DisplayDataRightXpos[3], DisplayDataRightYpos[3], DisplayDataRightWidth[3], DisplayDataRightHeight[3], 0x0000);
    tft.setTextColor(0x035C, 0x0000);
    tft.drawNumber(MotorData[RightStepCounter].RightPWM, DisplayDataRightXcenter[3], DisplayDataRightYcenter[3], 1);
  }

  if (MotorData[RightStepCounter].RightDIR != MotorDataOld.RightDIR)
  {
    tft.setTextColor(0x035C, 0x0000);
    tft.drawString(DisplayDataDIR[MotorData[RightStepCounter].RightDIR], DisplayDataRightXcenter[4], DisplayDataRightYcenter[4], 1);
  }
}


// **************************************************************************************
void AutoMotion()
{
  if (AutoModeLeftDone == 0 && LeftStepCounter <= MotorData[0].LeftSteps && LeftStepTime <= millis() && espNowMessageDone == true)
  {
    if (MotorData[LeftStepCounter].LeftDIR != MotorDataOld.LeftDIR || MotorData[LeftStepCounter].LeftCTR != MotorDataOld.LeftCTR || MotorData[LeftStepCounter].LeftPWM != MotorDataOld.LeftPWM)
    {
      switch (MotorData[LeftStepCounter].LeftCTR)
      {
        case 0:   // motor OFF
        MotorFIXLeftStopBrake();
        break;
        
        case 1:   // motor FIX
        switch (MotorData[LeftStepCounter].LeftDIR)
        {
          case 0: // motor OFF
          MotorFIXLeftStopOff();
          break;
          case 1: // motor FORWARD
          MotorFIXLeftForward();
          break;
          case 2: // motor STOP COAST
          MotorFIXLeftStopCoast();
          break;
          case 3: // motor REVERSE
          MotorFIXLeftReverse();
          break;
          case 4: // motor STOP BRAKE
          MotorFIXLeftStopBrake();
          break;
        }
        break;
        
        case 2:   // motor PWM
        switch (MotorData[LeftStepCounter].LeftDIR)
        {
          case 0: // motor OFF
          MotorPWMLeftStopOff();
          break;
          case 1: // motor FORWARD
          MotorPWMLeftForward();
          break;
          case 2: // motor STOP COAST
          MotorPWMLeftStopCoast();
          break;
          case 3: // motor REVERSE
          MotorPWMLeftReverse();
          break;
          case 4: // motor STOP BRAKE
          MotorPWMLeftStopBrake();
          break;
        }
        break;
      }
    }
    DisplayDataUpdateSide = 1;
    DisplayUpdate();
    memcpy(&MotorDataOld.LeftMode, &MotorData[LeftStepCounter].LeftMode, sizeof(MotorDataOld) / 2);
    if (LeftStepCounter < MotorData[0].LeftSteps)
    {
    LeftStepCounter++;
    LeftStepTime = TimeBase + MotorData[LeftStepCounter].LeftTime;    // next left step start time in milliseconds
    }
    else
    {
      AutoModeLeftDone = 1;
    }
  }

  if (AutoModeRightDone == 0 && RightStepCounter <= MotorData[0].RightSteps && RightStepTime <= millis() && espNowMessageDone == true)
  {
    if (MotorData[RightStepCounter].RightDIR != MotorDataOld.RightDIR || MotorData[RightStepCounter].RightCTR != MotorDataOld.RightCTR || MotorData[RightStepCounter].RightPWM != MotorDataOld.RightPWM)
    {
      switch (MotorData[RightStepCounter].RightCTR)
      {
        case 0:   // motor OFF
        MotorFIXRightStopBrake();
        break;
      
        case 1:   // motor FIX
        switch (MotorData[RightStepCounter].RightDIR)
        {
          case 0: // motor OFF
          MotorFIXRightStopOff();
          break;
          case 1: // motor FORWARD
          MotorFIXRightForward();
          break;
          case 2: // motor STOP COAST
          MotorFIXRightStopCoast();
          break;
          case 3: // motor REVERSE
          MotorFIXRightReverse();
          break;
          case 4: // motor STOP BRAKE
          MotorFIXRightStopBrake();
          break;
        }
        break;
      
        case 2:   // motor PWM
        switch (MotorData[RightStepCounter].RightDIR)
        {
          case 0: // motor OFF
          MotorPWMRightStopOff();
          break;
          case 1: // motor FORWARD
          MotorPWMRightForward();
          break;
          case 2: // motor STOP COAST
          MotorPWMRightStopCoast();
          break;
          case 3: // motor REVERSE
          MotorPWMRightReverse();
          break;
          case 4: // motor STOP BRAKE
          MotorPWMRightStopBrake();
          break;
        }
        break;
      }
    }
    DisplayDataUpdateSide = 2;
    DisplayUpdate();
    memcpy(&MotorDataOld.RightMode, &MotorData[RightStepCounter].RightMode, sizeof(MotorDataOld) / 2);
    if (RightStepCounter < MotorData[0].RightSteps)
    {
    RightStepCounter++;
    RightStepTime = TimeBase + MotorData[RightStepCounter].RightTime;    // next left step start time in milliseconds
    }
    else
    {
      AutoModeRightDone = 1;
    }
  }
}

// **************************************************************************************
void MotorMotion()
{
  switch (MotorData[0].LeftCTR)
  {
    case 0:   // motor OFF
    MotorFIXLeftStopBrake();
    break;
        
    case 1:   // motor FIX
    switch (MotorData[0].LeftDIR)
    {
      case 0: // motor OFF
      MotorFIXLeftStopOff();
      break;
      case 1: // motor FORWARD
      MotorFIXLeftForward();
      break;
      case 2: // motor STOP COAST
      MotorFIXLeftStopCoast();
      break;
      case 3: // motor REVERSE
      MotorFIXLeftReverse();
      break;
      case 4: // motor STOP BRAKE
      MotorFIXLeftStopBrake();
      break;
    }
    break;
        
    case 2:   // motor PWM
    switch (MotorData[0].LeftDIR)
    {
      case 0: // motor OFF
      MotorPWMLeftStopOff();
      break;
      case 1: // motor FORWARD
      MotorPWMLeftForward();
      break;
      case 2: // motor STOP COAST
      MotorPWMLeftStopCoast();
      break;
      case 3: // motor REVERSE
      MotorPWMLeftReverse();
      break;
      case 4: // motor STOP BRAKE
      MotorPWMLeftStopBrake();
      break;
    }
    break;
  }
  
  switch (MotorData[0].RightCTR)
  {
    case 0:   // motor OFF
    MotorFIXRightStopBrake();
    break;
      
    case 1:   // motor FIX
    switch (MotorData[0].RightDIR)
    {
      case 0: // motor OFF
      MotorFIXRightStopOff();
      break;
      case 1: // motor FORWARD
      MotorFIXRightForward();
      break;
      case 2: // motor STOP COAST
      MotorFIXRightStopCoast();
      break;
      case 3: // motor REVERSE
      MotorFIXRightReverse();
      break;
      case 4: // motor STOP BRAKE
      MotorFIXRightStopBrake();
      break;
    }
    break;
      
    case 2:   // motor PWM
    switch (MotorData[0].RightDIR)
    {
      case 0: // motor OFF
      MotorPWMRightStopOff();
      break;
      case 1: // motor FORWARD
      MotorPWMRightForward();
      break;
      case 2: // motor STOP COAST
      MotorPWMRightStopCoast();
      break;
      case 3: // motor REVERSE
      MotorPWMRightReverse();
      break;
      case 4: // motor STOP BRAKE
      MotorPWMRightStopBrake();
      break;
    }
    break;
  }
}


// **************************************************************************************
void ControlButton()
{
  // Button left
  if (digitalRead(ButtonLeft) == LOW)
  {
    if (ButtonLeftActive == false)
    {
      ButtonLeftActive = true;
      timerButton = millis();
    }
    if ((millis() - timerButton > longPressTime) && (ButtonLeftActiveLong == false))
    {
      ButtonLeftActiveLong = true;

      MotorDataOld.LeftDIR = MotorData[LeftStepCounter].LeftDIR;
      if (MotorData[LeftStepCounter].LeftDIR < 4)
      {MotorData[LeftStepCounter].LeftDIR++;}
      else
      {MotorData[LeftStepCounter].LeftDIR = 0;}

      if (MotorData[LeftStepCounter].LeftCTR == 1)
      {ButtonControlLeftFIXFunctions[MotorData[LeftStepCounter].LeftDIR]();}
      if (MotorData[LeftStepCounter].LeftCTR == 2)
      {ButtonControlLeftPWMFunctions[MotorData[LeftStepCounter].LeftDIR]();}

      DisplayDataUpdateSide = 1;
      DisplayUpdate();
    }
  }
  else
  {
    if (ButtonLeftActive == true)
    {
      if (ButtonLeftActiveLong == true)
      {
        ButtonLeftActiveLong = false;
      }
      else
      {
        MotorDataOld.LeftCTR = MotorData[LeftStepCounter].LeftCTR;
        MotorDataOld.LeftPWM = MotorData[LeftStepCounter].LeftPWM;
        if (MotorData[LeftStepCounter].LeftPWM < 250)
        {
          MotorData[LeftStepCounter].LeftCTR = 2;
          MotorData[LeftStepCounter].LeftPWM = MotorData[LeftStepCounter].LeftPWM + 10;  // short press > increase PWM
        }
        else
        {
          MotorData[LeftStepCounter].LeftCTR = 1;
          MotorData[LeftStepCounter].LeftPWM = 0;
        }
        
        if (MotorData[LeftStepCounter].LeftCTR == 1 && MotorData[LeftStepCounter].LeftDIR == 1)
        {MotorFIXLeftForward();}
        if (MotorData[LeftStepCounter].LeftCTR == 1 && MotorData[LeftStepCounter].LeftDIR == 3)
        {MotorFIXLeftReverse();}

        if (MotorData[LeftStepCounter].LeftCTR == 2 && MotorData[LeftStepCounter].LeftDIR == 1)
        {MotorPWMLeftForward();}
        if (MotorData[LeftStepCounter].LeftCTR == 2 && MotorData[LeftStepCounter].LeftDIR == 3)
        {MotorPWMLeftReverse();}

        DisplayDataUpdateSide = 1;
        DisplayUpdate();
      }
      ButtonLeftActive = false;
    }
  }

  // Button right
  if (digitalRead(ButtonRight) == LOW)
  {
    if (ButtonRightActive == false)
    {
      ButtonRightActive = true;
      timerButton = millis();
    }
    if ((millis() - timerButton > longPressTime) && (ButtonRightActiveLong == false))
    {
      ButtonRightActiveLong = true;

      MotorDataOld.RightDIR = MotorData[RightStepCounter].RightDIR;
      if (MotorData[RightStepCounter].RightDIR < 4)
      {MotorData[RightStepCounter].RightDIR++;}
      else
      {MotorData[RightStepCounter].RightDIR = 0;}
      
      if (MotorData[RightStepCounter].RightCTR == 1)
      {ButtonControlRightFIXFunctions[MotorData[RightStepCounter].RightDIR]();}
      if (MotorData[RightStepCounter].LeftCTR == 2)
      {ButtonControlRightPWMFunctions[MotorData[RightStepCounter].RightDIR]();}

      DisplayDataUpdateSide = 2;
      DisplayUpdate();
    }
  }
  else
  {
    if (ButtonRightActive == true)
    {
      if (ButtonRightActiveLong == true)
      {
        ButtonRightActiveLong = false;
      }
      else
      {
        MotorDataOld.RightCTR = MotorData[RightStepCounter].RightCTR;
        MotorDataOld.RightPWM = MotorData[RightStepCounter].RightPWM;
        if (MotorData[RightStepCounter].RightPWM < 250)
        {
          MotorData[RightStepCounter].RightCTR = 2;
          MotorData[RightStepCounter].RightPWM = MotorData[RightStepCounter].RightPWM + 10;  // short press > increase PWM
        }
        else
        {
          MotorData[RightStepCounter].RightCTR = 1;
          MotorData[RightStepCounter].RightPWM = 0;
        }
        
        if (MotorData[RightStepCounter].RightCTR == 1 && MotorData[RightStepCounter].RightDIR == 1)
        {MotorFIXRightForward();}
        if (MotorData[RightStepCounter].RightCTR == 1 && MotorData[RightStepCounter].RightDIR == 3)
        {MotorFIXRightReverse();}
        
        if (MotorData[RightStepCounter].RightCTR == 2 && MotorData[RightStepCounter].RightDIR == 1)
        {MotorPWMRightForward();}
        if (MotorData[RightStepCounter].RightCTR == 2 && MotorData[RightStepCounter].RightDIR == 3)
        {MotorPWMRightReverse();}

        DisplayDataUpdateSide = 2;
        DisplayUpdate();
      }
      ButtonRightActive = false;
    }
  }
}


// **************************************************************************************
void ControlEspNow()
{

}


// **************************************************************************************
// Motor Control - FIX
// --------------------------------------------------------------------------------------
void MotorFIXLeftRightStopOff ()                       // motor left and right stop off (coast)
{
  digitalWrite(MotorLeftOut1, LOW);
  digitalWrite(MotorLeftOut2, LOW);
  digitalWrite(MotorRightOut1, LOW);
  digitalWrite(MotorRightOut2, LOW);
  if (Display == 1)
  {
    SpriteStopOff.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
    SpriteStopOff.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
  }
}

void MotorFIXLeftRightForward()                         // motor left and right forward
{
  ledcDetachPin(MotorLeftOut1);
  ledcDetachPin(MotorLeftOut2);
  ledcDetachPin(MotorRightOut1);
  ledcDetachPin(MotorRightOut2);
  digitalWrite(MotorLeftOut1, HIGH);
  digitalWrite(MotorLeftOut2, LOW);
  digitalWrite(MotorRightOut1, HIGH);
  digitalWrite(MotorRightOut2, LOW);
  if (Display == 1)
  {
    SpriteForward.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
    SpriteForward.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
  }
}

void MotorFIXLeftRightStopCoast()                       // motor left and right stop coast
{
  digitalWrite(MotorLeftOut1, LOW);
  digitalWrite(MotorLeftOut2, LOW);
  digitalWrite(MotorRightOut1, LOW);
  digitalWrite(MotorRightOut2, LOW);
  if (Display == 1)
  {
    SpriteStopCoast.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
    SpriteStopCoast.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
  }
}

void MotorFIXLeftRightStopBrake()                       // motor left and right stop brake
{
  digitalWrite(MotorLeftOut1, HIGH);
  digitalWrite(MotorLeftOut2, HIGH);
  digitalWrite(MotorRightOut1, HIGH);
  digitalWrite(MotorRightOut2, HIGH);
  if (Display == 1)
  {
    SpriteStopBrake.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
    SpriteStopBrake.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
  }
}

void MotorFIXLeftRightReverse()                         // motor left and right reverse
{
  ledcDetachPin(MotorLeftOut1);
  ledcDetachPin(MotorLeftOut2);
  ledcDetachPin(MotorRightOut1);
  ledcDetachPin(MotorRightOut2);
  digitalWrite(MotorLeftOut1, LOW);
  digitalWrite(MotorLeftOut2, HIGH);
  digitalWrite(MotorRightOut1, LOW);
  digitalWrite(MotorRightOut2, HIGH);
  if (Display == 1)
  {
    SpriteReverse.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
    SpriteReverse.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
  }
}


// --------------------------------------------------------------------------------------
void MotorFIXLeftStopOff()                              // motor left stop off (coast)
{
  ledcDetachPin(MotorLeftOut1);
  ledcDetachPin(MotorLeftOut2);
  digitalWrite(MotorLeftOut1, LOW);
  digitalWrite(MotorLeftOut2, LOW);
  if (Display == 1)
  {
    SpriteStopOff.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
  }
}

void MotorFIXLeftForward()                              // motor left forward
{
  ledcDetachPin(MotorLeftOut1);
  ledcDetachPin(MotorLeftOut2);
  digitalWrite(MotorLeftOut1, HIGH);
  digitalWrite(MotorLeftOut2, LOW);
  if (Display == 1)
  {
    SpriteForward.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
  }
}

void MotorFIXLeftStopCoast()                            // motor left stop coast
{
  ledcDetachPin(MotorLeftOut1);
  ledcDetachPin(MotorLeftOut2);
  digitalWrite(MotorLeftOut1, LOW);
  digitalWrite(MotorLeftOut2, LOW);
  if (Display == 1)
  {
    SpriteStopCoast.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
  }
}

void MotorFIXLeftStopBrake()                            // motor left stop brake
{
  ledcDetachPin(MotorLeftOut1);
  ledcDetachPin(MotorLeftOut2);
  digitalWrite(MotorLeftOut1, HIGH);
  digitalWrite(MotorLeftOut2, HIGH);
  if (Display == 1)
  {
    SpriteStopBrake.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
  }
}

void MotorFIXLeftReverse()                              // motor left reverse
{
  ledcDetachPin(MotorLeftOut1);
  ledcDetachPin(MotorLeftOut2);
  digitalWrite(MotorLeftOut1, LOW);
  digitalWrite(MotorLeftOut2, HIGH);
  if (Display == 1)
  {
    SpriteReverse.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
  }
}


// --------------------------------------------------------------------------------------
void MotorFIXRightStopOff()                             // motor right stop off (coast)
{
  ledcDetachPin(MotorRightOut1);
  ledcDetachPin(MotorRightOut2);
  digitalWrite(MotorRightOut1, LOW);
  digitalWrite(MotorRightOut2, LOW);
  if (Display == 1)
  {
    SpriteStopOff.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
  }
}

void MotorFIXRightForward()                             // motor right forward
{
  ledcDetachPin(MotorRightOut1);
  ledcDetachPin(MotorRightOut2);
  digitalWrite(MotorRightOut1, HIGH);
  digitalWrite(MotorRightOut2, LOW);
  if (Display == 1)
  {
    SpriteForward.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
  }
}

void MotorFIXRightStopCoast()                           // motor right stop coast
{
  ledcDetachPin(MotorRightOut1);
  ledcDetachPin(MotorRightOut2);
  digitalWrite(MotorRightOut1, LOW);
  digitalWrite(MotorRightOut2, LOW);
  if (Display == 1)
  {
    SpriteStopCoast.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
  }
}

void MotorFIXRightStopBrake()                           // motor right stop brake
{
  ledcDetachPin(MotorRightOut1);
  ledcDetachPin(MotorRightOut2);
  digitalWrite(MotorRightOut1, HIGH);
  digitalWrite(MotorRightOut2, HIGH);
  if (Display == 1)
  {
    SpriteStopBrake.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
  }
}

void MotorFIXRightReverse()                             // motor right reverse
{
  ledcDetachPin(MotorRightOut1);
  ledcDetachPin(MotorRightOut2);
  digitalWrite(MotorRightOut1, LOW);
  digitalWrite(MotorRightOut2, HIGH);
  if (Display == 1)
  {
    SpriteReverse.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
  }
}

// **************************************************************************************
// Motor Control - PWM
// --------------------------------------------------------------------------------------
void MotorPWMLeftRightStopOff()                         // motor left and right stop off (coast)
{
  ledcDetachPin(MotorLeftOut1);
  ledcDetachPin(MotorLeftOut2);
  ledcDetachPin(MotorRightOut1);
  ledcDetachPin(MotorRightOut2);
  digitalWrite(MotorLeftOut1, LOW);
  digitalWrite(MotorLeftOut2, LOW);
  digitalWrite(MotorRightOut1, LOW);
  digitalWrite(MotorRightOut2, LOW);
  if (Display == 1)
  {
    SpriteStopOff.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
    SpriteStopOff.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
  }
}

void MotorPWMLeftRightForward()                         // motor left and right forward
{
  ledcAttachPin(MotorLeftOut1, 0);
  ledcAttachPin(MotorLeftOut2, 1);
  ledcAttachPin(MotorRightOut1, 2);
  ledcAttachPin(MotorRightOut2, 3);
  ledcWrite(1, 0);
  ledcWrite(0, MotorData[LeftStepCounter].LeftPWM);
  digitalWrite(MotorLeftOut2, LOW);
  ledcWrite(3, 0);
  ledcWrite(2, MotorData[RightStepCounter].RightPWM);
  digitalWrite(MotorRightOut2, LOW);
  if (Display == 1)
  {
    SpriteForward.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
    SpriteForward.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
  }
}

void MotorPWMLeftRightStopCoast()                       // motor left and right stop coast
{
  ledcDetachPin(MotorLeftOut1);
  ledcDetachPin(MotorLeftOut2);
  ledcDetachPin(MotorRightOut1);
  ledcDetachPin(MotorRightOut2);
  digitalWrite(MotorLeftOut1, LOW);
  digitalWrite(MotorLeftOut2, LOW);
  digitalWrite(MotorRightOut1, LOW);
  digitalWrite(MotorRightOut2, LOW);
  if (Display == 1)
  {
    SpriteStopCoast.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
    SpriteStopCoast.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
  }
}

void MotorPWMLeftRightStopBrake()                       // motor left and right stop brake
{
  ledcDetachPin(MotorLeftOut1);
  ledcDetachPin(MotorLeftOut2);
  ledcDetachPin(MotorRightOut1);
  ledcDetachPin(MotorRightOut2);
  digitalWrite(MotorLeftOut1, HIGH);
  digitalWrite(MotorLeftOut2, HIGH);
  digitalWrite(MotorRightOut1, HIGH);
  digitalWrite(MotorRightOut2, HIGH);
  if (Display == 1)
  {
    SpriteStopBrake.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
    SpriteStopBrake.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
  }
}

void MotorPWMLeftRightReverse()                         // motor left and right reverse
{
  ledcAttachPin(MotorLeftOut1, 0);
  ledcAttachPin(MotorLeftOut2, 1);
  ledcAttachPin(MotorRightOut1, 2);
  ledcAttachPin(MotorRightOut2, 3);
  ledcWrite(0, 0);
  ledcWrite(1, MotorData[LeftStepCounter].LeftPWM);
  digitalWrite(MotorLeftOut1, LOW);
  ledcWrite(2, 0);
  ledcWrite(3, MotorData[RightStepCounter].RightPWM);
  digitalWrite(MotorRightOut1, LOW);
  if (Display == 1)
  {
    SpriteReverse.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
    SpriteReverse.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
  }
}


// --------------------------------------------------------------------------------------
void MotorPWMLeftStopOff()                              // motor left stop off (coast)
{
  ledcDetachPin(MotorLeftOut1);
  ledcDetachPin(MotorLeftOut2);
  digitalWrite(MotorLeftOut1, LOW);
  digitalWrite(MotorLeftOut2, LOW);
  if (Display == 1)
  {
    SpriteStopOff.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
  }
}

void MotorPWMLeftForward()                              // motor left forward
{
  ledcAttachPin(MotorLeftOut1, 0);
  ledcAttachPin(MotorLeftOut2, 1);
  ledcWrite(1, 0);
  ledcWrite(0, MotorData[LeftStepCounter].LeftPWM);
  digitalWrite(MotorLeftOut2, LOW);
  if (Display == 1)
  {
    SpriteForward.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
  }
}

void MotorPWMLeftStopCoast()                            // motor left stop coast
{
  ledcDetachPin(MotorLeftOut1);
  ledcDetachPin(MotorLeftOut2);
  digitalWrite(MotorLeftOut1, LOW);
  digitalWrite(MotorLeftOut2, LOW);
  if (Display == 1)
  {
    SpriteStopCoast.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
  }
}

void MotorPWMLeftStopBrake()                            // motor left stop brake
{
  ledcDetachPin(MotorLeftOut1);
  ledcDetachPin(MotorLeftOut2);
  digitalWrite(MotorLeftOut1, HIGH);
  digitalWrite(MotorLeftOut2, HIGH);
  if (Display == 1)
  {
    SpriteStopBrake.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
  }
}

void MotorPWMLeftReverse()                              // motor left reverse
{
  ledcAttachPin(MotorLeftOut1, 0);
  ledcAttachPin(MotorLeftOut2, 1);
  ledcWrite(0, 0);
  ledcWrite(1, MotorData[LeftStepCounter].LeftPWM);
  digitalWrite(MotorLeftOut1, LOW);
  if (Display == 1)
  {
    SpriteReverse.pushSprite(DisplayShowLeftDIRXpos, DisplayShowLeftDIRYpos);
  }
}


// --------------------------------------------------------------------------------------
void MotorPWMRightStopOff()                             // motor right stop off (coast)
{
  ledcDetachPin(MotorRightOut1);
  ledcDetachPin(MotorRightOut2);
  digitalWrite(MotorRightOut1, LOW);
  digitalWrite(MotorRightOut2, LOW);
  if (Display == 1)
  {
    SpriteStopOff.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
  }
}

void MotorPWMRightForward()                             // motor right forward
{
  ledcAttachPin(MotorRightOut1, 2);
  ledcAttachPin(MotorRightOut2, 3);
  ledcWrite(3, 0);
  ledcWrite(2, MotorData[RightStepCounter].RightPWM);
  digitalWrite(MotorRightOut2, LOW);
  if (Display == 1)
  {
    SpriteForward.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
  }
}

void MotorPWMRightStopCoast()                           // motor right stop coast
{
  ledcDetachPin(MotorRightOut1);
  ledcDetachPin(MotorRightOut2);
  digitalWrite(MotorRightOut1, LOW);
  digitalWrite(MotorRightOut2, LOW);
  if (Display == 1)
  {
    SpriteStopCoast.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
  }
}

void MotorPWMRightStopBrake()                           // motor right stop brake
{
  ledcDetachPin(MotorRightOut1);
  ledcDetachPin(MotorRightOut2);
  digitalWrite(MotorRightOut1, HIGH);
  digitalWrite(MotorRightOut2, HIGH);
  if (Display == 1)
  {
    SpriteStopBrake.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
  }
}

void MotorPWMRightReverse()                             // motor right reverse
{
  ledcAttachPin(MotorRightOut1, 2);
  ledcAttachPin(MotorRightOut2, 3);
  ledcWrite(2, 0);
  ledcWrite(3, MotorData[RightStepCounter].RightPWM);
  digitalWrite(MotorRightOut1, LOW);
  if (Display == 1)
  {
    SpriteReverse.pushSprite(DisplayShowRightDIRXpos, DisplayShowRightDIRYpos);
  }
}


// **************************************************************************************
void DisplayShowCreate()
{
  // DisplayShow - create
  DisplayShow.setColorDepth(8);
  DisplayShow.createSprite(width, height);
  DisplayShow.fillSprite(0x0000);

  // DisplayShow - text appearance
  DisplayShow.setTextFont(2);
  DisplayShow.setTextSize(2);
  DisplayShow.setTextDatum(MC_DATUM);
  DisplayShow.setTextColor(0xFFFE, 0x0000);
  
  // DisplayShow - build screen
  ArrowWhiteLeft.pushToSprite(&DisplayShow, DisplayShowLeftArrowXpos, DisplayShowLeftArrowYpos);
  ArrowWhiteRight.pushToSprite(&DisplayShow, DisplayShowRightArrowXpos, DisplayShowRightArrowYpos);
  DisplayShow.drawString("PWM", DisplayShowXcenter[2], DisplayShowYcenter[2], 2);
}


// **************************************************************************************
void ArrowWhiteLeftCreate()
{
  // ArrowWhiteLeft - create
  ArrowWhiteLeft.setColorDepth(8);
  ArrowWhiteLeft.createSprite(ArrowWhiteLeftWidth, ArrowWhiteLeftHeight);
  ArrowWhiteLeft.fillSprite(0x0000);
  ArrowWhiteLeft.fillTriangle(0, 12, 32, 0, 32, 24, 0xC618);  // arrow left GRAY
}


// **************************************************************************************
void ArrowWhiteRightCreate()
{
  // ArrowWhiteRight - create
  ArrowWhiteRight.setColorDepth(8);
  ArrowWhiteRight.createSprite(ArrowWhiteRightWidth, ArrowWhiteRightHeight);
  ArrowWhiteRight.fillSprite(0x0000);
  ArrowWhiteRight.fillTriangle(32, 12, 0, 0, 0, 24, 0xC618);  // arrow right GRAY
}


// **************************************************************************************
void SpriteStopOffCreate()                                // stop off
{
  SpriteStopOff.setColorDepth(8);
  SpriteStopOff.createSprite(48, 48);
  SpriteStopOff.fillSprite(0x0000);
  SpriteStopOff.fillRect(0, 0, 48, 48, 0x4208);
  SpriteStopOff.fillRect(12, 12, 24, 24, 0x0000);
}


// **************************************************************************************
void SpriteStopCoastCreate()                              // stop coast
{
  SpriteStopCoast.setColorDepth(8);
  SpriteStopCoast.createSprite(48, 48);
  SpriteStopCoast.fillSprite(0x0000);
  SpriteStopCoast.fillRect(0, 0, 48, 48, 0x4208);
  SpriteStopCoast.fillRect(12, 12, 24, 24, 0x0012);
}


// **************************************************************************************
void SpriteStopBrakeCreate()                              // stop brake
{
  SpriteStopBrake.setColorDepth(8);
  SpriteStopBrake.createSprite(48, 48);
  SpriteStopBrake.fillSprite(0x0000);
  SpriteStopBrake.fillRect(0, 0, 48, 48, 0x4208);
  SpriteStopBrake.fillRect(12, 12, 24, 24, 0x6000);
}


// **************************************************************************************
void SpriteForwardCreate()                                // forward
{
  SpriteForward.setColorDepth(8);
  SpriteForward.createSprite(48, 48);
  SpriteForward.fillSprite(0x0000);
  SpriteForward.fillTriangle(24, 0, 0, 48, 48, 48, 0x07E0);
}


// **************************************************************************************
void SpriteReverseCreate()                                // reverse
{
  SpriteReverse.setColorDepth(8);
  SpriteReverse.createSprite(48, 48);
  SpriteReverse.fillSprite(0x0000);
  SpriteReverse.fillTriangle(24, 48, 48, 0, 0, 0, 0xF800);
}


// **************************************************************************************
void DisplayDataCreate()
{
  // DisplayData - create
  DisplayData.setColorDepth(8);
  DisplayData.createSprite(width, height);
  DisplayData.fillSprite(0x0000);

  // DisplayData - text appearance
  DisplayData.setTextSize(2);
  DisplayData.setTextDatum(MC_DATUM);

  // DisplayData - build screen
  DisplayData.setTextColor(0x0500, 0x0000);
  DisplayData.drawString("Left", 40, 14, 1);
  DisplayData.setTextColor(0x035C, 0x0000);
  DisplayData.drawString("Right", 200, 14, 1);
  DisplayData.setTextColor(0xFFFE, 0x0000);
  DisplayData.drawString("Step", 120, 40, 1);
  DisplayData.drawString("Time", 120, 61, 1);
  DisplayData.drawString("Ctrl", 120, 82, 1);
  DisplayData.drawString("PWM", 120, 103, 1);
  DisplayData.drawString("DIR", 120, 124, 1);
}
