// **************************************************************************************
// el Comandante                                                    guerilla.bplaced.net
// --------------------------------------------------------------------------------------
// Information:
// Controlling a 30 years old RC-Car toy with 2 motors remotly using the wireless ESP-NOW protocol.
// This software is designed for the ESP32 microcontroller unit and the DRV8833 motor driver.
// The complete arduino software bundle contains two parts and the manual:
// - The software for the WT32-SC01 ESP32 Development Board (ESP-NOW Transmitter).
// - The software for the LILYGO TTGO-T-Display ESP32 Development Board (ESP-NOW Receiver).
// This is the arduino software for the WT32-SC01 device (Transmitter).
// BEFORE COMPILING INSERT THE MAC ADDRESS OF THE ESP NOW RECEIVER DEVICE --> uint8_t   broadcastAddress[]
// For the WT32-SC01 device (FT6336U touch screen) the Adafruit_FT6206 touch library is modified > Wire.begin(18, 19).
// Option to control the motors directly without ESP-NOW, using the build in buttons of the LILYGO TTGO T ESP32 device.
// DRV8833 motor driver: No ENABLE pin available, the PWM signal must be connected to the correct IN pin of the DRV8833.
// --------------------------------------------------------------------------------------
// Arduino:
// Tools --> Board --> ESP32 Wrover Module
// --------------------------------------------------------------------------------------
// Used Libraries:
// TFT_eSPI                 --> https://github.com/Bodmer/TFT_eSPI
// Adafruit_FT6206_Library  --> https://github.com/adafruit/Adafruit_FT6206_Library
// --------------------------------------------------------------------------------------
// WT32-SC01 Device:
// ESP32-WROVER-B, 4 MB SPI Flash and 8 MB PSRAM (16 MB version availabale on Alibaba)
// Dual-Core XtensaRO 32-Bit LX6 MCU, 240 Mhz
// 3.5" 320 x 480 TFT (ST7796) capacitive 2-Touch (FT6336U)
// USB-C, WiFi 2.4 GHz WLAN 802.11 b/g/n, Bluetooth 4.2 BR/EDR- and BLE-Standards
// 5V DC / up to 2A, -20℃ ~ 85℃
//
// Goodies: Extra 5 Volt power supply possible, but not mounted. 5 Volt and 3.3 Volt available at Expansion Interface.
// Warning: Reading pixel from the TFT using HSPI with the TFT_eSPI library or the LovyanGFX library isn't working.
// Issues : No IPS-TFT. Touchscreen is okay, but could be better. In old version positioning holes without mod not usable.
//
// WT32-SC01 Pins:
// TFT_MISO = GPIO 12
// TFT_MOSI = GPIO 13
// TFT_SCLK = GPIO 14
// TFT_CS   = GPIO 15
// TFT_DC   = GPIO 21
// TFT_RST  = GPIO 22
// TFT_BL   = GPIO 23
// I2C_SDA  = GPIO 18
// I2C_SCL  = GPIO 19
//
// --------------------------------------------------------------------------------------
// ESP32 - Restricted Pins:
// GPIO0, GPIO2, GPIO4, GPIO5, GPIO12 and GPIO15 are strapping pins that bring the ESP32 into bootloader or flash mode
// while booting. Connecting something to it during boot which pull the pins into the HIGH or LOW state can lead to
// unexpected behavior.
// The ADC2_CHx pins cannot be used as analog inputs when WiFi is active.
// GPIO34-39 are inputs only, no internal pull-up or pull-down resistor available on these pins.
//
// ESP32 - Input Pins:
// Using GPIOs as an input must be enabled with INPUT_PULLDOWN when using the ESP32, i.e: pinMode(pin, INPUT_PULLDOWN)
// Choosing just INPUT, the pin level is undefined.
//
// ESP32 - DEFAULT SPI interfaces, HSPI and VSPI, GPIO pins: 
// SPI-Interface    MOSI      MISO      CLK       CS
//     HSPI        GPIO 13   GPIO 12   GPIO 14   GPIO 15
//     VSPI        GPIO 23   GPIO 19   GPIO 18   GPIO 5
//
// ESP32 - DEFAULT I2C interface GPIO pins:
// I2C-interface  I2C_SDA   I2C_SCL
//                GPIO 21   GPIO 22
//
// --------------------------------------------------------------------------------------
// Documentation and Libraries:
// Overview ESP32 mcu's   - https://socialcompare.com/en/comparison/esp8266-vs-esp32-vs-esp32-s2
// Espressif ESP32 docu's - https://www.espressif.com/en/support/documents/technical-documents
// Device WT32-SC01       - http://myosuploads3.banggood.com/products/20220111/20220111020446WT32-SC01DataSheetV3.3.pdf
// TFT Device ST7796      - https://cdn.hackaday.io/files/1625926956336128/ST7796S.pdf
// Touch Device FT6336U   - https://focuslcds.com/content/FT6236.pdf
// TFT_eSPI library       - https://github.com/Bodmer/TFT_eSPI
// ST7796 library         - included in the TFT_eSPI library
// FT6336U library        - https://github.com/adafruit/Adafruit_FT6206_Library
// **************************************************************************************

#include <Arduino.h>                            // Arduino
#include <esp_now.h>                            // ESP-NOW
#include <SPIFFS.h>                             // SPIFFS Filesystem
#include <WiFi.h>                               // WiFi
#include <Wire.h>                               // Wire
#include <math.h>                               // Mathe
#include <TFT_eSPI.h>                           // TFT_espi library
TFT_eSPI tft          = TFT_eSPI();             // 
TFT_eSprite Auto      = TFT_eSprite(&tft);      // auto - sprite menu
TFT_eSprite Timeline  = TFT_eSprite(&tft);      // auto - sprite timeline
TFT_eSprite AutoOption= TFT_eSprite(&tft);      // auto - sprite option
TFT_eSprite Needle    = TFT_eSprite(&tft);      // fun  - sprite needle

// FT6206, FT6336U - touchscreen library, modified
#include "Adafruit_FT6206.h"
Adafruit_FT6206 ts = Adafruit_FT6206();

// espNOW - MAC address                       // REPLACE WITH YOUR ESP32-NOW RECEIVER MAC Address
uint8_t   broadcastAddress[]          = {0x78, 0x00, 0x00, 0x00, 0x00, 0xAC};

// espNOW - variables
byte      espNowMsgDelay              = 20;   // standard message delay in ms, the esp32 needs time to proceed a message
const int espNowMsgDataArrayMax       = 999;  // max. motor data arrays, index 0 is the first array
int       espNowMsgDataArrayIndex     = 0;    // max. messages send = espNowMsgDataArrayMax + 1
int       espNowMsgDataArrayToSend    = 0;    // messages send
int       espNowMessageError          = 0;
int       espNowStatusError           = 0;

// espNOW - sender data structure, must match the receiver data structure, 32 bit aligned!
typedef struct struct_MotorData
{
  uint16_t  LeftMode    = 0;    // 3 = Live, 4 = Auto, 5 = Fun - only used in array index 0
  uint16_t  LeftSteps   = 0;    // Motor left, total steps - only used in array index 0
  uint32_t  LeftTime    = 0;    // Time left motor action starts, in ms
  byte      LeftDIR     = 0;    // 0 = OFF, 1 = FORWARD, 2 = STOP COAST, 3 = REVERSE, 4 = STOP BRAKE
  byte      LeftCTR     = 1;    // 0 = OFF, 1 = FIX, 2 = PWM
  uint16_t  LeftPWM     = 180;  // PWM duty cycle
  uint16_t  RightMode   = 0;    // 3 = Live, 4 = Auto, 5 = Fun - only used in array index 0
  uint16_t  RightSteps  = 0;    // Motor right, total steps - only used in array index 0
  uint32_t  RightTime   = 0;    // Time right motor action starts, in ms
  byte      RightDIR    = 0;    // 0 = OFF, 1 = FORWARD, 2 = STOP COAST, 3 = REVERSE, 4 = STOP BRAKE
  byte      RightCTR    = 1;    // 0 = OFF, 1 = FIX, 2 = PWM
  uint16_t  RightPWM    = 180;  // PWM duty cycle
}
struct_MotorData;

// espNOW - create a struct_espNowData called MotorData
struct_MotorData MotorData[espNowMsgDataArrayMax + 1];

// espNOW - create a variable esp_now_peer_info_t to store peer information
esp_now_peer_info_t peerInfo;

// Variables - misc
uint32_t  update                      = 0;
int32_t   e                           = 0;
int32_t   i                           = 0;
int32_t   v                           = 0;
int32_t   x                           = 0;
int32_t   y                           = 0;
int32_t   z                           = 0;

// Variables - screen
int32_t   width                       = 0;
int32_t   height                      = 0;

// Variables - menu
byte      menu                        = 0;

// Variables - touch
bool      touch                       = false;  // false = touchscreen init error
typedef struct struct_Touch0                    // touchpoint 0   
{
  uint16_t  x;                                  // x point 
  uint16_t  y;                                  // y point
  uint16_t  z;                                  // 1 = currently touched ?
}
struct_Touch0;
struct_Touch0 tp0;                              // create a struct_Touch0 called tp0
typedef struct struct_Touch1                    // touchpoint 1    
{
  uint16_t  x;                                  // x point 
  uint16_t  y;                                  // y point
  uint16_t  z;                                  // 1 = currently touched ?
}
struct_Touch1;
struct_Touch1 tp1;                              // create a struct_Touch1 called tp1

// Variables  - spiffs
File root;                                      // SPIFFS root variable
File file;                                      // SPIFFS file variable
bool      spiffs                      = false;  // false = SPIFFS init error
String    SPIFFSFilePathRecord        = "/WT32-SC01_ESPNOW_DRV8833_V1_0";
String    SPIFFSFilePathBase          = "/WT32-SC01_ESPNOW_DRV8833_V1_";
String    SPIFFSFilePathTemp          = "";
uint32_t  SPIFFSFilesize              = 0;

// Live - title
uint16_t  LiveTitleLeftXpos           = 80;
uint16_t  LiveTitleLeftYpos           = 22;
uint16_t  LiveTitleRightXpos          = 242;
uint16_t  LiveTitleRightYpos          = 22;

// Live - motor left buttons
char      LiveButtonLeftChar[9][11]   = {"PWM", "-", "+", "Stop Coast", "Stop Brake", "Forward", "Reverse", "Forward", "Reverse"};
uint16_t  LiveButtonLeftXpos[9]       = {4, 56, 108, 4, 4, 4, 4, 4, 4};
uint16_t  LiveButtonLeftXend[9]       = {0, 0, 0, 0, 0, 0, 0, 0, 0};
uint16_t  LiveButtonLeftYpos[9]       = {54, 54, 54, 108, 162, 216, 270, 324, 378};
uint16_t  LiveButtonLeftYend[9]       = {0, 0, 0, 0, 0, 0, 0, 0, 0};
uint16_t  LiveButtonLeftWidth[9]      = {48, 48, 48, 112, 112, 112, 112, 112, 112};
uint16_t  LiveButtonLeftHeight[9]     = {48, 48, 48, 48, 48, 48, 48, 48, 48};
uint16_t  LiveButtonLeftXcenter[9]    = {0, 0, 0, 0, 0, 0, 0, 0, 0};
uint16_t  LiveButtonLeftYcenter[9]    = {0, 0, 0, 0, 0, 0, 0, 0, 0};
uint16_t  LiveButtonLeftRadius[9]     = {6, 6, 6, 6, 6, 6, 6, 6, 6};

// Live - motor both buttons
uint16_t  LiveButtonBothXpos[6]       = {124, 124, 124, 124, 124, 124};
uint16_t  LiveButtonBothXend[6]       = {0, 0, 0, 0, 0, 0};
uint16_t  LiveButtonBothYpos[6]       = {112, 166, 220, 274, 328, 382};
uint16_t  LiveButtonBothYend[6]       = {0, 0, 0, 0, 0, 0};
uint16_t  LiveButtonBothWidth[6]      = {72, 72, 72, 72, 72, 72};
uint16_t  LiveButtonBothHeight[6]     = {40, 40, 40, 40, 40, 40};
uint16_t  LiveButtonBothRadius[6]     = {6, 6, 6, 6, 6, 6};

// Live - motor right buttons
char      LiveButtonRightChar[9][11]  = {"PWM", "-", "+", "Stop Coast", "Stop Brake", "Reverse", "Forward", "Forward", "Reverse"};
uint16_t  LiveButtonRightXpos[9]      = {164, 216, 268, 204, 204, 204, 204, 204, 204};
uint16_t  LiveButtonRightXend[9]      = {0, 0, 0, 0, 0, 0, 0, 0, 0};
uint16_t  LiveButtonRightYpos[9]      = {54, 54, 54, 108, 162, 216, 270, 324, 378};
uint16_t  LiveButtonRightYend[9]      = {0, 0, 0, 0, 0, 0, 0, 0, 0};
uint16_t  LiveButtonRightWidth[9]     = {48, 48, 48, 112, 112, 112, 112, 112, 112};
uint16_t  LiveButtonRightHeight[9]    = {48, 48, 48, 48, 48, 48, 48, 48, 48};
uint16_t  LiveButtonRightXcenter[9]   = {0, 0, 0, 0, 0, 0, 0, 0, 0};
uint16_t  LiveButtonRightYcenter[9]   = {0, 0, 0, 0, 0, 0, 0, 0, 0};
uint16_t  LiveButtonRightRadius[9]    = {6, 6, 6, 6, 6, 6, 6, 6, 6};

// Live - button reboot
uint16_t  LiveButtonRebootXpos        = 2;
uint16_t  LiveButtonRebootXend        = 0;
uint16_t  LiveButtonRebootYpos        = 442;
uint16_t  LiveButtonRebootYend        = 0;
uint16_t  LiveButtonRebootWidth       = 40;
uint16_t  LiveButtonRebootHeight      = 36;
uint16_t  LiveButtonRebootXcenter     = 0;
uint16_t  LiveButtonRebootYcenter     = 0;
uint16_t  LiveButtonRebootRadius      = 4;

// Live - status title and result
uint16_t  LiveStatusTitleXpos[4]      = {64, 94, 128, 174};
uint16_t  LiveStatusTitleYpos[4]      = {448, 448, 448, 448};
uint16_t  LiveStatusResultXpos[4]     = {63, 92, 127, 174};
uint16_t  LiveStatusResultYpos[4]     = {466, 466, 466, 466};

// Live - button off
uint16_t  LiveButtonOffXpos           = 208;
uint16_t  LiveButtonOffXend           = 0;
uint16_t  LiveButtonOffYpos           = 442;
uint16_t  LiveButtonOffYend           = 0;
uint16_t  LiveButtonOffWidth          = 106;
uint16_t  LiveButtonOffHeight         = 36;
uint16_t  LiveButtonOffXcenter        = 0;
uint16_t  LiveButtonOffYcenter        = 0;
uint16_t  LiveButtonOffRadius         = 8;

// Auto - variables and arrays
char      AutoDIR[5][6]               = {"OFF", "For", "Coast", "Rev", "Brake"};
char      AutoCTR[3][4]               = {"OFF", "FIX", "PWM"};
uint16_t  AutoMenuXpos                = 204;
uint16_t  AutoMenuYpos                = 0;
uint16_t  AutoMenuWidth               = 116;
uint16_t  AutoMenuHeight              = 320;
uint16_t  AutoMenuXcenter             = 262;
uint16_t  AutoMenuValueColorBG        = 0x0300;
byte      AutoOptions                 = 0;
byte      AutoOptionFileLoadSlot      = 0;
byte      AutoOptionFileSaveSlot      = 1;
byte      AutoMotor                   = 0;
uint16_t  AutoColor                   = 0;
uint16_t  AutoLeftStep                = 0;
uint16_t  AutoRightStep               = 0;
float     AutoMilliseconds            = 1000.0;

// Auto - menu title
char      AutoMenuTitleChar[17][17]   = {"", "Motor Selection", "", "Step Number", "", "Start Time in s", "", "Direction", "", "Control Mode", "", "PWM", "", "", "", "", ""};
uint16_t  AutoMenuTitleXcenter[17]    = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
uint16_t  AutoMenuTitleYcenter[17]    = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

// Auto - timeline
uint16_t  AutoTimelineBase            = 0;
uint16_t  AutoTimelineEnd             = 10;
uint16_t  AutoTimelineAdd             = 1;
uint16_t  AutoTimelineInc[4]          = {1, 10, 100, 1000};
uint16_t  AutoTimelineBaseInc[2]      = {100, 10};
uint16_t  AutoTimelineBaseIncMax[2]   = {9800, 9890};
uint16_t  AutoTimelineBaseDec[2]      = {10, 100};
uint16_t  AutoTimelineBaseDecMin[2]   = {10, 100};

// Auto - menu value
uint16_t  AutoMenuValueXcenter[17]    = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
uint16_t  AutoMenuValueYcenter[17]    = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
uint16_t  AutoMenuValueIncMulti       = 1;

// Auto - menu on top
float     AutoMenuFloatOnTopTemp      = 0.0;
uint32_t  AutoMenuValueOnTopTemp      = 0;
uint32_t  AutoMenuOnTopTimer          = 0;
bool      AutoMenuOnTopFlag           = false;
bool      AutoMenuStringOnTopFlag     = false;
String    AutoMenuStringOnTopValue    = "Value x ";
String    AutoMenuStringOnTopTime     = "Time  x ";
String    AutoMenuStringOnTopBase     = "Base = ";
String    AutoMenuStringOnTopAdd      = "ADD";
String    AutoMenuStringOnTopDel      = "DEL";
String    AutoMenuStringOnTopSync     = "SYNC";
String    AutoMenuStringOnTopTemp     = "";

// Auto - option on top
uint32_t  AutoOptionValueOnTopTemp    = 0;
uint32_t  AutoOptionOnTopTimer        = 0;
bool      AutoOptionOnTopFlag         = false;
bool      AutoOptionStringOnTopFlag   = false;
String    AutoOptionStringOnTopFmtOK  = "Format Okay ";
String    AutoOptionStringOnTopFmtER  = "Format Error";
String    AutoOptionStringOnTopByte   = " Byte";
String    AutoOptionStringOnTopLoadOK = "Load Okay   ";
String    AutoOptionStringOnTopLoadER = "Load Error  ";
String    AutoOptionStringOnTopSaveOK = "Save Okay   ";
String    AutoOptionStringOnTopSaveER = "Save Error  ";
String    AutoOptionStringOnTopResetL = "Reset Left  ";
String    AutoOptionStringOnTopResetR = "Reset Right ";
String    AutoOptionStringOnTopTemp   = "";

// Auto - options, files to display
uint16_t  AutoOptionFilesBase         = 0;
uint16_t  AutoOptionFilesBaseOld      = 0;
uint16_t  AutoOptionFilesXpos[4]      = {2, 52, 102, 152};
uint16_t  AutoOptionFilesYpos[4]      = {32, 32, 32, 32};
uint16_t  AutoOptionFilesWidth[4]     = {44, 44, 44, 44};
uint16_t  AutoOptionFilesHeight[4]    = {36, 36, 36, 36};
uint16_t  AutoOptionFilesRadius[4]    = {6, 6, 6, 6};
uint16_t  AutoOptionFilesValue[4]     = {0, 30, 60, 90};

// Auto - bots graphic
uint16_t  AutoBotsLeftYpos[10];
uint16_t  AutoBotsLeftYend[10];
uint16_t  AutoBotsRightYpos[10];
uint16_t  AutoBotsRightYend[10];

// Auto - menu buttons
char      AutoMenuButtonChar[17][10]  = {"=", "<", ">", "-", "+", "-", "+", "-", "+", "-", "+", "-", "+", "ADD", "DEL", "Sync Step", "Now"};
uint16_t  AutoMenuButtonXpos[17]      = {2, 2, 60, 2, 60, 2, 60, 2, 60, 2, 60, 2, 60, 2, 60, 2, 2};
uint16_t  AutoMenuButtonYpos[17]      = {2, 54, 54, 104, 104, 154, 154, 204, 204, 254, 254, 304, 304, 351, 351, 398, 444};
uint16_t  AutoMenuButtonWidth[17]     = {112, 54, 54, 54, 54, 54, 54, 54, 54, 54, 54, 54, 54, 54, 54, 112, 112};
uint16_t  AutoMenuButtonHeight[17]    = {32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 36};
uint16_t  AutoMenuButtonRadius[17]    = {4, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 4};

// Auto - menu touch areas
uint16_t  AutoMenuTouchXpos[17]       = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
uint16_t  AutoMenuTouchYpos[17]       = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
uint16_t  AutoMenuTouchXend[17]       = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
uint16_t  AutoMenuTouchYend[17]       = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

// Auto - value multiplicator
uint16_t  AutoMultiplicator[4]        = {1, 10, 100, 1000};

// Auto - time touch areas
uint16_t  AutoTimeMultiTouchXpos[4]   = {0, 0, 0, 0};
uint16_t  AutoTimeMultiTouchYpos[4]   = {30, 130, 230, 330};
uint16_t  AutoTimeMultiTouchXend[4]   = {75, 75, 75, 75};
uint16_t  AutoTimeMultiTouchYend[4]   = {130, 230, 330, 430};

// Auto - value touch areas
uint16_t  AutoValueMultiTouchXpos[4]  = {125, 125, 125, 125};
uint16_t  AutoValueMultiTouchYpos[4]  = {30, 130, 230, 330};
uint16_t  AutoValueMultiTouchXend[4]  = {200, 200, 200, 200};
uint16_t  AutoValueMultiTouchYend[4]  = {130, 230, 330, 430};

// Auto - timeline base increment touch
uint16_t  AutoTBaseIncTouchXpos[2]    = {75, 75};
uint16_t  AutoTBaseIncTouchYpos[2]    = {30, 130};
uint16_t  AutoTBaseIncTouchXend[2]    = {125, 125};
uint16_t  AutoTBaseIncTouchYend[2]    = {130, 230};

// Auto - timeline base decrement touch
uint16_t  AutoTBaseDecTouchXpos[2]    = {75, 75};
uint16_t  AutoTBaseDecTouchYpos[2]    = {230, 330};
uint16_t  AutoTBaseDecTouchXend[2]    = {125, 125};
uint16_t  AutoTBaseDecTouchYend[2]    = {330, 430};

// Auto - option touch areas
uint16_t  AutoOptionTouchXpos[10]     = {206, 206, 206, 262, 206, 206, 262, 206, 206, 206};
uint16_t  AutoOptionTouchYpos[10]     = {2, 76, 150, 150, 194, 268, 268, 312, 386, 432};
uint16_t  AutoOptionTouchXend[10]     = {318, 318, 262, 318, 318, 262, 318, 318, 318, 318};
uint16_t  AutoOptionTouchYend[10]     = {34, 108, 182, 182, 226, 300, 300, 344, 418, 464};

// Auto - menu / option, long touch
uint32_t  AutoButtonTouchLongTime     = 1500;
uint32_t  AutoButtonOptionTouchTime   = 0;
byte      AutoButtonOptionTouchLong   = 0;
uint32_t  AutoButtonAddTouchTime      = 0;
byte      AutoButtonAddTouchLong      = 0;
uint32_t  AutoButtonDelTouchTime      = 0;
byte      AutoButtonDelTouchLong      = 0;
uint32_t  AutoButtonSyncTouchTime     = 0;
byte      AutoButtonSyncTouchLong     = 0;
uint32_t  AutoButtonFormatTouchTime   = 0;
byte      AutoButtonFormatTouchLong   = 0;
uint32_t  AutoButtonLoadTouchTime     = 0;
byte      AutoButtonLoadTouchLong     = 0;
uint32_t  AutoButtonSaveTouchTime     = 0;
byte      AutoButtonSaveTouchLong     = 0;
uint32_t  AutoButtonResetLTouchTime   = 0;
byte      AutoButtonResetLTouchLong   = 0;
uint32_t  AutoButtonResetRTouchTime   = 0;
byte      AutoButtonResetRTouchLong   = 0;
uint32_t  AutoButtonRebootTouchTime   = 0;
byte      AutoButtonRebootTouchLong   = 0;

// Auto - button reboot
uint16_t  AutoButtonRebootXpos        = 2;
uint16_t  AutoButtonRebootXend        = 0;
uint16_t  AutoButtonRebootYpos        = 442;
uint16_t  AutoButtonRebootYend        = 0;
uint16_t  AutoButtonRebootWidth       = 40;
uint16_t  AutoButtonRebootHeight      = 36;
uint16_t  AutoButtonRebootXcenter     = 0;
uint16_t  AutoButtonRebootYcenter     = 0;
uint16_t  AutoButtonRebootRadius      = 4;

// Auto - status title and result     = Init, Peer, Send, Delivery
uint16_t  AutoStatusTitleXpos[4]      = {64, 94, 128, 174};
uint16_t  AutoStatusTitleYpos[4]      = {448, 448, 448, 448};
uint16_t  AutoStatusResultXpos[4]     = {63, 92, 127, 174};
uint16_t  AutoStatusResultYpos[4]     = {466, 466, 466, 466};

// Fun - variables
bool      FunRecord                   = false;
uint32_t  FunRecordTime               = 0;
uint32_t  FunRecordTimeBase           = 0;
uint16_t  FunRecordStep               = 0;
uint16_t  FunNeedle                   = 0;
uint16_t  FunNeedleOld                = 0xFFFF;
bool      FunForwardDirection         = false;  // true = direction forward touched, false = direction forward not touched
bool      FunReverseDirection         = false;  // true = direction reverse touched, false = direction reverse not touched
bool      FunForwardSteeringLeft      = false;  // true = forward steering left touched, false = forward steering left not touched
bool      FunForwardSteeringRight     = false;  // true = forward steering right touched, false = forward steering right not touched
bool      FunReverseSteeringLeft      = false;  // true = reverse steering left touched, false = reverse steering left not touched
bool      FunReverseSteeringRight     = false;  // true = reverse steering right touched, false = reversing steering right not touched
byte      FunStopMode                 = 1;      // stop mode, 0 = coast, 1 = brake, 2 = reverse
char      FunStopModeChar[3][14]      = {"    Coast    ", "    Brake    ", "Reverse"};

// Fun - title
uint16_t  FunTitleLeftXpos            = 120;
uint16_t  FunTitleLeftYpos            = 22;
uint16_t  FunTitleMiddleXpos          = 240;
uint16_t  FunTitleMiddleYpos          = 22;
uint16_t  FunTitleRightXpos           = 360;
uint16_t  FunTitleRightYpos           = 22;

// Fun - joypad
uint16_t  FunJoypadXpos[4]            = {20, 20, 260, 260};
uint16_t  FunJoypadXend[4]            = {0, 0, 0, 0};
uint16_t  FunJoypadYpos[4]            = {54, 167, 54, 167};
uint16_t  FunJoypadYend[4]            = {0, 0, 0, 0};
uint16_t  FunJoypadWidth[4]           = {200, 200, 200, 200};
uint16_t  FunJoypadHeight[4]          = {100, 100, 100, 100};
uint16_t  FunJoypadXcenter[4]         = {0, 0, 0, 0};
uint16_t  FunJoypadYcenter[4]         = {0, 0, 0, 0};
uint16_t  FunJoypadRadius[4]          = {16, 16, 16, 16};

// Fun - button, long touch
uint32_t  FunButtonTouchLongTime      = 1000;
uint32_t  FunButtonModeTouchTime      = 0;
byte      FunButtonModeTouchLong      = 0;
uint32_t  FunButtonRecordTouchTime    = 0;
byte      FunButtonRecordTouchLong    = 0;
uint32_t  FunButtonRebootTouchTime    = 0;
byte      FunButtonRebootTouchLong    = 0;

// Fun - button reboot
uint16_t  FunButtonRebootXpos         = 2;
uint16_t  FunButtonRebootXend         = 0;
uint16_t  FunButtonRebootYpos         = 282;
uint16_t  FunButtonRebootYend         = 0;
uint16_t  FunButtonRebootWidth        = 40;
uint16_t  FunButtonRebootHeight       = 36;
uint16_t  FunButtonRebootXcenter      = 0;
uint16_t  FunButtonRebootYcenter      = 0;
uint16_t  FunButtonRebootRadius       = 4;

// Fun - status title and result
uint16_t  FunStatusTitleXpos[4]       = {90, 150, 330, 390};
uint16_t  FunStatusTitleYpos[4]       = {288, 288, 288, 288};
uint16_t  FunStatusResultXpos[4]      = {90, 150, 330, 390};
uint16_t  FunStatusResultYpos[4]      = {306, 306, 306, 306};

// Fun - button record
uint16_t  FunButtonRecordXpos         = 187;
uint16_t  FunButtonRecordXend         = 0;
uint16_t  FunButtonRecordYpos         = 282;
uint16_t  FunButtonRecordYend         = 0;
uint16_t  FunButtonRecordWidth        = 106;
uint16_t  FunButtonRecordHeight       = 36;
uint16_t  FunButtonRecordXcenter      = 0;
uint16_t  FunButtonRecordYcenter      = 0;
uint16_t  FunButtonRecordRadius       = 4;

// Fun - button stop mode
uint16_t  FunButtonStopModeXpos       = 438;
uint16_t  FunButtonStopModeXend       = 0;
uint16_t  FunButtonStopModeYpos       = 282;
uint16_t  FunButtonStopModeYend       = 0;
uint16_t  FunButtonStopModeWidth      = 40;
uint16_t  FunButtonStopModeHeight     = 36;
uint16_t  FunButtonStopModeXcenter    = 0;
uint16_t  FunButtonStopModeYcenter    = 0;
uint16_t  FunButtonStopModeRadius     = 4;

// Status - title and result
char      StatusTitleChar[4][9]       = {"Init", "Peer", "Send","Delivery"};
uint16_t  StatusTitleXpos[4]          = {0, 0, 0, 0};
uint16_t  StatusTitleYpos[4]          = {0, 0, 0, 0};
uint16_t  StatusResultXpos[4]         = {0, 0, 0, 0};
uint16_t  StatusResultYpos[4]         = {0, 0, 0, 0};
uint16_t  StatusResultRadius[4]       = {6, 6, 6, 6};


// **************************************************************************************
// espNOW - callback function
void ESPNowOnSend(const uint8_t *mac_addr, esp_now_send_status_t status)
{
  tft.setTextSize(1);
  if (status != ESP_NOW_SEND_SUCCESS)
  {
    tft.setTextColor(0xF800, 0x0000);
    tft.drawNumber(espNowMsgDataArrayIndex, StatusResultXpos[3], StatusResultYpos[3], 2);
    espNowStatusError++;
  }
  else
  {
    tft.setTextColor(0xFFFE, 0x0000);
    tft.drawNumber(espNowMsgDataArrayIndex, StatusResultXpos[3], StatusResultYpos[3], 2);
  }

  if (espNowMsgDataArrayIndex >= espNowMsgDataArrayToSend && espNowStatusError != 0)
  {
    tft.setTextColor(0xF800, 0x0000);
    tft.drawNumber(espNowMsgDataArrayIndex, StatusResultXpos[3], StatusResultYpos[3], 2);
  }
  if (espNowMsgDataArrayIndex >= espNowMsgDataArrayToSend && espNowStatusError == 0)
  {
    tft.setTextColor(0x0700, 0x0000);
    tft.drawNumber(espNowMsgDataArrayIndex, StatusResultXpos[3], StatusResultYpos[3], 2);
  }
}


// **************************************************************************************
// SPIFFS functions
void SpiffsFormat()
{
  if (SPIFFS.format())
  {AutoOptionStringOnTopTemp = AutoOptionStringOnTopFmtOK;}
  else
  {AutoOptionStringOnTopTemp = AutoOptionStringOnTopFmtER;}
  tft.setFreeFont(&FreeSans9pt7b);
  tft.setTextSize(1);
  for (i = 0; i <= 3; i++)
  {
    tft.setTextColor(0xC618);
    if (i == 0)
    {tft.setTextColor(0xFFE0);}
    tft.drawNumber(AutoOptionFilesValue[i], AutoOptionFilesXpos[i] + AutoOptionFilesWidth[i] / 2, AutoOptionFilesYpos[i] + AutoOptionFilesHeight[i] / 2);
  }
  AutoOptionFilesBase = AutoOptionFilesValue[0];
  AutoOptionStringOnTopFlag = true;
  AutoOptionStringOrValueOnTop();
}

void SpiffsFileList()
{
  tft.fillRect(0, 27 + AutoOptionFilesHeight[0] + 6, width / 16 * 10, height - 28 - AutoOptionFilesHeight[0] - 6 - 44, 0x0000);
  root = SPIFFS.open("/");
  file = root.openNextFile();
  tft.setTextFont(2);
  tft.setTextColor(0xFFFE);
  x = 0;
  y = 62;
  while(file)
  {
    if (x >= AutoOptionFilesBase && x < AutoOptionFilesBase + 30)
    {
      y = y + 12;
      tft.setCursor(8, y, 1);
      tft.println(file.name());
    }
    file = root.openNextFile();
    x++;
  }
  root.close(); 
}

void SpiffsFileRead()
{
  memset(&MotorData, 0, sizeof(MotorData));
  file = SPIFFS.open(SPIFFSFilePathTemp, "r");
  file.read((byte *)&MotorData, file.size());
  if (file.size() == sizeof(MotorData[0]) + sizeof(MotorData[0]) * MotorData[0].LeftSteps)
  {
    AutoOptionStringOnTopTemp = file.size();
    AutoOptionStringOnTopTemp.concat(AutoOptionStringOnTopByte);
    AutoOptionStringOnTopFlag = true;
    AutoOptionStringOrValueOnTop();
  }
  else
  {
    memset(&MotorData, 0, sizeof(MotorData));
    AutoOptionStringOnTopTemp = AutoOptionStringOnTopLoadER;
    AutoOptionStringOnTopFlag = true;
    AutoOptionStringOrValueOnTop();
  }
  file.close();
}

void SpiffsFileWrite()
{
  SpiffsFileWriteCalc();
  file = SPIFFS.open(SPIFFSFilePathTemp, "w");
  file.write((byte *)&MotorData, SPIFFSFilesize);
  file.close();
  file = SPIFFS.open(SPIFFSFilePathTemp, "r");
  if (file == true && file.size() == SPIFFSFilesize)
  {
    AutoOptionStringOnTopTemp = SPIFFSFilesize;
    AutoOptionStringOnTopTemp.concat(AutoOptionStringOnTopByte);
    AutoOptionStringOnTopFlag = true;
    AutoOptionStringOrValueOnTop();
  }
  else
  {
    AutoOptionStringOnTopTemp = AutoOptionStringOnTopSaveER;
    AutoOptionStringOnTopFlag = true;
    AutoOptionStringOrValueOnTop();
  }
  file.close(); 
}

void SpiffsRecordWrite()
{
  SpiffsFileWriteCalc();
  file = SPIFFS.open(SPIFFSFilePathRecord, "w");
  file.write((byte *)&MotorData, SPIFFSFilesize);
  file.close();
  file = SPIFFS.open(SPIFFSFilePathRecord, "r");
  if (file == true && file.size() == SPIFFSFilesize)
  {
    tft.setFreeFont(&FreeSerifBold12pt7b);
    tft.setTextSize(1);
    tft.setTextColor(0x0000);
    tft.drawString("Record", FunButtonRecordXcenter, FunButtonRecordYcenter - 2);
  }
  else
  {
    tft.setFreeFont(&FreeSerifBold12pt7b);
    tft.setTextSize(1);
    tft.setTextColor(0xE000);
    tft.drawString("Record", FunButtonRecordXcenter, FunButtonRecordYcenter - 2);
  }
  file.close(); 
}

void SpiffsFileWriteCalc()
{
  // max. left steps to write, last step where time is not 0
  for (i = espNowMsgDataArrayMax; i > 0; i--)
  {
    if (MotorData[i].LeftTime != 0)
    {
      MotorData[0].LeftSteps = i;
      break;
    }
  }
  // max. right steps to send, last step where time is not 0
  for (i = espNowMsgDataArrayMax; i > 0; i--)
  {
    if (MotorData[i].RightTime != 0)
    {
      MotorData[0].RightSteps = i;
      break;
    }
  }
  if (MotorData[0].LeftSteps > MotorData[0].RightSteps)
  {z = MotorData[0].LeftSteps;}
  else
  {z = MotorData[0].RightSteps;};
  MotorData[0].LeftSteps = z;
  MotorData[0].RightSteps = z;
  SPIFFSFilesize = sizeof(MotorData[0]) + sizeof(MotorData[0]) * z;
  }


// **************************************************************************************
// Auto menu functions
void AutoMenuClear()
{
  for (i = 3; i <= 11; i++)
  {tft.fillRect(AutoMenuValueXcenter[i] - 38, AutoMenuValueYcenter[i] - 15, 76, 30, AutoMenuValueColorBG);}
}

void AutoMenuLeftUpdate()
{
  tft.setFreeFont(&FreeSans9pt7b);
  tft.setTextSize(1);
  tft.setTextColor(0xFFFE);
  AutoMenuValueColorBG = 0x0300;
  AutoMenuClear();
  tft.drawNumber(AutoLeftStep, AutoMenuValueXcenter[3], AutoMenuValueYcenter[3]);
  tft.drawFloat(MotorData[AutoLeftStep].LeftTime / AutoMilliseconds, 1, AutoMenuValueXcenter[5], AutoMenuValueYcenter[5]);
  tft.drawString(AutoDIR[(MotorData[AutoLeftStep].LeftDIR)], AutoMenuValueXcenter[7], AutoMenuValueYcenter[7]);
  tft.drawString(AutoCTR[(MotorData[AutoLeftStep].LeftCTR)], AutoMenuValueXcenter[9], AutoMenuValueYcenter[9]);
  tft.drawNumber(MotorData[AutoLeftStep].LeftPWM, AutoMenuValueXcenter[11], AutoMenuValueYcenter[11]);
  AutoMenuBotsLeft();
}

void AutoMenuRightUpdate()
{
  tft.setFreeFont(&FreeSans9pt7b);
  tft.setTextSize(1);
  tft.setTextColor(0xFFFE);
  AutoMenuValueColorBG = 0x0014;
  AutoMenuClear();
  tft.drawNumber(AutoRightStep, AutoMenuValueXcenter[3], AutoMenuValueYcenter[3]);
  tft.drawFloat(MotorData[AutoRightStep].RightTime / AutoMilliseconds, 1, AutoMenuValueXcenter[5], AutoMenuValueYcenter[5]);
  tft.drawString(AutoDIR[(MotorData[AutoRightStep].RightDIR)], AutoMenuValueXcenter[7], AutoMenuValueYcenter[7]);
  tft.drawString(AutoCTR[(MotorData[AutoRightStep].RightCTR)], AutoMenuValueXcenter[9], AutoMenuValueYcenter[9]);
  tft.drawNumber(MotorData[AutoRightStep].RightPWM, AutoMenuValueXcenter[11], AutoMenuValueYcenter[11]);
  AutoMenuBotsRight();
}

void AutoMenuStepUpdate()
{
  if (AutoMotor == 0)
  {
    AutoMenuValueColorBG = 0x0300;
    AutoMenuClear();
    AutoMenuLeftUpdate();
  }
  if (AutoMotor == 1)
  {
    AutoMenuValueColorBG = 0x0014;
    AutoMenuClear();
    AutoMenuRightUpdate();
  }
}

// Auto - enable options
void AutoMenuToggle()
{
  if (AutoButtonOptionTouchLong == 1)
  {
    AutoButtonOptionTouchTime = millis() + AutoButtonTouchLongTime;
    AutoButtonOptionTouchLong = 2;
  }
  if (AutoButtonOptionTouchLong == 2 && AutoButtonOptionTouchTime <= millis())
  {
    digitalWrite(TFT_BL, 0);
    AutoOptionTitle();
    AutoOption.pushSprite(AutoMenuXpos, AutoMenuYpos);
    AutoOptionClear();
    tft.setFreeFont(&FreeSans9pt7b);
    tft.setTextSize(1);
    tft.setTextColor(0xFFFE);
    tft.drawNumber(AutoOptionFileLoadSlot, AutoOptionTouchXpos[2] + 56, AutoOptionTouchYpos[2] + 16);
    tft.drawNumber(AutoOptionFileSaveSlot, AutoOptionTouchXpos[5] + 56, AutoOptionTouchYpos[5] + 16);
    tft.fillRect(0, 27, width / 16 * 10, height - 28 - 44, 0x0000);
    for (i = 0; i <= 3; i++)
    {
      // Auto - option, file numbers to display
      tft.setFreeFont(&FreeSans9pt7b);
      tft.setTextSize(1);
      tft.setTextColor(0xC618);
      if (i == 0)
      {tft.setTextColor(0xFFE0);}
      tft.fillRoundRect(AutoOptionFilesXpos[i], AutoOptionFilesYpos[i], AutoOptionFilesWidth[i], AutoOptionFilesHeight[i], AutoOptionFilesRadius[i], 0x0186);
      tft.drawRoundRect(AutoOptionFilesXpos[i], AutoOptionFilesYpos[i], AutoOptionFilesWidth[i], AutoOptionFilesHeight[i], AutoOptionFilesRadius[i], 0x9492);
      tft.drawNumber(AutoOptionFilesValue[i], AutoOptionFilesXpos[i] + AutoOptionFilesWidth[i] / 2, AutoOptionFilesYpos[i] + AutoOptionFilesHeight[i] / 2);
    }
    AutoOptionFilesBase = 0;
    SpiffsFileList();
    AutoOptions = 1;
    AutoButtonOptionTouchLong = 3;
    digitalWrite(TFT_BL, 1);
  }
}

void AutoMenuLeft()
{
  if (AutoMotor != 0)
  {
    Auto.deleteSprite();
    AutoMotor = 0;
    AutoCreate();
    Auto.pushSprite(AutoMenuXpos, AutoMenuYpos);
    AutoMenuLeftUpdate();
  }
}

void AutoMenuRight()
{
  if (AutoMotor != 1)
  {
    Auto.deleteSprite();
    AutoMotor = 1;
    AutoCreate();
    Auto.pushSprite(AutoMenuXpos, AutoMenuYpos);
    AutoMenuRightUpdate();
  }
}

void AutoMenuStepMinus()
{
  if (AutoMotor == 0)
  {
    AutoLeftStep = AutoLeftStep - AutoMenuValueIncMulti;
    if (AutoLeftStep > espNowMsgDataArrayMax)
    {AutoLeftStep = AutoLeftStep + 1000;}
    AutoMenuValueOnTopTemp = AutoLeftStep;
    AutoMenuStringOrValueOnTop();
    AutoMenuLeftUpdate();
  }
  if (AutoMotor == 1)
  {
    AutoRightStep = AutoRightStep - AutoMenuValueIncMulti;
    if (AutoRightStep > espNowMsgDataArrayMax)
    {AutoRightStep = AutoRightStep + 1000;}
    AutoMenuValueOnTopTemp = AutoRightStep;
    AutoMenuStringOrValueOnTop();
    AutoMenuRightUpdate();
  }
}

void AutoMenuStepPlus()
{
  if (AutoMotor == 0)
  {
    AutoLeftStep = AutoLeftStep + AutoMenuValueIncMulti;
    if (AutoLeftStep > espNowMsgDataArrayMax)
    {AutoLeftStep = AutoLeftStep - 1000;}
    AutoMenuValueOnTopTemp = AutoLeftStep;
    AutoMenuStringOrValueOnTop();
    AutoMenuLeftUpdate();
  }
  if (AutoMotor == 1)
  {
    AutoRightStep = AutoRightStep + AutoMenuValueIncMulti;
    if (AutoRightStep > espNowMsgDataArrayMax)
    {AutoRightStep = AutoRightStep - 1000;}
    AutoMenuValueOnTopTemp = AutoRightStep;
    AutoMenuStringOrValueOnTop();
    AutoMenuRightUpdate();
  }
}

void AutoMenuTimeMinus()
{
  if (AutoMotor == 0)
  {
    if (MotorData[AutoLeftStep].LeftTime >= 100 * AutoMenuValueIncMulti)
    {
      MotorData[AutoLeftStep].LeftTime = MotorData[AutoLeftStep].LeftTime - 100 * AutoMenuValueIncMulti;
    }
    else
    {
      MotorData[AutoLeftStep].LeftTime = 0;
    }
    AutoMenuFloatOnTopTemp = MotorData[AutoLeftStep].LeftTime / AutoMilliseconds;
    tft.fillRect(AutoMenuValueXcenter[5] - 38, AutoMenuValueYcenter[5] - 15, 76, 30, AutoMenuValueColorBG);
    tft.drawFloat(AutoMenuFloatOnTopTemp, 1, AutoMenuValueXcenter[5], AutoMenuValueYcenter[5]);
    AutoMenuFloatOnTop();
    AutoMenuBotsLeft();
  }
  if (AutoMotor == 1)
  {
    if (MotorData[AutoRightStep].RightTime >= 100 * AutoMenuValueIncMulti)
    {
    MotorData[AutoRightStep].RightTime = MotorData[AutoRightStep].RightTime - 100 * AutoMenuValueIncMulti;
    }
    else
    {
      MotorData[AutoRightStep].RightTime = 0;
    }
    AutoMenuFloatOnTopTemp = MotorData[AutoRightStep].RightTime / AutoMilliseconds;
    tft.fillRect(AutoMenuValueXcenter[5] - 38, AutoMenuValueYcenter[5] - 15, 76, 30, AutoMenuValueColorBG);
    tft.drawFloat(AutoMenuFloatOnTopTemp, 1, AutoMenuValueXcenter[5], AutoMenuValueYcenter[5]);
    AutoMenuFloatOnTop();
    AutoMenuBotsRight();
  }
}

void AutoMenuTimePlus()
{
  if (AutoMotor == 0 && MotorData[AutoLeftStep].LeftTime <= 10000000 - 100 * AutoMenuValueIncMulti)
  {
    MotorData[AutoLeftStep].LeftTime = MotorData[AutoLeftStep].LeftTime + 100 * AutoMenuValueIncMulti;
    AutoMenuFloatOnTopTemp = MotorData[AutoLeftStep].LeftTime / AutoMilliseconds;
    tft.fillRect(AutoMenuValueXcenter[6] - 38, AutoMenuValueYcenter[6] - 15, 76, 30, AutoMenuValueColorBG);
    tft.drawFloat(AutoMenuFloatOnTopTemp, 1, AutoMenuValueXcenter[6], AutoMenuValueYcenter[6]);
    AutoMenuFloatOnTop();
    AutoMenuBotsLeft();
  }
  if (AutoMotor == 1 && MotorData[AutoRightStep].RightTime <= 10000000 - 100 * AutoMenuValueIncMulti)
  {
    MotorData[AutoRightStep].RightTime = MotorData[AutoRightStep].RightTime + 100 * AutoMenuValueIncMulti;
    AutoMenuFloatOnTopTemp = MotorData[AutoRightStep].RightTime / AutoMilliseconds;
    tft.fillRect(AutoMenuValueXcenter[6] - 38, AutoMenuValueYcenter[6] - 15, 76, 30, AutoMenuValueColorBG);
    tft.drawFloat(AutoMenuFloatOnTopTemp, 1, AutoMenuValueXcenter[6], AutoMenuValueYcenter[6]);
    AutoMenuFloatOnTop();
    AutoMenuBotsRight();
  }
}

void AutoMenuDIRMinus()
{
  if (AutoMotor == 0 && MotorData[AutoLeftStep].LeftDIR > 0)
  {
    MotorData[AutoLeftStep].LeftDIR--;
    tft.fillRect(AutoMenuValueXcenter[7] - 38, AutoMenuValueYcenter[7] - 15, 76, 30, AutoMenuValueColorBG);
    tft.drawString(AutoDIR[(MotorData[AutoLeftStep].LeftDIR)], AutoMenuValueXcenter[7], AutoMenuValueYcenter[7]);
  }
  if (AutoMotor == 1 && MotorData[AutoRightStep].RightDIR > 0)
  {
    MotorData[AutoRightStep].RightDIR--;
    tft.fillRect(AutoMenuValueXcenter[7] - 38, AutoMenuValueYcenter[7] - 15, 76, 30, AutoMenuValueColorBG);
    tft.drawString(AutoDIR[(MotorData[AutoRightStep].RightDIR)], AutoMenuValueXcenter[7], AutoMenuValueYcenter[7]);
  }
}

void AutoMenuDIRPlus()
{
  if (AutoMotor == 0 && MotorData[AutoLeftStep].LeftDIR < 4)
  {
    MotorData[AutoLeftStep].LeftDIR++;
    tft.fillRect(AutoMenuValueXcenter[8] - 38, AutoMenuValueYcenter[8] - 15, 76, 30, AutoMenuValueColorBG);
    tft.drawString(AutoDIR[MotorData[AutoLeftStep].LeftDIR], AutoMenuValueXcenter[8], AutoMenuValueYcenter[8]);
  }
  if (AutoMotor == 1 && MotorData[AutoRightStep].RightDIR < 4)
  {
    MotorData[AutoRightStep].RightDIR++;
    tft.fillRect(AutoMenuValueXcenter[8] - 38, AutoMenuValueYcenter[8] - 15, 76, 30, AutoMenuValueColorBG);
    tft.drawString(AutoDIR[MotorData[AutoRightStep].RightDIR], AutoMenuValueXcenter[8], AutoMenuValueYcenter[8]);
  }
}

void AutoMenuCTRMinus()
{
  if (AutoMotor == 0 && MotorData[AutoLeftStep].LeftCTR > 0)
  {
    MotorData[AutoLeftStep].LeftCTR--;
    tft.fillRect(AutoMenuValueXcenter[9] - 38, AutoMenuValueYcenter[9] - 15, 76, 30, AutoMenuValueColorBG);
    tft.drawString(AutoCTR[MotorData[AutoLeftStep].LeftCTR], AutoMenuValueXcenter[9], AutoMenuValueYcenter[9]);
  }
  if (AutoMotor == 1 && MotorData[AutoRightStep].RightCTR > 0)
  {
    MotorData[AutoRightStep].RightCTR--;
    tft.fillRect(AutoMenuValueXcenter[9] - 38, AutoMenuValueYcenter[9] - 15, 76, 30, AutoMenuValueColorBG);
    tft.drawString(AutoCTR[MotorData[AutoRightStep].RightCTR], AutoMenuValueXcenter[9], AutoMenuValueYcenter[9]);
  }
}

void AutoMenuCTRPlus()
{
  if (AutoMotor == 0 && MotorData[AutoLeftStep].LeftCTR < 2)
  {
    MotorData[AutoLeftStep].LeftCTR++;
    tft.fillRect(AutoMenuValueXcenter[10] - 38, AutoMenuValueYcenter[10] - 15, 76, 30, AutoMenuValueColorBG);
    tft.drawString(AutoCTR[MotorData[AutoLeftStep].LeftCTR], AutoMenuValueXcenter[10], AutoMenuValueYcenter[10]);
  }
  if (AutoMotor == 1 && MotorData[AutoRightStep].RightCTR < 2)
  {
    MotorData[AutoRightStep].RightCTR++;
    tft.fillRect(AutoMenuValueXcenter[10] - 38, AutoMenuValueYcenter[10] - 15, 76, 30, AutoMenuValueColorBG);
    tft.drawString(AutoCTR[MotorData[AutoRightStep].RightCTR], AutoMenuValueXcenter[10], AutoMenuValueYcenter[10]);
  }
}

void AutoMenuPWMMinus()
{
  if (AutoMotor == 0 && MotorData[AutoLeftStep].LeftPWM >= 5)
  {
    MotorData[AutoLeftStep].LeftPWM = MotorData[AutoLeftStep].LeftPWM - 5;
    tft.fillRect(AutoMenuValueXcenter[11] - 38, AutoMenuValueYcenter[11] - 15, 76, 30, AutoMenuValueColorBG);
    tft.drawNumber(MotorData[AutoLeftStep].LeftPWM, AutoMenuValueXcenter[11], AutoMenuValueYcenter[11]);
    AutoMenuValueOnTopTemp = MotorData[AutoLeftStep].LeftPWM;
    AutoMenuStringOrValueOnTop();
  }
  if (AutoMotor == 1 && MotorData[AutoRightStep].RightPWM >= 5)
  {
    MotorData[AutoRightStep].RightPWM = MotorData[AutoRightStep].RightPWM - 5;
    tft.fillRect(AutoMenuValueXcenter[11] - 38, AutoMenuValueYcenter[11] - 15, 76, 30, AutoMenuValueColorBG);
    tft.drawNumber(MotorData[AutoRightStep].RightPWM, AutoMenuValueXcenter[11], AutoMenuValueYcenter[11]);
    AutoMenuValueOnTopTemp = MotorData[AutoRightStep].RightPWM;
    AutoMenuStringOrValueOnTop();
  }
}

void AutoMenuPWMPlus()
{
  if (AutoMotor == 0 && MotorData[AutoLeftStep].LeftPWM <= 250)
  {
    MotorData[AutoLeftStep].LeftPWM = MotorData[AutoLeftStep].LeftPWM + 5;
    tft.fillRect(AutoMenuValueXcenter[12] - 38, AutoMenuValueYcenter[12] - 15, 76, 30, AutoMenuValueColorBG);
    tft.drawNumber(MotorData[AutoLeftStep].LeftPWM, AutoMenuValueXcenter[12], AutoMenuValueYcenter[12]);
    AutoMenuValueOnTopTemp = MotorData[AutoLeftStep].LeftPWM;
    AutoMenuStringOrValueOnTop();
  }
  if (AutoMotor == 1 && MotorData[AutoRightStep].RightPWM <= 250)
  {
    MotorData[AutoRightStep].RightPWM = MotorData[AutoRightStep].RightPWM + 5;
    tft.fillRect(AutoMenuValueXcenter[12] - 38, AutoMenuValueYcenter[12] - 15, 76, 30, AutoMenuValueColorBG);
    tft.drawNumber(MotorData[AutoRightStep].RightPWM, AutoMenuValueXcenter[12], AutoMenuValueYcenter[12]);
    AutoMenuValueOnTopTemp = MotorData[AutoRightStep].RightPWM;
    AutoMenuStringOrValueOnTop();
  }
}

void AutoMenuStepCopy()
{
  if (AutoButtonSyncTouchLong == 1)
  {
    AutoButtonSyncTouchTime = millis() + AutoButtonTouchLongTime;
    AutoButtonSyncTouchLong = 2;
  }
  if (AutoButtonSyncTouchLong == 2 && AutoButtonSyncTouchTime <= millis())
  {
    AutoMenuStringOnTopTemp = AutoMenuStringOnTopSync;
    AutoMenuStringOnTopFlag = true;
    AutoMenuStringOrValueOnTop();
    if (AutoMotor == 0)
    {
      memcpy(&MotorData[AutoLeftStep].RightMode, &MotorData[AutoLeftStep].LeftMode, sizeof(MotorData[AutoLeftStep]) / 2);
      AutoMenuBotsRight();
    }
    if (AutoMotor == 1)
    {
      memcpy(&MotorData[AutoRightStep].LeftMode, &MotorData[AutoRightStep].RightMode, sizeof(MotorData[AutoRightStep]) / 2);
      AutoMenuBotsLeft();
    }
    AutoButtonSyncTouchLong = 3;
  }
}

void AutoMenuStepAdd()
{
  if (AutoButtonAddTouchLong == 1)
  {
    AutoButtonAddTouchTime = millis() + AutoButtonTouchLongTime;
    AutoButtonAddTouchLong = 2;
  }
  if (AutoButtonAddTouchLong == 2 && AutoButtonAddTouchTime <= millis())
  {
    AutoMenuStringOnTopTemp = AutoMenuStringOnTopAdd;
    AutoMenuStringOnTopFlag = true;
    AutoMenuStringOrValueOnTop();
    if (AutoMotor == 0)
    {
      if (AutoMotor == 0 && AutoLeftStep < espNowMsgDataArrayMax)
      {
        for (i = espNowMsgDataArrayMax - 1; i > AutoLeftStep; i--)
        {
          memcpy(&MotorData[i + 1].LeftMode, &MotorData[i].LeftMode, sizeof(MotorData[i]) / 2);
        }
      }
      memset (&MotorData[AutoLeftStep + 1].LeftMode, 0, sizeof(MotorData[AutoLeftStep + 1]) / 2);
    }
    if (AutoMotor != 0)
    {
      if (AutoMotor != 0 && AutoRightStep < espNowMsgDataArrayMax)
      {
        for (i = espNowMsgDataArrayMax - 1; i > AutoRightStep; i--)
        {
          memcpy(&MotorData[i + 1].RightMode, &MotorData[i].RightMode, sizeof(MotorData[i]) / 2);
        }
      }
      memset (&MotorData[AutoRightStep + 1].RightMode, 0, sizeof(MotorData[AutoRightStep + 1]) / 2);
    }
    AutoButtonAddTouchLong = 3;
    AutoMenuStepUpdate();
  }
}

void AutoMenuStepDelete()
{
  if (AutoButtonDelTouchLong == 1)
  {
    AutoButtonDelTouchTime = millis() + AutoButtonTouchLongTime;
    AutoButtonDelTouchLong = 2;
  }
  if (AutoButtonDelTouchLong == 2 && AutoButtonDelTouchTime <= millis())
  {
    AutoMenuStringOnTopTemp = AutoMenuStringOnTopDel;
    AutoMenuStringOnTopFlag = true;
    AutoMenuStringOrValueOnTop();
    if (AutoMotor == 0)
    {
      if (AutoMotor == 0 && AutoLeftStep < espNowMsgDataArrayMax)
      {
        for (i = AutoLeftStep; i <= espNowMsgDataArrayMax; i++)
        {
          memcpy(&MotorData[i].LeftMode, &MotorData[i + 1].LeftMode, sizeof(MotorData[i + 1]) / 2);
        }
      }
      memset (&MotorData[espNowMsgDataArrayMax].LeftMode, 0, sizeof(MotorData[espNowMsgDataArrayMax]) / 2);
    }
    if (AutoMotor != 0)
    {
      if (AutoMotor != 0 && AutoRightStep < espNowMsgDataArrayMax)
      {
        for (i = AutoRightStep; i <= espNowMsgDataArrayMax; i++)
        {
          memcpy(&MotorData[i].RightMode, &MotorData[i + 1].RightMode, sizeof(MotorData[i + 1]) / 2);
        }
      }
      memset (&MotorData[espNowMsgDataArrayMax].RightMode, 0, sizeof(MotorData[espNowMsgDataArrayMax]) / 2);
    }
    AutoButtonDelTouchLong = 3;
    AutoMenuStepUpdate();
  }
}

void AutoMenuNow()
{
  ESPNowSend();
}

// Auto - menu, draw value on top
void AutoMenuFloatOnTop()
{
  tft.fillRoundRect(AutoMenuValueXcenter[0] - AutoMenuButtonWidth[0] / 2 + 1, AutoMenuValueYcenter[0] - AutoMenuButtonHeight[0] / 2 + 1, AutoMenuButtonWidth[0] - 2, AutoMenuButtonHeight[0] - 2, AutoMenuButtonRadius[0], 0x0000);
  tft.setFreeFont(&FreeSans9pt7b);
  tft.setTextSize(1);
  tft.setTextColor(0xFFFE);
  tft.drawFloat(AutoMenuFloatOnTopTemp, 1, AutoMenuValueXcenter[0], AutoMenuValueYcenter[0]);
  AutoMenuOnTopFlag = true;
  AutoMenuOnTopTimer = millis() + 1000;
}

// Auto - menu, draw value on top
void AutoMenuStringOrValueOnTop()
{
  tft.fillRoundRect(AutoMenuValueXcenter[0] - AutoMenuButtonWidth[0] / 2 + 1, AutoMenuValueYcenter[0] - AutoMenuButtonHeight[0] / 2 + 1, AutoMenuButtonWidth[0] - 2, AutoMenuButtonHeight[0] - 2, AutoMenuButtonRadius[0], 0x0000);
  tft.setFreeFont(&FreeSans9pt7b);
  tft.setTextSize(1);
  tft.setTextColor(0xFFFE);
  if (AutoMenuStringOnTopFlag == true)
  {
  tft.drawString(AutoMenuStringOnTopTemp, AutoMenuValueXcenter[0], AutoMenuValueYcenter[0]);
  AutoMenuStringOnTopTemp.remove(0);
  AutoMenuStringOnTopFlag = false;
  AutoMenuOnTopFlag = true;
  AutoMenuOnTopTimer = millis() + 1000;
  }
  else
  {
  tft.drawNumber(AutoMenuValueOnTopTemp, AutoMenuValueXcenter[0], AutoMenuValueYcenter[0]);
  AutoMenuOnTopFlag = true;
  AutoMenuOnTopTimer = millis() + 1000;
  } 
}

// Auto - motor left bots
void AutoMenuBotsLeft()
{
  tft.fillRect(0 + 12, 28, width / 16 * 5 - 20 - 12, height - 73, 0x0000);
  Timeline.pushSprite(width / 16 * 5 - 20, 27, 0, 0, 20, 407);
  tft.setTextFont(2);
  tft.setTextSize(1);
  tft.setTextColor(0xFFFE);
  tft.setTextDatum(MC_DATUM);
  e = 0;
  for (i = AutoLeftStep - 5; i <= AutoLeftStep + 5; i++)
  {
    if (i >= 0 && i <= espNowMsgDataArrayMax && MotorData[i].LeftTime != 0)
    {
      y = 48 + e * 36;

      if (MotorData[i].LeftTime >= AutoTimelineBase * 1000 && MotorData[i].LeftTime <= AutoTimelineEnd * 1000)
      {
        z = 28 + ((MotorData[i].LeftTime - AutoTimelineBase * 1000) / 100 * 4) / AutoTimelineAdd;
        v = 100;
      }

      if (MotorData[i].LeftTime < AutoTimelineBase * 1000)
      {
        z = 28;
        v = 30 + 70 * (float(MotorData[i].LeftTime) / (AutoTimelineBase * 1000));
      }
      
      if (MotorData[i].LeftTime > AutoTimelineEnd * 1000)
      {
        z = 433;
        v = 30 + 70 * ((AutoTimelineEnd * 1000) / float(MotorData[i].LeftTime));
      }
      
      tft.drawLine(width / 16 * 5 - 70, y, v, z, 0x07FF);
      tft.fillCircle(width / 16 * 5 - 70, y, 14, 0x0300);
      tft.drawCircle(width / 16 * 5 - 70, y, 14, 0x9492);
      tft.drawNumber(i, width / 16 * 5 - 69, y, 2);
      e++;
    }
  }
}

// Auto - motor right bots
void AutoMenuBotsRight()
{
  tft.fillRect(width / 16 * 5 + 20, 28, width / 16 * 5 - 20 - 12, height - 73, 0x0000);
  Timeline.pushSprite(width / 16 * 5 - 0, 27, 20, 0, 20, 407);
  tft.setTextFont(2);
  tft.setTextSize(1);
  tft.setTextColor(0xFFFE);
  tft.setTextDatum(MC_DATUM);
  e = 0;
  for (i = AutoRightStep - 5; i <= AutoRightStep + 5; i++)
  {
    if (i >= 0 && i <= espNowMsgDataArrayMax && MotorData[i].RightTime != 0)
    {
      y = 48 + e * 36;

      if (MotorData[i].RightTime >= AutoTimelineBase * 1000 && MotorData[i].RightTime <= AutoTimelineEnd * 1000)
      {
        z = 28 + ((MotorData[i].RightTime - AutoTimelineBase * 1000) / 100 * 4) / AutoTimelineAdd;
        v = 100;
      }

      if (MotorData[i].RightTime < AutoTimelineBase * 1000)
      {
        z = 28;
        v = 170 - 70 * (float(MotorData[i].RightTime) / (AutoTimelineBase * 1000));
      }
      
      if (MotorData[i].RightTime > AutoTimelineEnd * 1000)
      {
        z = 433;
        v = 170 - 70 * ((AutoTimelineEnd * 1000) / float(MotorData[i].RightTime));
      }
      
      tft.drawLine(width / 16 * 5 + 70, y, v, z, 0x07FF);
      tft.fillCircle(width / 16 * 5 + 70, y, 14, 0x0014);
      tft.drawCircle(width / 16 * 5 + 70, y, 14, 0x9492);
      tft.drawNumber(i, width / 16 * 5 + 71, y, 2);
      e++;
    }
  }
}

// Auto menu functions table
void (*AutoMenuFunctions[])(void) =
{
  AutoMenuToggle,
  AutoMenuLeft,
  AutoMenuRight,
  AutoMenuStepMinus,
  AutoMenuStepPlus,
  AutoMenuTimeMinus,
  AutoMenuTimePlus,
  AutoMenuDIRMinus,
  AutoMenuDIRPlus,
  AutoMenuCTRMinus,
  AutoMenuCTRPlus,
  AutoMenuPWMMinus,
  AutoMenuPWMPlus,
  AutoMenuStepAdd,
  AutoMenuStepDelete,
  AutoMenuStepCopy,
  AutoMenuNow
};


// **************************************************************************************
// Auto options functions
void AutoOptionClear()
{
  tft.fillRect(AutoOptionTouchXpos[0] + 24, AutoOptionTouchYpos[2] + 1, 112 - 48, 30, 0x0000);
  tft.fillRect(AutoOptionTouchXpos[0] + 24, AutoOptionTouchYpos[5] + 1, 112 - 48, 30, 0x0000);
}

void AutoOptionToggle()
{
  if (AutoButtonOptionTouchLong == 1)
  {
    AutoButtonOptionTouchTime = millis() + AutoButtonTouchLongTime;
    AutoButtonOptionTouchLong = 2;
  }
  if (AutoButtonOptionTouchLong == 2 && AutoButtonOptionTouchTime <= millis())
  {
    digitalWrite(TFT_BL, 0);
    AutoMenuTitle();
    tft.fillRect(0, 27, width / 16 * 10, height - 28 - 44, 0x0000);
    AutoBotsAreaHelpLines();
    Auto.pushSprite(AutoMenuXpos, AutoMenuYpos);
    if (AutoMotor == 0)
    {
      AutoMenuRightUpdate();
      AutoMenuLeftUpdate();
    }
    if (AutoMotor == 1)
    {
      AutoMenuLeftUpdate();
      AutoMenuRightUpdate();
    }
    AutoOptions = 0;
    AutoButtonOptionTouchLong = 3;
    digitalWrite(TFT_BL, 1);
  }
}

void AutoOptionFormat()
{
  if (AutoButtonFormatTouchLong == 1)
  {
    AutoButtonFormatTouchTime = millis() + AutoButtonTouchLongTime;
    AutoButtonFormatTouchLong = 2;
  }
  if (AutoButtonFormatTouchLong == 2 && AutoButtonFormatTouchTime <= millis())
  {
  SpiffsFormat();
  SpiffsFileList();
  AutoButtonFormatTouchLong = 3;
  }
}

void AutoOptionFileLoadSlotMinus()
{
  if (AutoOptionFileLoadSlot > 0)
  {
    AutoOptionFileLoadSlot--;
    tft.fillRect(AutoOptionTouchXpos[0] + 24, AutoOptionTouchYend[i] - 31, 112 - 48, 30, 0x0000);
    tft.setFreeFont(&FreeSans9pt7b);
    tft.setTextSize(1);
    tft.setTextColor(0xFFFE);
    tft.drawNumber(AutoOptionFileLoadSlot, AutoOptionTouchXpos[0] + 56, AutoOptionTouchYpos[i] + 16);
    AutoOptionValueOnTopTemp = AutoOptionFileLoadSlot;
    AutoOptionStringOrValueOnTop();
  }
}

void AutoOptionFileLoadSlotPlus()
{
  if (AutoOptionFileLoadSlot < 19)
  {
    AutoOptionFileLoadSlot++;
    tft.fillRect(AutoOptionTouchXpos[0] + 24, AutoOptionTouchYend[i] - 31, 112 - 48, 30, 0x0000);
    tft.setFreeFont(&FreeSans9pt7b);
    tft.setTextSize(1);
    tft.setTextColor(0xFFFE);
    tft.drawNumber(AutoOptionFileLoadSlot, AutoOptionTouchXpos[0] + 56, AutoOptionTouchYpos[i] + 16);
    AutoOptionValueOnTopTemp = AutoOptionFileLoadSlot;
    AutoOptionStringOrValueOnTop();
  }
}

void AutoOptionFileLoad()
{
  if (AutoButtonLoadTouchLong == 1)
  {
    AutoButtonLoadTouchTime = millis() + AutoButtonTouchLongTime;
    AutoButtonLoadTouchLong = 2;
  }
  if (AutoButtonLoadTouchLong == 2 && AutoButtonLoadTouchTime <= millis())
  {
  SPIFFSFilePathTemp.remove(0);
  SPIFFSFilePathTemp.concat(SPIFFSFilePathBase);
  SPIFFSFilePathTemp.concat(AutoOptionFileLoadSlot);
  SpiffsFileRead();
  AutoButtonLoadTouchLong = 3;
  }
}

void AutoOptionFileSaveSlotMinus()
{
  if (AutoOptionFileSaveSlot > 1)
  {
    AutoOptionFileSaveSlot--;
    tft.fillRect(AutoOptionTouchXpos[0] + 24, AutoOptionTouchYend[i] - 31, 112 - 48, 30, 0x0000);
    tft.setFreeFont(&FreeSans9pt7b);
    tft.setTextSize(1);
    tft.setTextColor(0xFFFE);
    tft.drawNumber(AutoOptionFileSaveSlot, AutoOptionTouchXpos[0] + 56, AutoOptionTouchYpos[i] + 16);
    AutoOptionValueOnTopTemp = AutoOptionFileSaveSlot;
    AutoOptionStringOrValueOnTop();
  }
}

void AutoOptionFileSaveSlotPlus()
{
  if (AutoOptionFileSaveSlot < 19)
  {
    AutoOptionFileSaveSlot++;
    tft.fillRect(AutoOptionTouchXpos[0] + 24, AutoOptionTouchYend[i] - 31, 112 - 48, 30, 0x0000);
    tft.setFreeFont(&FreeSans9pt7b);
    tft.setTextSize(1);
    tft.setTextColor(0xFFFE);
    tft.drawNumber(AutoOptionFileSaveSlot, AutoOptionTouchXpos[0] + 56, AutoOptionTouchYpos[i] + 16);
    AutoOptionValueOnTopTemp = AutoOptionFileSaveSlot;
    AutoOptionStringOrValueOnTop();
  }
}

void AutoOptionFileSave()
{
  if (AutoButtonSaveTouchLong == 1)
  {
    AutoButtonSaveTouchTime = millis() + AutoButtonTouchLongTime;
    AutoButtonSaveTouchLong = 2;
  }
  if (AutoButtonSaveTouchLong == 2 && AutoButtonSaveTouchTime <= millis())
  {
  SPIFFSFilePathTemp.remove(0);
  SPIFFSFilePathTemp.concat(SPIFFSFilePathBase);
  SPIFFSFilePathTemp.concat(AutoOptionFileSaveSlot);
  SpiffsFileWrite();
  SpiffsFileList();
  AutoButtonSaveTouchLong = 3;
  }
}

void AutoOptionResetStepsLeft()
{
  if (AutoButtonResetLTouchLong == 1)
  {
    AutoButtonResetLTouchTime = millis() + AutoButtonTouchLongTime;
    AutoButtonResetLTouchLong = 2;
  }
  if (AutoButtonResetLTouchLong == 2 && AutoButtonResetLTouchTime <= millis())
  {
    AutoOptionStringOnTopTemp = AutoOptionStringOnTopResetL;
    AutoOptionStringOnTopFlag = true;
    AutoOptionStringOrValueOnTop();
    for (i = 0; i <= espNowMsgDataArrayMax; i++)
    {
      memset (&MotorData[i].LeftMode, 0, sizeof(MotorData[i]) / 2);
    }
    AutoLeftStep = 0;
    AutoButtonResetLTouchLong = 3;
  }
}

void AutoOptionResetStepsRight()
{
  if (AutoButtonResetRTouchLong == 1)
  {
    AutoButtonResetRTouchTime = millis() + AutoButtonTouchLongTime;
    AutoButtonResetRTouchLong = 2;
  }
  if (AutoButtonResetRTouchLong == 2 && AutoButtonResetRTouchTime <= millis())
  {
    AutoOptionStringOnTopTemp = AutoOptionStringOnTopResetR;
    AutoOptionStringOnTopFlag = true;
    AutoOptionStringOrValueOnTop();
    for (i = 0; i <= espNowMsgDataArrayMax; i++)
    {
      memset (&MotorData[i].RightMode, 0, sizeof(MotorData[i]) / 2);
    }
    AutoRightStep = 0;
    AutoButtonResetRTouchLong = 3;
  }
}

// Auto - option, draw string / value on top
void AutoOptionStringOrValueOnTop()
{
  tft.fillRoundRect(AutoOptionTouchXpos[0] + 1, AutoOptionTouchYpos[0] + 1, AutoOptionTouchXend[0] - AutoOptionTouchXpos[0] - 2, AutoOptionTouchYend[0] - AutoOptionTouchYpos[0] - 2, 6, 0x0000);
  if (AutoOptionStringOnTopFlag == true)
  {
  tft.setFreeFont(&FreeSans9pt7b);
  tft.setTextSize(1);
  tft.setTextColor(0xFFFE);
  tft.drawString(AutoOptionStringOnTopTemp, 262, 18);
  AutoOptionStringOnTopTemp.remove(0);
  AutoOptionStringOnTopFlag = false;
  AutoOptionOnTopFlag = true;
  AutoOptionOnTopTimer = millis() + 1000;
  }
  else
  {
  tft.setFreeFont(&FreeSans9pt7b);
  tft.setTextSize(1);
  tft.setTextColor(0xFFFE);
  tft.drawNumber(AutoOptionValueOnTopTemp, 262, 18);
  AutoOptionOnTopFlag = true;
  AutoOptionOnTopTimer = millis() + 1000;
  } 
}


// Auto options functions table
void (*AutoOptionFunctions[])(void) =
{
  AutoOptionToggle,
  AutoOptionFormat,
  AutoOptionFileLoadSlotMinus,
  AutoOptionFileLoadSlotPlus,
  AutoOptionFileLoad,
  AutoOptionFileSaveSlotMinus,
  AutoOptionFileSaveSlotPlus,
  AutoOptionFileSave,
  AutoOptionResetStepsLeft,
  AutoOptionResetStepsRight
};


// **************************************************************************************
// Live functions
void MotorLeftRightDefault()
{
  MotorData[0].LeftSteps = 0;
  MotorData[0].LeftTime = 0;
  MotorData[0].RightSteps = 0;
  MotorData[0].RightTime = 0;
}

void LiveLeftPWM()
{
  tft.setTextColor(0xFFFE, 0xA000);
  tft.setTextSize(2);
  if (MotorData[0].LeftCTR <= 1)
  {
    MotorData[0].LeftCTR = 2;
    tft.drawString("   ", LiveButtonLeftXcenter[0], 90, 1);
    tft.drawNumber(MotorData[0].LeftPWM, LiveButtonLeftXcenter[0], 90, 1);
  }
  else if (MotorData[0].LeftCTR == 2)
  {
    MotorData[0].LeftCTR = 1;
    tft.drawString("OFF", LiveButtonLeftXcenter[0], 90, 1);
  }
  ESPNowSend();
}

void LiveLeftPWMMinus()
{
  tft.setTextColor(0xFFFE, 0xA000);
  tft.setTextSize(2);
  if (MotorData[0].LeftPWM >= 5)
  {
    MotorData[0].LeftPWM = MotorData[0].LeftPWM - 5;
    if (MotorData[0].LeftPWM == 0)
    {
      MotorData[0].LeftCTR = 1;
      tft.drawString("OFF", LiveButtonLeftXcenter[0], 90, 1);
    }
    else
    {
      MotorData[0].LeftCTR = 2;
      tft.drawString("   ", LiveButtonLeftXcenter[0], 90, 1);
      tft.drawNumber(MotorData[0].LeftPWM, LiveButtonLeftXcenter[0], 90, 1);
    }
    ESPNowSend();
  }
}

void LiveLeftPWMPlus()
{
  tft.setTextColor(0xFFFE, 0xA000);
  tft.setTextSize(2);
  if (MotorData[0].LeftPWM <= 250)
  {
    MotorData[0].LeftCTR = 2;
    MotorData[0].LeftPWM = MotorData[0].LeftPWM + 5;
    tft.drawString("   ", LiveButtonLeftXcenter[0], 90, 1);
    tft.drawNumber(MotorData[0].LeftPWM, LiveButtonLeftXcenter[0], 90, 1);
    ESPNowSend();
  }
}

// ---

void LiveRightPWM()
{
  tft.setTextColor(0xFFFE, 0xA000);
  tft.setTextSize(2);
  if (MotorData[0].RightCTR <= 1)
  {
    MotorData[0].RightCTR = 2;
    tft.drawString("   ", LiveButtonRightXcenter[0], 90, 1);
    tft.drawNumber(MotorData[0].RightPWM, LiveButtonRightXcenter[0], 90, 1);
  }
  else if (MotorData[0].RightCTR == 2)
  {
    MotorData[0].RightCTR = 1;
    tft.drawString("OFF", LiveButtonRightXcenter[0], 90, 1);
  }
  ESPNowSend();
}

void LiveRightPWMMinus()
{
  tft.setTextColor(0xFFFE, 0xA000);
  tft.setTextSize(2);
  if (MotorData[0].RightPWM >= 5)
  {
    MotorData[0].RightPWM = MotorData[0].RightPWM - 5;
    if (MotorData[0].RightPWM == 0)
    {
      MotorData[0].RightCTR = 1;
      tft.drawString("OFF", LiveButtonRightXcenter[0], 90, 1);
    }
    else
    {
      MotorData[0].RightCTR = 2;
      tft.drawString("   ", LiveButtonRightXcenter[0], 90, 1);
      tft.drawNumber(MotorData[0].RightPWM, LiveButtonRightXcenter[0], 90, 1);
    }
    ESPNowSend();
  }
}

void LiveRightPWMPlus()
{
  tft.setTextColor(0xFFFE, 0xA000);
  tft.setTextSize(2);
  if (MotorData[0].RightPWM <= 250)
  {
    MotorData[0].RightPWM = MotorData[0].RightPWM + 5;
    tft.drawString("   ", LiveButtonRightXcenter[0], 90, 1);
    tft.drawNumber(MotorData[0].RightPWM, LiveButtonRightXcenter[0], 90, 1);
    MotorData[0].RightCTR = 2;
    ESPNowSend();
  }
}


// **************************************************************************************
// Motor functions - used by various modes
void MotorLeftStopOff()
{
  MotorLeftRightDefault();
  MotorData[0].LeftDIR = 0;
  ESPNowSend();
}

void MotorLeftStopCoast()
{
  MotorLeftRightDefault();
  MotorData[0].LeftDIR = 2;
  ESPNowSend();
}

void MotorLeftStopBrake()
{
  MotorLeftRightDefault();
  MotorData[0].LeftDIR = 4;
  ESPNowSend();
}

void MotorLeftForward0()
{
  MotorLeftRightDefault();
  MotorData[0].LeftDIR = 1;
  ESPNowSend();
}

void MotorLeftReverse0()
{
  MotorLeftRightDefault();
  MotorData[0].LeftDIR = 3;
  ESPNowSend();
}

void MotorLeftForward1()
{
  MotorLeftRightDefault();
  MotorData[0].LeftDIR = 1;
  ESPNowSend();
}

void MotorLeftReverse1()
{
  MotorLeftRightDefault();
  MotorData[0].LeftDIR = 3;
  ESPNowSend();
}

// ---

void MotorBothStopOff()
{
  MotorLeftRightDefault();
  MotorData[0].LeftDIR = 0;
  MotorData[0].RightDIR = 0;
  ESPNowSend();
}

void MotorBothStopCoast()
{
  MotorLeftRightDefault();
  MotorData[0].LeftDIR = 2;
  MotorData[0].RightDIR = 2;
  ESPNowSend();
}

void MotorBothStopBrake()
{
  MotorLeftRightDefault();
  MotorData[0].LeftDIR = 4;
  MotorData[0].RightDIR = 4;
  ESPNowSend();
}

void MotorBothLeftForwardRightReverse()
{
  MotorLeftRightDefault();
  MotorData[0].LeftDIR = 1;
  MotorData[0].RightDIR = 3;
  ESPNowSend();
}

void MotorBothLeftReverseRightForward()
{
  MotorLeftRightDefault();
  MotorData[0].LeftDIR = 3;
  MotorData[0].RightDIR = 1;
  ESPNowSend();
}

void MotorBothForward()
{
  MotorLeftRightDefault();
  MotorData[0].LeftDIR = 1;
  MotorData[0].RightDIR = 1;
  ESPNowSend();
}

void MotorBothReverse()
{
  MotorLeftRightDefault();
  MotorData[0].LeftDIR = 3;
  MotorData[0].RightDIR = 3;
  ESPNowSend();
}

// ---

void MotorRightStopOff()
{
  MotorLeftRightDefault();
  MotorData[0].RightDIR = 0;
  ESPNowSend();
}

void MotorRightStopCoast()
{
  MotorLeftRightDefault();
  MotorData[0].RightDIR = 2;
  ESPNowSend();
}

void MotorRightStopBrake()
{
  MotorLeftRightDefault();
  MotorData[0].RightDIR = 4;
  ESPNowSend();
}

void MotorRightForward0()
{
  MotorLeftRightDefault();
  MotorData[0].RightDIR = 3;
  ESPNowSend();
}

void MotorRightReverse0()
{
  MotorLeftRightDefault();
  MotorData[0].RightDIR = 1;
  ESPNowSend();
}

void MotorRightForward1()
{
  MotorLeftRightDefault();
  MotorData[0].RightDIR = 1;
  ESPNowSend();
}

void MotorRightReverse1()
{
  MotorLeftRightDefault();
  MotorData[0].RightDIR = 3;
  ESPNowSend();
}

// Live functions table
void (*LiveFunctions[])(void) =
{
  LiveLeftPWM,
  LiveLeftPWMMinus,
  LiveLeftPWMPlus,
  MotorLeftStopCoast,
  MotorLeftStopBrake,
  MotorLeftForward0,
  MotorLeftReverse0,
  MotorLeftForward1,
  MotorLeftReverse1,
  MotorBothStopCoast,
  MotorBothStopBrake,
  MotorBothLeftForwardRightReverse,
  MotorBothLeftReverseRightForward,
  MotorBothForward,
  MotorBothReverse,
  LiveRightPWM,
  LiveRightPWMMinus,
  LiveRightPWMPlus,
  MotorRightStopCoast,
  MotorRightStopBrake,
  MotorRightForward0,
  MotorRightReverse0,
  MotorRightForward1,
  MotorRightReverse1
};


// **************************************************************************************
// General setup
void setup(void)
{
 // GPIO - init
 pinMode(TFT_BL, OUTPUT);

 // Switch backlight off
 digitalWrite(TFT_BL, 0);

 // TFT_espi - init
 tft.init();

 // Screen - basics
 width = tft.width();
 height = tft.height();

 // Screen - build
 tft.setRotation(0);
 tft.fillScreen(0x0000);
 tft.setPivot(width / 2, height / 2);
 tft.setTextFont(2);
 tft.setTextSize(1);
 tft.setTextColor(0xFFFE, 0x0000);
 tft.setTextDatum(MC_DATUM);


// **************************************************************************************
 // Touchscreen - init FT6206 / FT6336U
 if (!ts.begin(40))
 {
  touch = false;
  tft.drawString("Touchscreen Error", 160, 16, 2);
 } 
 else
 {
  touch = true;
  tft.drawString("Touchscreen Okay", 160, 16, 2);
 }

 // SPIFFS - init 
 if (!SPIFFS.begin(true))
  {
    tft.drawString("SPIFFS Error", 160, 38, 2);
    spiffs = false;
  }
  else
  {
    tft.drawString("SPIFFS Okay", 160, 38, 2);
    spiffs = true;
  }


// **************************************************************************************
 // Main menu
 tft.setFreeFont(&FreeSansBold12pt7b);
 tft.setTextSize(1);
 tft.setTextColor(0x0000);
 tft.fillRoundRect(60, 68, 200, 48, 8, 0xFC00);
 tft.drawRoundRect(60, 68, 200, 48, 8, 0x9492),
 tft.drawString("ESP32 NOW", 160, 92);

 tft.setTextFont(2);
 tft.setTextSize(2);
 tft.setTextColor(0xFFFE);
 
 tft.fillRoundRect(40, 160, 240, 48, 8, 0xA000);
 tft.drawRoundRect(40, 160, 240, 48, 8, 0x9492),
 tft.drawString("Live Control", 160, 184, 1);

 tft.fillRoundRect(40, 260, 240, 48, 8, 0xA000);
 tft.drawRoundRect(40, 260, 240, 48, 8, 0x9492),
 tft.drawString("Auto Control", 160, 284, 1);

 tft.fillRoundRect(40, 360, 240, 48, 8, 0xA000);
 tft.drawRoundRect(40, 360, 240, 48, 8, 0x9492),
 tft.drawString("Fun Control", 160, 384, 1);

 // Switch backlight on
 digitalWrite(TFT_BL, 1);

 while (menu == 0)
 {
  if (ts.touched())
  {
    TS_Point p = ts.getPoint();
    if (p.x >= 40 && p.x <= 280 && p.y >= 160 && p.y <= 208)
    {
      LiveSetup();
      ESPNowSetup();
      menu = 3;
      MotorBothStopOff();
    }
    if (p.x >= 40 && p.x <= 280 && p.y >= 260 && p.y <= 308)
    {
      AutoSetup();
      ESPNowSetup();
      AutoMenuLeftUpdate();
      menu = 4;
      MotorBothStopOff();
    }
      if (p.x >= 40 && p.x <= 280 && p.y >= 360 && p.y <= 408)
    {
      FunSetup();
      ESPNowSetup();
      menu = 5;
      MotorBothStopOff();
    }
  }
 }
}


// **************************************************************************************
// Main loop
void loop()
{
  switch (menu)
  {
    case 3:
    {
      LiveRun();                                          // live control machine, with buttons
    }
    break;
    
    case 4:
    {
      AutoRun();                                          // auto control machine, with program
    }
    break;
    case 5:
    {
      FunRun();                                          // fun control machine, with joypad
    }
    break;
  }
}


// **************************************************************************************
void LiveRun()
{
  // Live - run code
  if (update <= millis())
  {
    update = millis() + 125;                                  // update time in milliseconds
    
    tft.setTextSize(1);
    
    // Touchscreen - retrieve touch point
    if (ts.touched())
    {
      TS_Point tp0 = ts.getPoint(0);
      // LIve - rotate coordinate system to match the screen
      if (tft.getRotation() == 0)
      {
        tp0.x = map(tp0.x, 0, width, 0, width);
        tp0.y = map(tp0.y, 0, height, 0, height);
      }
      if (tft.getRotation() == 2)
      {
        tp0.x = map(tp0.x, 0, width, width, 0);
        tp0.y = map(tp0.y, 0, height, height, 0);
      }
      
      // Live -  left motor item touched ?
      for (i = 0; i < 9; i++)
      {
        if (tp0.x >= LiveButtonLeftXpos[i] && tp0.x <= LiveButtonLeftXend[i] && tp0.y >= LiveButtonLeftYpos[i] && tp0.y <= LiveButtonLeftYend[i])
        {
          if (i > 2)
          {
            tft.fillRect(0, 0, 160, 45, 0x0000);
            tft.setFreeFont(&FreeSerifBold12pt7b);
            tft.setTextSize(1);
            tft.setTextColor(0x0500, 0x0000);
            tft.drawString(LiveButtonLeftChar[i], LiveTitleLeftXpos, LiveTitleLeftYpos);
          }
          tft.setTextFont(2);
          tft.setTextColor(0xFFFE, 0x0000);
          LiveFunctions[i]();
        }
      }

      // Live - both motors item touched ?
      for (i = 0; i < 6; i++)
      {
        if (tp0.x >= LiveButtonBothXpos[i] && tp0.x <= LiveButtonBothXend[i] && tp0.y >= LiveButtonBothYpos[i] && tp0.y <= LiveButtonBothYend[i])
        {
          tft.setFreeFont(&FreeSerifBold12pt7b);
          tft.setTextSize(1);
          tft.fillRect(0, 0, 160, 45, 0x0000);
          tft.fillRect(161, 0, 159, 45, 0x0000);
          tft.setTextColor(0x0500, 0x0000);
          tft.drawString(LiveButtonLeftChar[i + 3], LiveTitleLeftXpos, LiveTitleLeftYpos);
          tft.setTextColor(0x035C, 0x0000);
          tft.drawString(LiveButtonRightChar[i + 3], LiveTitleRightXpos, LiveTitleRightYpos);
          tft.setTextFont(2);
          tft.setTextColor(0xFFFE, 0x0000);
          LiveFunctions[i + 9]();
        }
      }

      // Live - right motor item touched ?
      for (i = 0; i < 9; i++)
      {
        if (tp0.x >= LiveButtonRightXpos[i] && tp0.x <= LiveButtonRightXend[i] && tp0.y >= LiveButtonRightYpos[i] && tp0.y <= LiveButtonRightYend[i])
        {
          if (i > 2)
          {
            tft.fillRect(161, 0, 159, 45, 0x0000);
            tft.setFreeFont(&FreeSerifBold12pt7b);
            tft.setTextSize(1);
            tft.setTextColor(0x035C, 0x0000);
            tft.drawString(LiveButtonRightChar[i], LiveTitleRightXpos, LiveTitleRightYpos);
          }
          tft.setTextFont(2);
          tft.setTextColor(0xFFFE, 0x0000);
          LiveFunctions[i + 15]();
        }
      }

      // Live - reboot esp32 device
      if (tp0.x >= LiveButtonRebootXpos && tp0.x <= LiveButtonRebootXend && tp0.y >= LiveButtonRebootYpos && tp0.y <= LiveButtonRebootYend)
      {
        digitalWrite(TFT_BL, 0);
        ESP.restart();
      }

      // Live - emergency stop all motors
      if (tp0.x >= 208 && tp0.x <= 314 && tp0.y >= 442 && tp0.y <= 478)
      {
        tft.setFreeFont(&FreeSerifBold12pt7b);
        tft.setTextSize(1);
        tft.fillRect(0, 0, 160, 45, 0x0000);
        tft.setTextColor(0x0500, 0x0000);
        tft.drawString("Motor Left", LiveTitleLeftXpos, LiveTitleLeftYpos);
        tft.fillRect(161, 0, 159, 45, 0x0000);
        tft.setTextColor(0x035C, 0x0000);
        tft.drawString("Motor Right", LiveTitleRightXpos, LiveTitleRightYpos);
        tft.setTextFont(2);
        tft.setTextColor(0xFFFE, 0x0000);
        MotorLeftRightDefault();
        MotorData[0].LeftDIR = 0;
        MotorData[0].RightDIR = 0;
        ESPNowSend();
      }
    }
  }
}


// **************************************************************************************
void AutoRun()
{
  // Auto - run code
  if (update <= millis())
  {
    update = millis() + 125;                                  // update time in milliseconds
    
    // Auto - Touchscreen, retrieve touch point
    if (ts.touched())
    {
      TS_Point tp0 = ts.getPoint(0);
      // Auto - rotate coordinate system to match the screen
      if (tft.getRotation() == 0)
      {
        tp0.x = map(tp0.x, 0, width, 0, width);
        tp0.y = map(tp0.y, 0, height, 0, height);
      }
      if (tft.getRotation() == 2)
      {
        tp0.x = map(tp0.x, 0, width, width, 0);
        tp0.y = map(tp0.y, 0, height, height, 0);
      }
    
      // Auto - menu item touched
      for (i = 0; i < 17; i++)
      {
        if (AutoOptions == 0 && tp0.x > AutoMenuTouchXpos[i] && tp0.x < AutoMenuTouchXend[i] && tp0.y > AutoMenuTouchYpos[i] && tp0.y < AutoMenuTouchYend[i])
        {
          tft.setFreeFont(&FreeSans9pt7b);
          tft.setTextColor(0xFFFE);
          tft.setTextSize(1);
          // Auto - menu, enable long touch
          if (i == 0 && AutoButtonOptionTouchLong == 0)
          {AutoButtonOptionTouchLong = 1;}
          if (i == 13 && AutoButtonAddTouchLong == 0)
          {AutoButtonAddTouchLong = 1;}
          if (i == 14 && AutoButtonDelTouchLong == 0)
          {AutoButtonDelTouchLong = 1;}
          if (i == 15 && AutoButtonSyncTouchLong == 0)
          {AutoButtonSyncTouchLong = 1;}
          // Auto - menu, call function
          AutoMenuFunctions[i]();
          break;
        }
      }

      // Auto - option, item touched
      for (i = 0; i < 10; i++)
      {
        if (AutoOptions == 1 && tp0.x > AutoOptionTouchXpos[i] && tp0.x < AutoOptionTouchXend[i] && tp0.y > AutoOptionTouchYpos[i] && tp0.y < AutoOptionTouchYend[i])
        {
          // Auto - menu, enable long touch
          if (i == 0 && AutoButtonOptionTouchLong == 0)
          {AutoButtonOptionTouchLong = 1;}
          if (i == 1 && AutoButtonFormatTouchLong == 0)
          {AutoButtonFormatTouchLong = 1;}
          if (i == 4 && AutoButtonLoadTouchLong == 0)
          {AutoButtonLoadTouchLong = 1;}
          if (i == 7 && AutoButtonSaveTouchLong == 0)
          {AutoButtonSaveTouchLong = 1;}
          if (i == 8 && AutoButtonResetLTouchLong == 0)
          {AutoButtonResetLTouchLong = 1;}
          if (i == 9 && AutoButtonResetRTouchLong == 0)
          {AutoButtonResetRTouchLong = 1;}
          AutoOptionFunctions[i]();
          break;
        }
      }

      // Auto - option, files to display touched
      for (i = 0; i <= 3; i++)
      {
        if (AutoOptions == 1 && tp0.x > AutoOptionFilesXpos[i] && tp0.x < AutoOptionFilesXpos[i] + AutoOptionFilesWidth[i] && tp0.y > AutoOptionFilesYpos[i] && tp0.y < AutoOptionFilesYpos[i] + AutoOptionFilesHeight[i])
        {
          if (AutoOptionFilesValue[i] != AutoOptionFilesBase)
          {
            tft.setFreeFont(&FreeSans9pt7b);
            tft.setTextSize(1);
            for (x = 0; x <= 3; x++)
            {
              if (x != i)
              {
                tft.setTextColor(0xC618, 0x0186);
                tft.drawNumber(AutoOptionFilesValue[x], AutoOptionFilesXpos[x] + AutoOptionFilesWidth[x] / 2, AutoOptionFilesYpos[x] + AutoOptionFilesHeight[x] / 2);
              }
            }
            tft.setTextColor(0xFFE0, 0x0186);
            tft.drawNumber(AutoOptionFilesValue[i], AutoOptionFilesXpos[i] + AutoOptionFilesWidth[i] / 2, AutoOptionFilesYpos[i] + AutoOptionFilesHeight[i] / 2);
            AutoOptionFilesBase = AutoOptionFilesValue[i];
            SpiffsFileList();
          }
          break;
        }
      }

      // Auto - timeline base plus
      for (i = 0; i <= 1; i++)
      {
        if (AutoOptions == 0 && tp0.x > AutoTBaseIncTouchXpos[i] && tp0.x < AutoTBaseIncTouchXend[i] && tp0.y > AutoTBaseIncTouchYpos[i] && tp0.y < AutoTBaseIncTouchYend[i])
        {
          if (AutoTimelineBase <= AutoTimelineBaseIncMax[i])
          {
            if (AutoTimelineBase == 1)
            {AutoTimelineBase = 0;}
            AutoTimelineAdd = 10;
            AutoTimelineBase = AutoTimelineBase + AutoTimelineBaseInc[i];
            AutoTimelineEnd = AutoTimelineBase + 100;
            AutoMenuStringOnTopFlag = true; 
            AutoMenuStringOnTopTemp.concat(AutoMenuStringOnTopBase);
            AutoMenuStringOnTopTemp.concat(AutoTimelineBase);
            AutoMenuStringOrValueOnTop();
            Timeline.deleteSprite();
            TimelineCreate();
            AutoMenuBotsLeft();
            AutoMenuBotsRight();
            break;
          }
        }
      }

      // Auto - timeline base minus
      for (i = 0; i <= 1; i++)
      {
        if (AutoOptions == 0 && tp0.x > AutoTBaseDecTouchXpos[i] && tp0.x < AutoTBaseDecTouchXend[i] && tp0.y > AutoTBaseDecTouchYpos[i] && tp0.y < AutoTBaseDecTouchYend[i])
        {
          if (AutoTimelineBase >= AutoTimelineBaseDecMin[i])// && AutoTimelineBase <= AutoTimelineBaseDecMaxValue[i])
          {
            if (AutoTimelineBase == 1)
            {AutoTimelineBase = 0;}
            AutoTimelineAdd = 10;
            AutoTimelineBase = AutoTimelineBase - AutoTimelineBaseDec[i];
            AutoTimelineEnd = AutoTimelineBase + 100;
            AutoMenuStringOnTopFlag = true; 
            AutoMenuStringOnTopTemp.concat(AutoMenuStringOnTopBase);
            AutoMenuStringOnTopTemp.concat(AutoTimelineBase);
            AutoMenuStringOrValueOnTop();
            Timeline.deleteSprite();
            TimelineCreate();
            AutoMenuBotsLeft();
            AutoMenuBotsRight();
            break;
          }
        }
      }

      // Auto - timeline value multiplicator x 1, 10, 100, 1000
      for (i = 0; i <= 3; i++)
      {
        if (AutoOptions == 0 && tp0.x > AutoTimeMultiTouchXpos[i] && tp0.x < AutoTimeMultiTouchXend[i] && tp0.y > AutoTimeMultiTouchYpos[i] && tp0.y < AutoTimeMultiTouchYend[i])
        {
          AutoTimelineBase = 0;
          AutoTimelineAdd = AutoTimelineInc[i];
          AutoTimelineEnd = AutoTimelineAdd * 10;
          AutoMenuStringOnTopFlag = true; 
          AutoMenuStringOnTopTemp.concat(AutoMenuStringOnTopTime);
          AutoMenuStringOnTopTemp.concat(AutoTimelineBase);
          AutoMenuStringOrValueOnTop();
          Timeline.deleteSprite();
          TimelineCreate();
          AutoMenuBotsLeft();
          AutoMenuBotsRight();
          break;
        }
      }

      // Auto - menu value multiplicator x 1, 10, 100, 1000
      for (i = 0; i <= 3; i++)
      {
        if (AutoOptions == 0 && tp0.x > AutoValueMultiTouchXpos[i] && tp0.x < AutoValueMultiTouchXend[i] && tp0.y > AutoValueMultiTouchYpos[i] && tp0.y < AutoValueMultiTouchYend[i])
        {
          AutoMenuValueIncMulti = AutoMultiplicator[i];
          AutoMenuStringOnTopFlag = true; 
          AutoMenuStringOnTopTemp.concat(AutoMenuStringOnTopValue);
          AutoMenuStringOnTopTemp.concat(AutoMenuValueIncMulti);
          AutoMenuStringOrValueOnTop();
          break;
        }
      }

      // Auto - reboot device
      if (tp0.x >= AutoButtonRebootXpos && tp0.x <= AutoButtonRebootXend && tp0.y >= AutoButtonRebootYpos && tp0.y <= AutoButtonRebootYend)
      {
        if (AutoButtonRebootTouchLong == 0)
        {AutoButtonRebootTouchLong = 1;}
        if (AutoButtonRebootTouchLong == 1)
        {
          AutoButtonRebootTouchTime = millis() + AutoButtonTouchLongTime;
          AutoButtonRebootTouchLong = 2;
        }
        if (AutoButtonRebootTouchLong == 2 && AutoButtonRebootTouchTime <= millis())
        {
          digitalWrite(TFT_BL, 0);
          ESP.restart();
        }
      }
    }
    else
    {
      // Auto - menu, clear long touch
      AutoButtonAddTouchLong = 0;
      AutoButtonDelTouchLong = 0;
      AutoButtonSyncTouchLong = 0;
      AutoButtonOptionTouchLong = 0;
      AutoButtonFormatTouchLong = 0;
      AutoButtonLoadTouchLong = 0;
      AutoButtonSaveTouchLong = 0;
      AutoButtonResetLTouchLong = 0;
      AutoButtonResetRTouchLong = 0;
      AutoButtonRebootTouchLong = 0;
    }
  }

  // Auto - menu, remove string / value from top and restore button
  if (AutoOptions == 0 && AutoMenuOnTopFlag == true && AutoMenuOnTopTimer <= millis())
  {
    tft.fillRoundRect(AutoMenuValueXcenter[0] - AutoMenuButtonWidth[0] / 2 + 1, AutoMenuValueYcenter[0] - AutoMenuButtonHeight[0] / 2 + 1, AutoMenuButtonWidth[0] - 2, AutoMenuButtonHeight[0] - 2, AutoMenuButtonRadius[0], 0xA000);
    tft.setFreeFont(&FreeMonoBold18pt7b);
    tft.setTextSize(1);
    tft.setTextColor(0xC618);
    tft.drawString(AutoMenuButtonChar[0], AutoMenuValueXcenter[0], AutoMenuValueYcenter[0] - 1);
    AutoMenuOnTopFlag = false;
    AutoMenuStringOnTopFlag = false;
  }

  // Auto - option, remove string / value from top and restore button
  if (AutoOptions == 1 && AutoOptionOnTopFlag == true && AutoOptionOnTopTimer <= millis())
  {
    tft.fillRoundRect(AutoOptionTouchXpos[0] + 1, AutoOptionTouchYpos[0] + 1, AutoOptionTouchXend[0] - AutoOptionTouchXpos[0] - 2, AutoOptionTouchYend[0] - AutoOptionTouchYpos[0] - 2, 6, 0xA000);
    tft.setFreeFont(&FreeMonoBold18pt7b);
    tft.setTextSize(1);
    tft.setTextColor(0xC618);
    tft.drawString(AutoMenuButtonChar[0], 262, 18 - 1);
    AutoOptionOnTopFlag = false;
    AutoOptionStringOnTopFlag = false;
  }
}


// **************************************************************************************
void FunRun()
{
  // Fun - run code
  tft.setTextSize(1);

  if (ts.touched())
  {
    TS_Point tp0 = ts.getPoint(0);
    TS_Point tp1 = ts.getPoint(1);
    // Fun - rotate coordinate system to match the screen
    if (tft.getRotation() == 1)
    {
      x = tp0.x;
      tp0.x = tp0.y;
      tp0.y = height - x;
      x = tp1.x;
      tp1.x = tp1.y;
      tp1.y = height - x;
    }
    if (tft.getRotation() == 3)
    {
      x = tp0.x;
      tp0.x = width - tp0.y;
      tp0.y = x;
      x = tp1.x;
      tp1.x = width - tp1.y;
      tp1.y = x;
    }

    // Fun - reboot esp32 device
    if (tp0.x >= FunButtonRebootXpos && tp0.x <= FunButtonRebootXend && tp0.y >= FunButtonRebootYpos && tp0.y <= FunButtonRebootYend)
    {
      if (FunButtonRebootTouchLong == 0)
      {FunButtonRebootTouchLong = 1;}
      if (FunButtonRebootTouchLong == 1)
      {
        FunButtonRebootTouchTime = millis() + FunButtonTouchLongTime;
        FunButtonRebootTouchLong = 2;
      }
      if (FunButtonRebootTouchLong == 2 && FunButtonRebootTouchTime <= millis())
      {
        digitalWrite(TFT_BL, 0);
        ESP.restart();
      }
    }

    // Fun - stop mode, 0 = brake, 1 = coast, 2 = opposite
    if (tp0.x >= FunButtonStopModeXpos && tp0.x <= FunButtonStopModeXend && tp0.y >= FunButtonStopModeYpos && tp0.y <= FunButtonStopModeYend)
    {
      if (FunButtonModeTouchLong == 0)
      {FunButtonModeTouchLong = 1;}
      if (FunButtonModeTouchLong == 1)
      {
        FunButtonModeTouchTime = millis() + FunButtonTouchLongTime;
        FunButtonModeTouchLong = 2;
      }
      if (FunButtonModeTouchLong == 2 && FunButtonModeTouchTime <= millis())
      {
        if (FunStopMode < 2)
        {FunStopMode++;}
        else
        {FunStopMode = 0;}
        tft.setFreeFont(&FreeSerifBold12pt7b);
        tft.setTextSize(1);
        tft.setTextColor(0x6018, 0x0000);
        tft.drawString(FunStopModeChar[FunStopMode], FunTitleMiddleXpos, FunTitleMiddleYpos, 1);
        FunButtonModeTouchLong = 3;
      }
    }

    // Fun - start / stop record
    if (tp0.x >= FunButtonRecordXpos && tp0.x <= FunButtonRecordXend && tp0.y >= FunButtonRecordYpos && tp0.y <= FunButtonRecordYend)
    {
      if (FunButtonRecordTouchLong == 0)
      {FunButtonRecordTouchLong = 1;}
      if (FunButtonRecordTouchLong == 1)
      {
        FunButtonRecordTouchTime = millis() + FunButtonTouchLongTime;
        FunButtonRecordTouchLong = 2;
      }

      if (FunButtonRecordTouchLong == 2 && FunButtonRecordTouchTime <= millis())
      {
        FunRecord ^= 1 << 0;  // toggle bit
        if (FunRecord == true)
        {
          FunRecordStep = 0;  // reset step counter
          FunNeedle = 460;    // reset needle degree
          FunNeedleOld = 460;
          for (i = 1; i <= espNowMsgDataArrayMax; i++)
          {
            memset(&MotorData[i].LeftMode, 0, sizeof(MotorData[i]));
          }

          tft.setFreeFont(&FreeSerifBold12pt7b);
          tft.setTextSize(1);
          tft.setTextColor(0x0000);
          tft.fillRoundRect(FunButtonRecordXpos, FunButtonRecordYpos, FunButtonRecordWidth, FunButtonRecordHeight, FunButtonRecordRadius, 0xA000);
          tft.drawRoundRect(FunButtonRecordXpos, FunButtonRecordYpos, FunButtonRecordWidth, FunButtonRecordHeight, FunButtonRecordRadius, 0x8410);
          tft.drawString("Record", FunButtonRecordXcenter, FunButtonRecordYcenter - 2);
          FunRecordTimeBase = millis(); // set timebase
          delay(1);
        }
        if (FunRecord == false)
        {
          // Fun - set array index 0 because it is used by Fun Control
          MotorData[0].LeftMode = 4;                  // mode auto, left motor
          MotorData[0].RightMode = 4;                 // mode auto, right motor
          MotorData[0].LeftSteps = FunRecordStep;     // max. left steps in this record
          MotorData[0].RightSteps = FunRecordStep;    // max. right steps in this record
          MotorData[0].LeftDIR = 2;                   // stop coast, left motor
          MotorData[0].RightDIR = 2;                  // stop coast, right motor
          MotorData[0].LeftCTR = 1;                   // control fix, left motor, no pwm
          MotorData[0].RightCTR = 1;                  // control fix, right motor, no pwm
          tft.fillRect(width / 2 - 20, height / 2 - 20, 40, 40, 0x0000);
          tft.fillRoundRect(FunButtonRecordXpos, FunButtonRecordYpos, FunButtonRecordWidth, FunButtonRecordHeight, FunButtonRecordRadius, 0xFC00);
          tft.drawRoundRect(FunButtonRecordXpos, FunButtonRecordYpos, FunButtonRecordWidth, FunButtonRecordHeight, FunButtonRecordRadius, 0x8410);
          SpiffsRecordWrite();
        }
        FunButtonRecordTouchLong = 3;
      }
    }
    
    // Fun - steering touchpoints should be always located at tp1
    tp0.x = abs(tp0.x);
    tp0.y = abs(tp0.y);
    tp0.z = abs(tp0.z);
    tp1.x = abs(tp1.x);
    tp1.y = abs(tp1.y);
    tp1.z = abs(tp1.z);
    
    if (tp0.x >= width / 2)
    {
      x = tp1.x;
      y = tp1.y;
      z = tp1.z;
      tp1.x = tp0.x;
      tp1.y = tp0.y;
      tp1.z = tp0.z;
      tp0.x = x;
      tp0.y = y;
      tp0.z = z;
    }

    /*
    // Fun - visualize touchpoints tp0 and tp1
    tft.fillRect(width / 2 - 20, 70, 40, 60, 0x0000);
    tft.drawNumber(tp0.x, width / 2, 80);
    tft.drawNumber(tp0.y, width / 2, 100);
    tft.drawNumber(tp0.z, width / 2, 120);
    tft.fillRect(width / 2 - 20, 190, 40, 60, 0x0000);
    tft.drawNumber(tp1.x, width / 2, 200);
    tft.drawNumber(tp1.y, width / 2, 220);
    tft.drawNumber(tp1.z, width / 2, 240);

    // Fun - check if both touchpoints on the left side
    if (tp0.x <= width / 2 && tp1.x <= width / 2 && FunNeedle != 360)
    {
      MotorBothStopCoast();
      tft.fillRect(width / 2 - 20, height / 2 - 20, 40, 40, 0x0000);
      FunNeedle = 360;
    }
    // Fun - check if both touchpoints on the right side
    if (tp0.x > width / 2 && tp1.x > width / 2 && FunNeedle != 360)
    {
      MotorBothStopCoast();
      tft.fillRect(width / 2 - 20, height / 2 - 20, 40, 40, 0x0000);
      FunNeedle = 360;
    }
    */

    // Fun - check if forward direction touched
    if (tp0.x >= FunJoypadXpos[0] && tp0.x <= FunJoypadXend[0] && tp0.y >= FunJoypadYpos[0] && tp0.y <= FunJoypadYend[0])
    {
      FunForwardDirection = true;                 // set forward direction flag
      // Fun - forward steering right
      if (tp1.x >= FunJoypadXpos[2] && tp1.x <= FunJoypadXend[2] && tp1.y >= FunJoypadYpos[2] && tp1.y <= FunJoypadYend[2])
      {
        if (FunStopMode == 0 && FunNeedle != 30)  // forward steering right coast
        {
          MotorLeftForward0();
          MotorRightStopCoast();
          FunNeedle = 30;
          Needle.pushRotated(FunNeedle);
          FunForwardSteeringRight = true;         // set forward steering right flag
        }
        if (FunStopMode == 1 && FunNeedle != 60)  // forward steering right brake
        {
          MotorLeftForward0();
          MotorRightStopBrake();
          FunNeedle = 60;
          Needle.pushRotated(FunNeedle);
          FunForwardSteeringRight = true;         // set forward steering right flag
        }
        if (FunStopMode == 2 && FunNeedle != 90)  // forward steering right reverse
        {
          MotorLeftForward0();
          MotorRightReverse1();
          FunNeedle = 90;
          Needle.pushRotated(FunNeedle);
          FunForwardSteeringRight = true;         // set forward steering right flag
        }
      }
      else
      {
        FunForwardSteeringRight = false;          // clear forward steering right flag
      }

      // Fun - forward steering left
      if (tp1.x >= FunJoypadXpos[3] && tp1.x <= FunJoypadXend[3] && tp1.y >= FunJoypadYpos[3] && tp1.y <= FunJoypadYend[3])
      {
        if (FunStopMode == 0 && FunNeedle != 330) // forward steering left coast
        {
          MotorRightForward1();
          MotorLeftStopCoast();
          FunNeedle = 330;
          Needle.pushRotated(FunNeedle);
          FunForwardSteeringLeft = true;          // set forward steering left flag
        }
        if (FunStopMode == 1 && FunNeedle != 300) // forward steering left brake
        {
          MotorRightForward1();
          MotorLeftStopBrake();
          FunNeedle = 300;
          Needle.pushRotated(FunNeedle);
          FunForwardSteeringLeft = true;          // set forward steering left flag
        }
        if (FunStopMode == 2 && FunNeedle != 270) // forward steering left reverse
        {
          MotorRightForward1();
          MotorLeftReverse1();
          FunNeedle = 270;
          Needle.pushRotated(FunNeedle);
          FunForwardSteeringLeft = true;          // set forward steering left flag
        }
      }
      else
      {
        FunForwardSteeringLeft = false;           // clear forward steering left flag
      }
      // Fun - forward steering none
      if (FunForwardDirection == true && FunForwardSteeringLeft == false && FunForwardSteeringRight == false && FunNeedle != 0)
      {
        MotorBothForward();
        FunNeedle = 0;
        Needle.pushRotated(FunNeedle);
      }
    }
    // Fun - forward direction none
    else
    {
      FunForwardDirection = false;
      FunForwardSteeringLeft = false;
      FunForwardSteeringRight = false;
    }

// ---

    // Fun - check if reverse direction touched
    if (tp0.x >= FunJoypadXpos[1] && tp0.x <= FunJoypadXend[1] && tp0.y >= FunJoypadYpos[1] && tp0.y <= FunJoypadYend[1])
    {
      FunReverseDirection = true;                 // set reverse direction flag
      // Fun - reverse steering right
      if (tp1.x >= FunJoypadXpos[2] && tp1.x <= FunJoypadXend[2] && tp1.y >= FunJoypadYpos[2] && tp1.y <= FunJoypadYend[2])
      {
        if (FunStopMode == 0 && FunNeedle != 150) // reverse steering right coast
        {
          MotorLeftReverse0();
          MotorRightStopCoast();
          FunNeedle = 150;
          Needle.pushRotated(FunNeedle);
          FunReverseSteeringRight = true;         // set reverse steering right flag
        }
        if (FunStopMode == 1 && FunNeedle != 120) // reverse steering right brake
        {
          MotorLeftReverse0();
          MotorRightStopBrake();
          FunNeedle = 120;
          Needle.pushRotated(FunNeedle);
          FunReverseSteeringRight = true;         // set reverse steering right flag
        }
        if (FunStopMode == 2 && FunNeedle != 90)  // reverse steering right reverse
        {
          MotorLeftReverse0();
          MotorRightForward1();
          FunNeedle = 90;
          Needle.pushRotated(FunNeedle);
          FunReverseSteeringRight = true;         // set reverse steering right flag
        }
      }
      else
      {
        FunReverseSteeringRight = false;          // clear reverse steering right flag
      }
      
      // Fun - reverse steering left
      if (tp1.x >= FunJoypadXpos[3] && tp1.x <= FunJoypadXend[3] && tp1.y >= FunJoypadYpos[3] && tp1.y <= FunJoypadYend[3])
      {
        if (FunStopMode == 0 && FunNeedle != 210) // reverse steering left coast
        {
        MotorRightReverse1();
        MotorLeftStopCoast();
        FunNeedle = 210;
        Needle.pushRotated(FunNeedle);
        FunReverseSteeringLeft = true;            // set reverse steering left flag
        }
        if (FunStopMode == 1 && FunNeedle != 240) // reverse steering left brake
        {
          MotorRightReverse1();
          MotorLeftStopBrake();
          FunNeedle = 240;
          Needle.pushRotated(FunNeedle);
          FunReverseSteeringLeft = true;          // set reverse steering left flag
        }
        if (FunStopMode == 2 && FunNeedle != 270) // reverse steering left reverse
        {
          MotorRightReverse1();
          MotorLeftForward1();
          FunNeedle = 270;
          Needle.pushRotated(FunNeedle);
          FunReverseSteeringLeft = true;          // set reverse steering left flag
        }
      }
      else
      {
        FunReverseSteeringLeft = false;           // clear reverse steering left flag
      }
      // Fun - reverse steering none
      if (FunReverseDirection == true && FunReverseSteeringLeft == false && FunReverseSteeringRight == false && FunNeedle != 180)
      {
        MotorBothReverse();
        FunNeedle = 180;
        Needle.pushRotated(FunNeedle);
      }
    }
    // Fun - reverse direction none
    else
    {
      FunReverseDirection = false;
      FunReverseSteeringLeft = false;
      FunReverseSteeringRight = false;
    }
  }
  else
  {
    if (FunForwardDirection == true || FunReverseDirection == true)
    {
      FunForwardDirection = false;
      FunReverseDirection = false;
    }
    // Fun - buttons, clear long touch
    FunButtonModeTouchLong = 0;
    FunButtonRecordTouchLong = 0;
    FunButtonRebootTouchLong = 0;
  }
  
  if (FunForwardDirection == false && FunReverseDirection == false && FunNeedle != 360)
  {
    MotorBothStopOff();
    FunNeedle = 360;
    tft.fillRect(width / 2 - 20, height / 2 - 20, 40, 40, 0x0000);
  }
  
  // Fun - record
    if (FunRecord == true && FunNeedle != FunNeedleOld)
    {
      FunNeedleOld = FunNeedle;
      if (FunRecordStep < espNowMsgDataArrayMax)
      {
        FunRecordStep++;
        FunRecordTime = ((millis() - FunRecordTimeBase + 99) / 100) * 100;  // round up to next 100
        MotorData[FunRecordStep].LeftTime = FunRecordTime;
        MotorData[FunRecordStep].RightTime = FunRecordTime;
        MotorData[FunRecordStep].LeftCTR = 1;                       // fix control, no pwm
        MotorData[FunRecordStep].RightCTR = 1;                      // fix control, no pwm
        MotorData[FunRecordStep].LeftDIR = MotorData[0].LeftDIR;    // copy current dir, left motor
        MotorData[FunRecordStep].RightDIR = MotorData[0].RightDIR;  // copy current dir, right motor
      }
      else
      {
        // Fun - set array index 0 because it is used by Fun Control
        MotorData[0].LeftMode = 4;                  // mode auto, left motor
        MotorData[0].RightMode = 4;                 // mode auto, right motor
        MotorData[0].LeftSteps = FunRecordStep;     // max. left steps in this record
        MotorData[0].RightSteps = FunRecordStep;    // max. right steps in this record
        MotorData[0].LeftDIR = 2;                   // stop coast, left motor
        MotorData[0].RightDIR = 2;                  // stop coast, right motor
        MotorData[0].LeftCTR = 1;                   // control fix, left motor, no pwm
        MotorData[0].RightCTR = 1;                  // control fix, right motor, no pwm
        // Fun - set last record array index to stop both motors
        FunRecordStep++;
        FunRecordTime = ((millis() - FunRecordTimeBase + 99) / 100) * 100;  // round up to next 100
        MotorData[FunRecordStep].LeftTime = FunRecordTime;
        MotorData[FunRecordStep].RightTime = FunRecordTime;
        MotorData[FunRecordStep].LeftDIR = 2;       // stop coast, left motor
        MotorData[FunRecordStep].RightDIR = 2;      // stop coast, right motor
        FunRecord = false;
        tft.fillRect(width / 2 - 20, height / 2 - 20, 40, 40, 0x0000);
        tft.fillRoundRect(FunButtonRecordXpos, FunButtonRecordYpos, FunButtonRecordWidth, FunButtonRecordHeight, FunButtonRecordRadius, 0xFC00);
        tft.drawRoundRect(FunButtonRecordXpos, FunButtonRecordYpos, FunButtonRecordWidth, FunButtonRecordHeight, FunButtonRecordRadius, 0x8410);;
        SpiffsRecordWrite();
      }
    }
}

 
// **************************************************************************************
void LiveSetup()
{
  // Live Screen - switch backlight off
  digitalWrite(TFT_BL, 0);
  
  // Live Screen - basics
  tft.setRotation(0);
  tft.fillScreen(0x0000);
  tft.setTextFont(2);
  tft.setTextSize(2);
  tft.setTextColor(0xFFFE);
  tft.setTextDatum(MC_DATUM);

  // Live Screen - some calculations
  for (i = 0; i < 9; i++)
  {
    LiveButtonLeftXend[i] = LiveButtonLeftXpos[i] + LiveButtonLeftWidth[i];
    LiveButtonLeftYend[i] = LiveButtonLeftYpos[i] + LiveButtonLeftHeight[i];
    LiveButtonRightXend[i] = LiveButtonRightXpos[i] + LiveButtonRightWidth[i];
    LiveButtonRightYend[i] = LiveButtonRightYpos[i] + LiveButtonRightHeight[i];
    LiveButtonLeftXcenter[i] = LiveButtonLeftXpos[i] + LiveButtonLeftWidth[i] / 2 - 1;
    LiveButtonLeftYcenter[i] = LiveButtonLeftYpos[i] + LiveButtonLeftHeight[i] / 2;
    LiveButtonRightXcenter[i] = LiveButtonRightXpos[i] + LiveButtonRightWidth[i] / 2 - 1;
    LiveButtonRightYcenter[i] = LiveButtonRightYpos[i] + LiveButtonRightHeight[i] / 2;
  }
  LiveButtonLeftXcenter[0] = LiveButtonLeftXpos[0] + LiveButtonLeftWidth[0] / 2 + 1;
  LiveButtonLeftYcenter[0] = LiveButtonLeftYpos[0] + LiveButtonLeftHeight[0] / 3 - 1;
  LiveButtonRightXcenter[0] = LiveButtonRightXpos[0] + LiveButtonRightWidth[0] / 2 + 1;
  LiveButtonRightYcenter[0] = LiveButtonRightYpos[0] + LiveButtonRightHeight[0] / 3 - 1;

  LiveButtonRebootXend = LiveButtonRebootXpos + LiveButtonRebootWidth;
  LiveButtonRebootYend = LiveButtonRebootYpos + LiveButtonRebootHeight;
  LiveButtonRebootXcenter = LiveButtonRebootXpos + LiveButtonRebootWidth / 2;
  LiveButtonRebootYcenter = LiveButtonRebootYpos + LiveButtonRebootHeight / 2;

  LiveButtonOffXend = LiveButtonOffXpos + LiveButtonOffWidth;
  LiveButtonOffYend = LiveButtonOffYpos + LiveButtonOffHeight;
  LiveButtonOffXcenter = LiveButtonOffXpos + LiveButtonOffWidth / 2;
  LiveButtonOffYcenter = LiveButtonOffYpos + LiveButtonOffHeight / 2;

  // Live Screen - horizontal top line
  tft.drawLine(0, 46, width, 46, 0x8410);

  // Live Screen - vertical centre line
  tft.drawLine(160, 12, 160, 54, 0x8410);

  // Live Screen - horizontal bottom line
  tft.drawLine(0, height - 45, width, height - 45, 0x8410);

  // Live Screen - vertical left bottom line
  tft.drawLine(48, height - 45, 48, height, 0x8410);

  // Live Screen - title
  tft.setFreeFont(&FreeSerifBold12pt7b);
  tft.setTextSize(1);
  tft.setTextColor(0x0500, 0x0000);
  tft.drawString("Motor Left", LiveTitleLeftXpos, LiveTitleLeftYpos, 1);
  tft.setTextColor(0x035C, 0x0000);
  tft.drawString("Motor Right", LiveTitleRightXpos, LiveTitleRightYpos, 1);

  // Live Screen - menu left items position
  tft.setTextFont(2);
  tft.setTextSize(2);
  tft.setTextColor(0xFFFE);
  tft.fillRoundRect(LiveButtonLeftXpos[0], LiveButtonLeftYpos[0], LiveButtonLeftWidth[0], LiveButtonLeftHeight[0], LiveButtonLeftRadius[0], 0xA000);
  tft.drawRoundRect(LiveButtonLeftXpos[0], LiveButtonLeftYpos[0], LiveButtonLeftWidth[0], LiveButtonLeftHeight[0], LiveButtonLeftRadius[0], 0x9492);
  tft.drawString(LiveButtonLeftChar[0], LiveButtonLeftXcenter[0], LiveButtonLeftYcenter[0], 1);
  
  tft.setTextFont(2);
  tft.setTextSize(2);
  tft.setTextColor(0xFFFE);
  tft.fillRoundRect(LiveButtonLeftXpos[1], LiveButtonLeftYpos[1], LiveButtonLeftWidth[1], LiveButtonLeftHeight[1], LiveButtonLeftRadius[1], 0xA000);
  tft.drawRoundRect(LiveButtonLeftXpos[1], LiveButtonLeftYpos[1], LiveButtonLeftWidth[1], LiveButtonLeftHeight[1], LiveButtonLeftRadius[1], 0x9492);
  tft.drawString(LiveButtonLeftChar[1], LiveButtonLeftXcenter[1], LiveButtonLeftYcenter[1], 2);

  tft.setTextFont(2);
  tft.setTextSize(2);
  tft.setTextColor(0xFFFE);
  tft.fillRoundRect(LiveButtonLeftXpos[2], LiveButtonLeftYpos[2], LiveButtonLeftWidth[2], LiveButtonLeftHeight[2], LiveButtonLeftRadius[2], 0xA000);
  tft.drawRoundRect(LiveButtonLeftXpos[2], LiveButtonLeftYpos[2], LiveButtonLeftWidth[2], LiveButtonLeftHeight[2], LiveButtonLeftRadius[2], 0x9492);
  tft.drawString(LiveButtonLeftChar[2], LiveButtonLeftXcenter[2], LiveButtonLeftYcenter[2], 1);

  tft.setFreeFont(&FreeSansBold9pt7b);
  tft.setTextSize(1);
  tft.setTextColor(0xFFFE);
  for (i = 3; i < 9; i++)
  {
    tft.fillRoundRect(LiveButtonLeftXpos[i], LiveButtonLeftYpos[i], LiveButtonLeftWidth[i], LiveButtonLeftHeight[i], LiveButtonLeftRadius[i], 0x0300);
    tft.drawRoundRect(LiveButtonLeftXpos[i], LiveButtonLeftYpos[i], LiveButtonLeftWidth[i], LiveButtonLeftHeight[i], LiveButtonLeftRadius[i], 0x9492);
    tft.drawString(LiveButtonLeftChar[i], LiveButtonLeftXcenter[i], LiveButtonLeftYcenter[i]);
    LiveButtonLeftXend[i] = LiveButtonLeftXpos[i] + LiveButtonLeftWidth[i];
    LiveButtonLeftYend[i] = LiveButtonLeftYpos[i] + LiveButtonLeftHeight[i];
  }

  // Live Screen - menu right items position
  tft.setTextFont(2);
  tft.setTextSize(2);
  tft.setTextColor(0xFFFE);
  tft.fillRoundRect(LiveButtonRightXpos[0], LiveButtonRightYpos[0], LiveButtonRightWidth[0], LiveButtonRightHeight[0], LiveButtonRightRadius[0], 0xA000);
  tft.drawRoundRect(LiveButtonRightXpos[0], LiveButtonRightYpos[0], LiveButtonRightWidth[0], LiveButtonRightHeight[0], LiveButtonRightRadius[0], 0x9492);
  tft.drawString(LiveButtonRightChar[0], LiveButtonRightXcenter[0], LiveButtonRightYcenter[0], 1);

  tft.setTextFont(2);
  tft.setTextSize(2);
  tft.setTextColor(0xFFFE);
  tft.fillRoundRect(LiveButtonRightXpos[1], LiveButtonRightYpos[1], LiveButtonRightWidth[1], LiveButtonRightHeight[1], LiveButtonRightRadius[1], 0xA000);
  tft.drawRoundRect(LiveButtonRightXpos[1], LiveButtonRightYpos[1], LiveButtonRightWidth[1], LiveButtonRightHeight[1], LiveButtonRightRadius[1], 0x9492);
  tft.drawString(LiveButtonRightChar[1], LiveButtonRightXcenter[1], LiveButtonRightYcenter[1], 2);

  tft.setTextFont(2);
  tft.setTextSize(2);
  tft.setTextColor(0xFFFE);
  tft.fillRoundRect(LiveButtonRightXpos[2], LiveButtonRightYpos[2], LiveButtonRightWidth[2], LiveButtonRightHeight[2], LiveButtonRightRadius[2], 0xA000);
  tft.drawRoundRect(LiveButtonRightXpos[2], LiveButtonRightYpos[2], LiveButtonRightWidth[2], LiveButtonRightHeight[2], LiveButtonRightRadius[2], 0x9492);
  tft.drawString(LiveButtonRightChar[2], LiveButtonRightXcenter[2], LiveButtonRightYcenter[2], 1);

  tft.setFreeFont(&FreeSansBold9pt7b);
  tft.setTextSize(1);
  tft.setTextColor(0xFFFE);
  for (i = 3; i < 9; i++)
  {
    tft.fillRoundRect(LiveButtonRightXpos[i], LiveButtonRightYpos[i], LiveButtonRightWidth[i], LiveButtonRightHeight[i], LiveButtonRightRadius[i], 0x0014);
    tft.drawRoundRect(LiveButtonRightXpos[i], LiveButtonRightYpos[i], LiveButtonRightWidth[i], LiveButtonRightHeight[i], LiveButtonRightRadius[i], 0x9492);
    tft.drawString(LiveButtonRightChar[i], LiveButtonRightXcenter[i], LiveButtonRightYcenter[i]);
    LiveButtonRightXend[i] = LiveButtonRightXpos[i] + LiveButtonRightWidth[i];
    LiveButtonRightYend[i] = LiveButtonRightYpos[i] + LiveButtonRightHeight[i];
  }

  // Live Screen - menu middle items position
  tft.setFreeFont(&FreeSansBold12pt7b);
  tft.setTextSize(1);
  tft.setTextColor(0x0000);
  for (i = 0; i < 6; i++)
  {
    tft.fillRoundRect(LiveButtonBothXpos[i], LiveButtonBothYpos[i], LiveButtonBothWidth[i], LiveButtonBothHeight[i], LiveButtonBothRadius[i], 0xFC00);
    tft.drawRoundRect(LiveButtonBothXpos[i], LiveButtonBothYpos[i], LiveButtonBothWidth[i], LiveButtonBothHeight[i], LiveButtonBothRadius[i], 0x9492);
    tft.drawString("Now", LiveButtonBothXpos[i] + LiveButtonBothWidth[i] / 2 - 1, LiveButtonBothYpos[i] + LiveButtonBothHeight[i] / 2 - 1);
    LiveButtonBothXend[i] = LiveButtonBothXpos[i] + LiveButtonBothWidth[i];
    LiveButtonBothYend[i] = LiveButtonBothYpos[i] + LiveButtonBothHeight[i];
  }

  // Live Screen - PWM left and right second row
  tft.setTextFont(2);
  tft.setTextSize(2);
  tft.setTextColor(0xFFFE);
  tft.drawString("OFF", LiveButtonLeftXcenter[0], 90, 1);
  tft.drawString("OFF", LiveButtonRightXcenter[0], 90, 1);

  // Live Screen - button reboot
  tft.setFreeFont(&FreeMonoBold18pt7b);
  tft.setTextSize(1);
  tft.setTextColor(0x528A);
  tft.fillRoundRect(LiveButtonRebootXpos, LiveButtonRebootYpos, LiveButtonRebootWidth, LiveButtonRebootHeight, LiveButtonRebootRadius, 0xA000);
  tft.drawRoundRect(LiveButtonRebootXpos, LiveButtonRebootYpos, LiveButtonRebootWidth, LiveButtonRebootHeight, LiveButtonRebootRadius, 0x9492),
  tft.drawString("x", LiveButtonRebootXcenter, LiveButtonRebootYcenter - 5);

  // Live Screen - espNOW status
  memcpy(&StatusResultXpos, &LiveStatusResultXpos, sizeof(LiveStatusResultXpos));
  memcpy(&StatusResultYpos, &LiveStatusResultYpos, sizeof(LiveStatusResultYpos));
  tft.setTextFont(2);
  tft.setTextSize(1);
  tft.setTextColor(0xFFFE, 0x0000);
  for (i = 0; i < 4; i++)
  {tft.drawString(StatusTitleChar[i], LiveStatusTitleXpos[i], LiveStatusTitleYpos[i], 2);}

  // Live Screen - button OFF, all motors stop
  tft.setFreeFont(&FreeMonoBold18pt7b);
  tft.setTextSize(1);
  tft.setTextColor(0x0000);
  tft.fillRoundRect(LiveButtonOffXpos, LiveButtonOffYpos, LiveButtonOffWidth, LiveButtonOffHeight, LiveButtonOffRadius, 0xFC00);
  tft.drawRoundRect(LiveButtonOffXpos, LiveButtonOffYpos, LiveButtonOffWidth, LiveButtonOffHeight, LiveButtonOffRadius, 0x9492);
  tft.drawString("OFF", LiveButtonOffXcenter, LiveButtonOffYcenter - 2);

  // Live Screen - restore basic text appearance
  tft.setTextFont(2);
  tft.setTextSize(1);
  tft.setTextColor(0xFFFE);

  // Live Screen - switch backlight on
  digitalWrite(TFT_BL, 1);
}


// **************************************************************************************
void AutoSetup()
{
  // Auto Screen - switch backlight off
  digitalWrite(TFT_BL, 0);

  // Auto - Menu, calculate touch areas
  for (x = 0; x < 17; x++)
  {
    AutoMenuTouchXpos[x] = AutoMenuXpos + AutoMenuButtonXpos[x];
    AutoMenuTouchYpos[x] = AutoMenuYpos + AutoMenuButtonYpos[x];
    AutoMenuTouchXend[x] = AutoMenuXpos + AutoMenuButtonXpos[x] + AutoMenuButtonWidth[x];
    AutoMenuTouchYend[x] = AutoMenuYpos + AutoMenuButtonYpos[x] + AutoMenuButtonHeight[x];
  }
  
  // Auto - Menu, calculate menu row x and y center points
  for (x = 0; x < 17; x++)
  {
    AutoMenuValueXcenter[x] = AutoMenuXpos + AutoMenuWidth / 2;
    AutoMenuValueYcenter[x] = AutoMenuYpos + AutoMenuButtonYpos[x] + 16;
  }

  // Auto - reboot button
  AutoButtonRebootXend = AutoButtonRebootXpos + AutoButtonRebootWidth;
  AutoButtonRebootYend = AutoButtonRebootYpos + AutoButtonRebootHeight;
  AutoButtonRebootXcenter = AutoButtonRebootXpos + AutoButtonRebootWidth / 2;
  AutoButtonRebootYcenter = AutoButtonRebootYpos + AutoButtonRebootHeight / 2;

  // Auto Screen - basics
  tft.setRotation(0);
  tft.fillScreen(0x0000);
  tft.setTextDatum(MC_DATUM);

  // Auto - draw title if menu 
  AutoMenuTitle();

  // Auto Screen - horizontal top line
  tft.drawLine(0, 26, width / 16 * 10, 26, 0x8410);

  // Auto Screen - vertical right menuline
  tft.drawLine(width / 16 * 10, 27, width / 16 * 10, height - 45, 0x8410);

  AutoBotsAreaHelpLines();

  // Auto Screen - horizontal bottom line
  tft.drawLine(0, height - 45, width / 16 * 10, height - 45, 0x8410);

  // Auto Screen - vertical left bottom line
  tft.drawLine(48, height - 45, 48, height, 0x8410);

  // Auto Screen - button reboot
  tft.setFreeFont(&FreeMonoBold18pt7b);
  tft.setTextSize(1);
  tft.fillRoundRect(AutoButtonRebootXpos, AutoButtonRebootYpos, AutoButtonRebootWidth, AutoButtonRebootHeight, AutoButtonRebootRadius, 0xA000);
  tft.drawRoundRect(AutoButtonRebootXpos, AutoButtonRebootYpos, AutoButtonRebootWidth, AutoButtonRebootHeight, AutoButtonRebootRadius, 0x9492),
  tft.setTextColor(0x528A);
  tft.drawString("x", AutoButtonRebootXcenter, AutoButtonRebootYcenter - 5);

  // Auto Screen - espNOW status
  memcpy(&StatusResultXpos, &AutoStatusResultXpos, sizeof(AutoStatusResultXpos));
  memcpy(&StatusResultYpos, &AutoStatusResultYpos, sizeof(AutoStatusResultYpos));
  tft.setTextFont(2);
  tft.setTextSize(1);
  tft.setTextColor(0xFFFE, 0x0000);
  for (i = 0; i < 4; i++)
  {tft.drawString(StatusTitleChar[i], AutoStatusTitleXpos[i], AutoStatusTitleYpos[i], 2);}

  // Auto Screen - restore basic text appearance
  tft.setTextFont(2);
  tft.setTextSize(1);
  tft.setTextColor(0xFFFE);

  // Auto Screen - create and push sprites
  AutoOptionCreate();
  AutoCreate();
  TimelineCreate();
  Auto.pushSprite(AutoMenuXpos, AutoMenuYpos, 0x0000);
  Timeline.pushSprite(width / 16 * 5 - 20, 27);

  // Auto Screen - switch backlight on
  digitalWrite(TFT_BL, 1);
}

void AutoMenuTitle()
{
  // Auto - title text if menu
  tft.fillRect(0, 0, AutoMenuXpos, 26, 0x0000);
  tft.setTextFont(2);
  tft.setTextSize(1);
  tft.setTextColor(0xFFFE, 0x0000);
  tft.drawString("Motor Left", width / 16 * 4 / 2 + 4, 14, 2);
  tft.drawString("Time", width / 16 * 5, 14, 2);
  tft.drawString("  Motor Right", width / 16 * 8 - 4, 14, 2);
}

void AutoOptionTitle()
{
  // Auto - title text if option
  tft.fillRect(0, 0, AutoMenuXpos, 26, 0x0000);
  tft.setTextFont(2);
  tft.setTextSize(1);
  tft.setTextColor(0xFFFE, 0x0000);
  tft.drawString("Files on SPIFFS", width / 16 * 5, 14, 2);
}

void AutoBotsAreaHelpLines()
{
// Auto Screen - visualize the time and value multiplicator areas, horizontal help lines
tft.drawLine(AutoTimeMultiTouchXpos[1], AutoTimeMultiTouchYpos[1], AutoTimeMultiTouchXpos[1] + 8, AutoTimeMultiTouchYpos[1], 0x8410); // time multiplicator
tft.drawLine(AutoTimeMultiTouchXpos[2], AutoTimeMultiTouchYpos[2], AutoTimeMultiTouchXpos[2] + 12, AutoTimeMultiTouchYpos[2], 0x8410);
tft.drawLine(AutoTimeMultiTouchXpos[3], AutoTimeMultiTouchYpos[3], AutoTimeMultiTouchXpos[3] + 8, AutoTimeMultiTouchYpos[3], 0x8410);
tft.drawLine(AutoValueMultiTouchXend[1], AutoValueMultiTouchYpos[1], AutoValueMultiTouchXend[1] - 8, AutoValueMultiTouchYpos[1], 0x8410); // value multiplicator
tft.drawLine(AutoValueMultiTouchXend[2], AutoValueMultiTouchYpos[2], AutoValueMultiTouchXend[2] - 12, AutoValueMultiTouchYpos[2], 0x8410);
tft.drawLine(AutoValueMultiTouchXend[3], AutoValueMultiTouchYpos[3], AutoValueMultiTouchXend[3] - 8, AutoValueMultiTouchYpos[3], 0x8410);
}


// **************************************************************************************
void FunSetup()
{
  // Fun Screen - switch backlight off
  digitalWrite(TFT_BL, 0);
  
  // Fun Screen - set rotation to widescreen
  tft.setRotation(3);
  width = tft.width();
  height = tft.height();
  
  // Fun Screen - basics
  tft.fillScreen(0x0000);
  tft.setTextFont(2);
  tft.setTextSize(2);
  tft.setTextColor(0xFFFE);
  tft.setTextDatum(MC_DATUM);
  tft.setPivot(width / 2, height / 2);

  // Fun Screen - some calculations
  for (i = 0; i < 4; i++)
  {
    FunJoypadXend[i] = FunJoypadXpos[i] + FunJoypadWidth[i];
    FunJoypadYend[i] = FunJoypadYpos[i] + FunJoypadHeight[i];
    FunJoypadXcenter[i] = FunJoypadXpos[i] + FunJoypadWidth[i] / 2;
    FunJoypadYcenter[i] = FunJoypadYpos[i] + FunJoypadHeight[i] / 2;
  }
  
  FunButtonRebootXend = FunButtonRebootXpos + FunButtonRebootWidth;
  FunButtonRebootYend = FunButtonRebootYpos + FunButtonRebootHeight;
  FunButtonRebootXcenter = FunButtonRebootXpos + FunButtonRebootWidth / 2;
  FunButtonRebootYcenter = FunButtonRebootYpos + FunButtonRebootHeight / 2;
  
  FunButtonRecordXend = FunButtonRecordXpos + FunButtonRecordWidth;
  FunButtonRecordYend = FunButtonRecordYpos + FunButtonRecordHeight;
  FunButtonRecordXcenter = FunButtonRecordXpos + FunButtonRecordWidth / 2;
  FunButtonRecordYcenter = FunButtonRecordYpos + FunButtonRecordHeight / 2;

  FunButtonStopModeXend = FunButtonStopModeXpos + FunButtonStopModeWidth;
  FunButtonStopModeYend = FunButtonStopModeYpos + FunButtonStopModeHeight;
  FunButtonStopModeXcenter = FunButtonStopModeXpos + FunButtonStopModeWidth / 2;
  FunButtonStopModeYcenter = FunButtonStopModeYpos + FunButtonStopModeHeight / 2;
  
  // Fun Screen - horizontal center line
  tft.drawLine(width / 2 - 8, height / 2, width / 2 + 8, height / 2, 0x8410);

  // Fun Screen - vertical center line
  tft.drawLine(width / 2, height / 2 - 8, width / 2, height / 2 + 8, 0x8410);

  // Fun Screen - horizontal bottom line
  tft.drawLine(0, height - 45, width, height - 45, 0x8410);

  // Fun Screen - vertical left bottom lines
  tft.drawLine(48, height - 45, 48, height, 0x8410);
  tft.drawLine(width - 48, height - 45, width - 48, height, 0x8410);

  // Fun Screen - title
  tft.setFreeFont(&FreeSerifBold12pt7b);
  tft.setTextSize(1);
  tft.setTextColor(0xFC00, 0x0000);
  tft.drawString("Direction", FunTitleLeftXpos, FunTitleLeftYpos, 1);
  tft.setTextColor(0x6018, 0x0000);
  tft.drawString(" Brake ", FunTitleMiddleXpos, FunTitleMiddleYpos, 1);
  tft.setTextColor(0xFC00, 0x0000);
  tft.drawString("Steering", FunTitleRightXpos, FunTitleRightYpos, 1);

  // Fun Screen - joypad
  for (i = 0; i < 4; i++)
  {
    tft.fillRoundRect(FunJoypadXpos[i], FunJoypadYpos[i], FunJoypadWidth[i], FunJoypadHeight[i], FunJoypadRadius[i], 0x18C3);
    tft.drawRoundRect(FunJoypadXpos[i], FunJoypadYpos[i], FunJoypadWidth[i], FunJoypadHeight[i], FunJoypadRadius[i], 0x2945);
  }

  // Fun Screen - draw direction and steering triangles
  tft.fillTriangle(FunJoypadXcenter[0], FunJoypadYcenter[0] - FunJoypadHeight[0] / 4, FunJoypadXcenter[0] - FunJoypadWidth[0] / 4, FunJoypadYcenter[0] + FunJoypadHeight[0] / 4, FunJoypadXcenter[0] + FunJoypadWidth[0] / 4, FunJoypadYcenter[0] + FunJoypadHeight[0] / 4, 0x0300);
  tft.fillTriangle(FunJoypadXcenter[1], FunJoypadYcenter[1] + FunJoypadHeight[1] / 4, FunJoypadXcenter[1] - FunJoypadWidth[1] / 4, FunJoypadYcenter[1] - FunJoypadHeight[1] / 4, FunJoypadXcenter[1] + FunJoypadWidth[1] / 4, FunJoypadYcenter[1] - FunJoypadHeight[1] / 4, 0x0300);
  tft.fillTriangle(FunJoypadXcenter[2] + FunJoypadWidth[2] / 6, FunJoypadYcenter[2], FunJoypadXcenter[2] - FunJoypadWidth[2] / 6, FunJoypadYcenter[2] - FunJoypadHeight[2] / 3, FunJoypadXcenter[2] - FunJoypadWidth[2] / 6, FunJoypadYcenter[2] + FunJoypadHeight[2] / 3, 0x0300);
  tft.fillTriangle(FunJoypadXcenter[3] - FunJoypadWidth[3] / 6, FunJoypadYcenter[3], FunJoypadXcenter[3] + FunJoypadWidth[3] / 6, FunJoypadYcenter[3] - FunJoypadHeight[3] / 3, FunJoypadXcenter[3] + FunJoypadWidth[3] / 6, FunJoypadYcenter[3] + FunJoypadHeight[3] / 3, 0x0300);

  // Fun Screen - button reboot
  tft.setFreeFont(&FreeMonoBold18pt7b);
  tft.setTextSize(1);
  tft.setTextColor(0x528A);
  tft.fillRoundRect(FunButtonRebootXpos, FunButtonRebootYpos, FunButtonRebootWidth, FunButtonRebootHeight, FunButtonRebootRadius, 0xA000);
  tft.drawRoundRect(FunButtonRebootXpos, FunButtonRebootYpos, FunButtonRebootWidth, FunButtonRebootHeight, FunButtonRebootRadius, 0x9492),
  tft.drawString("x", FunButtonRebootXcenter, FunButtonRebootYcenter - 5);

  // Fun Screen - espNOW status
  memcpy(&StatusResultXpos, &FunStatusResultXpos, sizeof(FunStatusResultXpos));
  memcpy(&StatusResultYpos, &FunStatusResultYpos, sizeof(FunStatusResultYpos));
  tft.setTextFont(2);
  tft.setTextSize(1);
  tft.setTextColor(0xFFFE, 0x0000);
  for (i = 0; i < 4; i++)
  {tft.drawString(StatusTitleChar[i], FunStatusTitleXpos[i], FunStatusTitleYpos[i], 2);}

  // Fun Screen - button record
  tft.setFreeFont(&FreeSerifBold12pt7b);
  tft.setTextSize(1);
  tft.setTextColor(0x0000);
  tft.fillRoundRect(FunButtonRecordXpos, FunButtonRecordYpos, FunButtonRecordWidth, FunButtonRecordHeight, FunButtonRecordRadius, 0xFC00);
  tft.drawRoundRect(FunButtonRecordXpos, FunButtonRecordYpos, FunButtonRecordWidth, FunButtonRecordHeight, FunButtonRecordRadius, 0x8410);
  tft.drawString("Record", FunButtonRecordXcenter, FunButtonRecordYcenter - 2);

  // Fun Screen - button stop mode
  tft.setFreeFont(&FreeMonoBold18pt7b);
  tft.setTextSize(1);
  tft.setTextColor(0x528A);
  tft.fillRoundRect(FunButtonStopModeXpos, FunButtonStopModeYpos, FunButtonStopModeWidth, FunButtonStopModeHeight, FunButtonStopModeRadius, 0x2008);
  tft.drawRoundRect(FunButtonStopModeXpos, FunButtonStopModeYpos, FunButtonStopModeWidth, FunButtonStopModeHeight, FunButtonStopModeRadius, 0x8410),
  tft.drawString("o", FunButtonStopModeXcenter, FunButtonStopModeYcenter - 5);

  // Fun Screen - create and display needle
  NeedleCreate();

  // Fun Screen - restore basic text appearance
  tft.setTextFont(2);
  tft.setTextSize(1);
  tft.setTextColor(0xFFFE);
  
  // Fun Screen - switch backlight on
  digitalWrite(TFT_BL, 1);
}


// **************************************************************************************
void ESPNowSetup()
{
  // ESP-NOW - set device as a Wi-Fi Station
  WiFi.mode(WIFI_STA);

  // ESP-NOW - init
  if (esp_now_init() != ESP_OK)
  {
    tft.fillCircle(StatusResultXpos[0], StatusResultYpos[0], StatusResultRadius[0], 0xF800);
    tft.drawCircle(StatusResultXpos[0], StatusResultYpos[0], StatusResultRadius[0], 0x9492);
  }
  else
  {
    tft.fillCircle(StatusResultXpos[0], StatusResultYpos[0], StatusResultRadius[0], 0x0700);
    tft.drawCircle(StatusResultXpos[0], StatusResultYpos[0], StatusResultRadius[0], 0x9492);
  }

  // ESP-NOW - register for send CB to get the status of trasnmitted packet
  esp_now_register_send_cb(ESPNowOnSend);

  // ESP-NOW - register peer MAC address
  memcpy(peerInfo.peer_addr, broadcastAddress, 6);
  peerInfo.channel = 0;
  peerInfo.encrypt = false;

  // ESP-NOW - add peer
  if (esp_now_add_peer(&peerInfo) != ESP_OK)
  {
    tft.fillCircle(StatusResultXpos[1], StatusResultYpos[1], StatusResultRadius[1], 0xF800);
    tft.drawCircle(StatusResultXpos[1], StatusResultYpos[1], StatusResultRadius[1], 0x9492);
  }
  else
  {
    tft.fillCircle(StatusResultXpos[1], StatusResultYpos[1], StatusResultRadius[1], 0x0700);
    tft.drawCircle(StatusResultXpos[1], StatusResultYpos[1], StatusResultRadius[1], 0x9492);
  }
}


// **************************************************************************************
void ESPNowSend(void)
{
  // ESP-NOW - reset send status
  //tft.fillCircle(StatusResultXpos[2], StatusResultYpos[2], StatusResultRadius[2], 0x0000);
  tft.fillRect(StatusResultXpos[2] - 12, StatusResultYpos[2] - 8, 24, 16, 0x0000);
  tft.fillRect(StatusResultXpos[3] - 12, StatusResultYpos[3] - 8, 24, 16, 0x0000);

  // ESP-NOW - live mode
  if (menu == 3)
  {
    MotorData[0].LeftMode = 3;
    MotorData[0].RightMode = 3;
    MotorData[0].LeftSteps = 0;
    MotorData[0].RightSteps = 0;
    espNowMsgDataArrayToSend = 0;
  }
  
  // ESP-NOW - auto mode
  if (menu == 4)
  {
    MotorData[0].LeftMode = 4;
    MotorData[0].RightMode = 4;
    MotorData[0].LeftSteps = 0;
    MotorData[0].RightSteps = 0;
    espNowMsgDataArrayToSend = 0;
    
    // ESP-NOW - max. left steps to send, last step where time is not 0
    for (i = espNowMsgDataArrayMax; i > 0; i--)
    {
      if (MotorData[i].LeftTime != 0)
      {
        MotorData[0].LeftSteps = i;
        break;
      }
    }
    
    // ESP-NOW - max. right steps to send, last step where time is not 0
    for (i = espNowMsgDataArrayMax; i > 0; i--)
    {
      if (MotorData[i].RightTime != 0)
      {
        MotorData[0].RightSteps = i;
        break;
      }
    }

    // ESP-NOW - max. steps to send
    if (MotorData[0].LeftSteps > MotorData[0].RightSteps)
    {espNowMsgDataArrayToSend = MotorData[0].LeftSteps;}
    else
    {espNowMsgDataArrayToSend = MotorData[0].RightSteps;}
    MotorData[0].LeftSteps = espNowMsgDataArrayToSend;
    MotorData[0].RightSteps = espNowMsgDataArrayToSend;
  }

  // ESP-NOW - fun mode
  if (menu == 5)
  {
    MotorData[0].LeftMode = 5;
    MotorData[0].RightMode = 5;
    MotorData[0].LeftSteps = 0;
    MotorData[0].RightSteps = 0;
    MotorData[0].LeftCTR = 1;
    MotorData[0].RightCTR = 1;
    espNowMsgDataArrayToSend = 0;
  }

  // ESP Now - set message flow variables
  espNowMsgDataArrayIndex = 0;
  espNowMessageError = 0;
  espNowStatusError  = 0;

  // ESP Now - send messages
  tft.setTextSize(1);
  while (espNowMsgDataArrayIndex <= espNowMsgDataArrayToSend)
  {
    esp_err_t result = esp_now_send(broadcastAddress, (uint8_t *) &MotorData[espNowMsgDataArrayIndex], sizeof(MotorData[espNowMsgDataArrayIndex]));
    if (result != ESP_OK)
    {
      tft.setTextColor(0xF800, 0x0000);
      tft.drawNumber(espNowMsgDataArrayIndex, StatusResultXpos[2], StatusResultYpos[2], 2);
      espNowMessageError++;
    }
    else
    {
      tft.setTextColor(0xFFFE, 0x0000);
      tft.drawNumber(espNowMsgDataArrayIndex, StatusResultXpos[2], StatusResultYpos[2], 2);
    }
    // ESP Now - give the receiver more time to proceed the first message, because the reveiver must init some variables, init the screen...
    if (espNowMsgDataArrayIndex == 0)
    {delay (30);}
    // ESP Now - standard delay
    delay (espNowMsgDelay);
    espNowMsgDataArrayIndex++;
  }

  // ESP Now - display error / success
  espNowMsgDataArrayIndex--;
  if (espNowMessageError != 0)
  {
    tft.setTextColor(0xF800, 0x0000);
    tft.drawNumber(espNowMsgDataArrayIndex, StatusResultXpos[2], StatusResultYpos[2], 2);
  }
  else
  {
    tft.setTextColor(0x0700, 0x0000);
    tft.drawNumber(espNowMsgDataArrayIndex, StatusResultXpos[2], StatusResultYpos[2], 2);
  }
}


// **************************************************************************************
void AutoCreate(void)
{
  // Auto - menu, create
  Auto.setColorDepth(8);
  Auto.createSprite(116, height);
  Auto.fillSprite(0x0000);

  // Auto - menu, text appearance
  Auto.setTextFont(2);
  Auto.setTextSize(2);
  Auto.setTextColor(0xFFFE);
  Auto.setTextDatum(MC_DATUM);

  // Auto - menu, draw buttons and text;
  for (x = 0; x < 17; x++)
  {
    if (x == 0)
    {
      Auto.setFreeFont(&FreeMonoBold18pt7b);
      Auto.setTextSize(1);
      Auto.setTextColor(0xC618);
      Auto.fillRoundRect(AutoMenuButtonXpos[x], AutoMenuButtonYpos[x], AutoMenuButtonWidth[x], AutoMenuButtonHeight[x], AutoMenuButtonRadius[x], 0xA000);
      Auto.drawRoundRect(AutoMenuButtonXpos[x], AutoMenuButtonYpos[x], AutoMenuButtonWidth[x], AutoMenuButtonHeight[x], AutoMenuButtonRadius[x], 0x9492),
      Auto.drawString(AutoMenuButtonChar[x], AutoMenuButtonXpos[x] + AutoMenuButtonWidth[x] / 2, AutoMenuButtonYpos[x] + AutoMenuButtonHeight[x] / 2 - 1);
    }
    
    if (x == 1)
    {
      AutoColor = 0x0300;
      Auto.setTextFont(2);
      Auto.setTextSize(2);
      Auto.fillRoundRect(AutoMenuButtonXpos[x], AutoMenuButtonYpos[x], AutoMenuButtonWidth[x], AutoMenuButtonHeight[x], AutoMenuButtonRadius[x], AutoColor);
      Auto.drawRoundRect(AutoMenuButtonXpos[x], AutoMenuButtonYpos[x], AutoMenuButtonWidth[x], AutoMenuButtonHeight[x], AutoMenuButtonRadius[x], 0x9492);
      Auto.drawString(AutoMenuButtonChar[x], AutoMenuButtonXpos[x] + AutoMenuButtonWidth[x] / 2, AutoMenuButtonYpos[x] + AutoMenuButtonHeight[x] / 2, 2);
    }
    
    if (x == 2)
    {
      AutoColor = 0x0014;
      Auto.setTextFont(2);
      Auto.setTextSize(2);
      Auto.fillRoundRect(AutoMenuButtonXpos[x], AutoMenuButtonYpos[x], AutoMenuButtonWidth[x], AutoMenuButtonHeight[x], AutoMenuButtonRadius[x], AutoColor);
      Auto.drawRoundRect(AutoMenuButtonXpos[x], AutoMenuButtonYpos[x], AutoMenuButtonWidth[x], AutoMenuButtonHeight[x], AutoMenuButtonRadius[x], 0x9492);
      Auto.drawString(AutoMenuButtonChar[x], AutoMenuButtonXpos[x] + AutoMenuButtonWidth[x] / 2, AutoMenuButtonYpos[x] + AutoMenuButtonHeight[x] / 2, 2);
    }

    if (AutoMotor == 0 && x > 2)
    {AutoColor = 0x0300;}
    if (AutoMotor == 1 && x > 2)
    {AutoColor = 0x0014;}

    if (x > 2 && x <= 12)
    {
      if (x & 1) // x = odd
      {
        Auto.fillRoundRect(AutoMenuButtonXpos[x], AutoMenuButtonYpos[x], AutoMenuWidth - 4, AutoMenuButtonHeight[x], AutoMenuButtonRadius[x], AutoColor);
        Auto.setTextFont(2);
        Auto.setTextSize(2);
        Auto.drawString(AutoMenuButtonChar[x], AutoMenuWidth / 2 - 44, AutoMenuValueYcenter[x]);
      }
      else  // x = even
      {
        Auto.setTextFont(2);
        Auto.setTextSize(2);
        Auto.drawString(AutoMenuButtonChar[x], AutoMenuWidth / 2 + 46, AutoMenuValueYcenter[x]);
        Auto.drawRoundRect(AutoMenuButtonXpos[x - 1], AutoMenuButtonYpos[x], AutoMenuWidth - 4, AutoMenuButtonHeight[x], AutoMenuButtonRadius[x], 0x9492);
      }
    }

    if (x == 13)
    {
      Auto.setTextFont(2);
      Auto.setTextSize(1);
      Auto.setTextColor(0xFFFE);
      Auto.fillRoundRect(AutoMenuButtonXpos[x], AutoMenuButtonYpos[x], AutoMenuButtonWidth[x], AutoMenuButtonHeight[x], AutoMenuButtonRadius[x], 0x500A);
      Auto.drawRoundRect(AutoMenuButtonXpos[x], AutoMenuButtonYpos[x], AutoMenuButtonWidth[x], AutoMenuButtonHeight[x], AutoMenuButtonRadius[x], 0x9492);
      Auto.drawString(AutoMenuButtonChar[x], AutoMenuButtonXpos[x] + AutoMenuButtonWidth[x] / 2, AutoMenuButtonYpos[x] + AutoMenuButtonHeight[x] / 2, 2);      
    }

    if (x == 14)
    {
      Auto.setTextFont(2);
      Auto.setTextSize(1);
      Auto.setTextColor(0xFFFE);
      Auto.fillRoundRect(AutoMenuButtonXpos[x], AutoMenuButtonYpos[x], AutoMenuButtonWidth[x], AutoMenuButtonHeight[x], AutoMenuButtonRadius[x], 0xA000);
      Auto.drawRoundRect(AutoMenuButtonXpos[x], AutoMenuButtonYpos[x], AutoMenuButtonWidth[x], AutoMenuButtonHeight[x], AutoMenuButtonRadius[x], 0x9492);
      Auto.drawString(AutoMenuButtonChar[x], AutoMenuButtonXpos[x] + AutoMenuButtonWidth[x] / 2, AutoMenuButtonYpos[x] + AutoMenuButtonHeight[x] / 2, 2);
    }

    if (x == 15)
    {
      Auto.setFreeFont(&FreeSans9pt7b);
      Auto.setTextSize(1);
      Auto.setTextColor(0xC618);
      Auto.fillRoundRect(AutoMenuButtonXpos[x], AutoMenuButtonYpos[x], AutoMenuButtonWidth[x], AutoMenuButtonHeight[x], AutoMenuButtonRadius[x], 0x0186);
      Auto.drawRoundRect(AutoMenuButtonXpos[x], AutoMenuButtonYpos[x], AutoMenuButtonWidth[x], AutoMenuButtonHeight[x], AutoMenuButtonRadius[x], 0x9492);
      Auto.drawString(AutoMenuButtonChar[x], AutoMenuButtonXpos[x] + AutoMenuButtonWidth[x] / 2, AutoMenuButtonYpos[x] + AutoMenuButtonHeight[x] / 2 - 2);
    }

    if (x == 16)
    {
      Auto.setFreeFont(&FreeMonoBold18pt7b);
      Auto.setTextSize(1);
      Auto.setTextColor(0x0000);
      Auto.fillRoundRect(AutoMenuButtonXpos[x], AutoMenuButtonYpos[x], AutoMenuButtonWidth[x], AutoMenuButtonHeight[x], AutoMenuButtonRadius[x], 0xFC00);
      Auto.drawRoundRect(AutoMenuButtonXpos[x], AutoMenuButtonYpos[x], AutoMenuButtonWidth[x], AutoMenuButtonHeight[x], AutoMenuButtonRadius[x], 0x9492);
      Auto.drawString(AutoMenuButtonChar[x], AutoMenuButtonXpos[x] + AutoMenuButtonWidth[x] / 2, AutoMenuButtonYpos[x] + AutoMenuButtonHeight[x] / 2 - 3);
    }
  }

  // Auto - menu, button title text
  Auto.setTextFont(2);
  Auto.setTextSize(1);
  Auto.setTextDatum(MC_DATUM);
  Auto.setTextColor(0xFFFE);
  for (x = 1; x <= 11; x = x + 2)
  {
    AutoMenuTitleXcenter[x] = AutoMenuWidth / 2;
    AutoMenuTitleYcenter[x] = AutoMenuButtonYpos[x] - 8;
    Auto.drawString(AutoMenuTitleChar[x], AutoMenuTitleXcenter[x], AutoMenuTitleYcenter[x], 2);
  }
}


// **************************************************************************************
void TimelineCreate(void)
{
  // Timeline - create
  Timeline.setColorDepth(8);
  Timeline.createSprite(40, 407);
  Timeline.fillSprite(0x0000);

  // Timeline - vertical timeline
  Timeline.drawLine(20, 0, 20, 400, 0xFFFE);

  // Timeline - horizontal timeline short lines
  for (y = 0; y <= 400; y = y + 4)
  {Timeline.drawLine(16, y, 24, y, 0x8410);}

  // Timeline - horizontal timeline long lines and second numbers
  Timeline.setTextSize(1);
  Timeline.setTextDatum(MC_DATUM);
  Timeline.setTextColor(0x07FF, 0x0000);
  i = AutoTimelineBase + AutoTimelineAdd;
  for (y = 40; y <= 400; y = y + 40)
  {
   Timeline.drawLine(14, y, 26, y, 0xC618);
   Timeline.drawNumber(i, 20, y, 2);
   i = i + AutoTimelineAdd;
  }
}


// **************************************************************************************
void AutoOptionCreate(void)
{
  // AutoOption - create
  AutoOption.setColorDepth(8);
  AutoOption.createSprite(116, height);
  AutoOption.fillSprite(0x0000);

  // AutoOption - basic text appearance
  AutoOption.setTextDatum(MC_DATUM);

  // AutoOption - menu button
  AutoOption.fillRoundRect(2, 2, 112, 32, 6, 0xA000);
  AutoOption.drawRoundRect(2, 2, 112, 32, 6, 0x9492);
  AutoOption.setFreeFont(&FreeMonoBold18pt7b);
  AutoOption.setTextSize(1);
  AutoOption.setTextColor(0xC618);
  AutoOption.drawString(AutoMenuButtonChar[0], 58, 16);
  
  AutoOption.drawLine(2, 46, 112, 46, 0x8410);

  // AutoOption - button format filesystem
  AutoOption.setTextFont(2);
  AutoOption.setTextSize(1);
  AutoOption.setTextColor(0xFFFE);
  AutoOption.drawString("Format SPIFFS", 58, 62, 2);
  AutoOption.fillRoundRect(2, 76, 112, 32, 6, 0x500A);
  AutoOption.drawRoundRect(2, 76, 112, 32, 6, 0x9492);
  AutoOption.setFreeFont(&FreeSans9pt7b);
  AutoOption.setTextColor(0xC618);
  AutoOption.drawString("Format", 58, 90);

  AutoOption.drawLine(2, 120, 112, 120, 0x8410);

  // AutoOption - load file slot
  AutoOption.setTextFont(2);
  AutoOption.setTextSize(1);
  AutoOption.setTextColor(0xFFFE);
  AutoOption.drawString("Load File Slot", 58, 136, 2);
  AutoOption.drawRoundRect(2, 150, 112, 32, 6, 0x9492);
  AutoOption.setTextSize(2);
  AutoOption.drawString("-      +", 58, 166, 2);
  AutoOption.setFreeFont(&FreeSans9pt7b);
  AutoOption.setTextSize(1);
  AutoOption.drawNumber(AutoOptionFileLoadSlot, 58, 166);
  
  // AutoOption - button load file
  AutoOption.setFreeFont(&FreeSans9pt7b);
  AutoOption.setTextSize(1);
  AutoOption.setTextColor(0xC618);
  AutoOption.fillRoundRect(2, 194, 112, 32, 6, 0x500A);
  AutoOption.drawRoundRect(2, 194, 112, 32, 6, 0x9492);
  AutoOption.drawString("Load File", 58, 208);

  AutoOption.drawLine(2, 238, 112, 238, 0x8410);

  // AutoOption - save file slot
  AutoOption.setTextFont(2);
  AutoOption.setTextSize(1);
  AutoOption.setTextColor(0xFFFE);
  AutoOption.drawString("Save File Slot", 58, 254, 2);
  AutoOption.drawRoundRect(2, 268, 112, 32, 6, 0x9492);
  AutoOption.setTextSize(2);
  AutoOption.drawString("-      +", 58, 284, 2);
  AutoOption.setFreeFont(&FreeSans9pt7b);
  AutoOption.setTextSize(1);
  AutoOption.drawNumber(AutoOptionFileSaveSlot, 58, 284);

  // AutoOption - button save file
  AutoOption.setFreeFont(&FreeSans9pt7b);
  AutoOption.setTextSize(1);
  AutoOption.setTextColor(0xC618);
  AutoOption.fillRoundRect(2, 312, 112, 32, 6, 0x500A);
  AutoOption.drawRoundRect(2, 312, 112, 32, 6, 0x9492);
  AutoOption.drawString("Save File", 58, 326);

  AutoOption.drawLine(2, 356, 112, 356, 0x8410);

  // AutoOption - reset steps
  AutoOption.setTextFont(2);
  AutoOption.setTextSize(1);
  AutoOption.setTextColor(0xFFFE);
  AutoOption.drawString("Reset all Steps", 58, 372, 2);

  // AutoOption - button reset left steps
  AutoOption.setFreeFont(&FreeSans9pt7b);
  AutoOption.setTextSize(1);
  AutoOption.setTextColor(0xC618);
  AutoOption.fillRoundRect(2, 386, 112, 32, 6, 0x0320);
  AutoOption.drawRoundRect(2, 386, 112, 32, 6, 0x9492);
  AutoOption.drawString("Reset Left", 58, 400);

  // AutoOption - button reset right steps
  AutoOption.setFreeFont(&FreeSans9pt7b);
  AutoOption.setTextSize(1);
  AutoOption.setTextColor(0xC618);
  AutoOption.fillRoundRect(2, 432, 112, 32, 6, 0x0014);
  AutoOption.drawRoundRect(2, 432, 112, 32, 6, 0x9492);
  AutoOption.drawString("Reset Right", 58, 446);
}


// **************************************************************************************
void NeedleCreate()
{
  Needle.setColorDepth(8);
  Needle.createSprite(32, 32);
  Needle.fillSprite(0x0000);
  Needle.fillTriangle(16, 5, 5, 27, 27, 27, 0xF800);
  Needle.fillCircle(16, 30, 8, 0x0000);
}
